@@ -2,13 +2,7 @@
 Header file for compression routine
 
 Copyright (c) 2006 - 2018, Intel Corporation. All rights reserved.<BR>
-This program and the accompanying materials
-are licensed and made available under the terms and conditions of the BSD License
-which accompanies this distribution.  The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php
-
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
@@ -17,13 +11,6 @@ WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
 
 #include <Common/UefiBaseTypes.h>
 
-EFI_STATUS
-EfiGetInfo (
-  IN      VOID    *Source,
-  IN      UINT32  SrcSize,
-  OUT     UINT32  *DstSize,
-  OUT     UINT32  *ScratchSize
-  );
 /**
 
 Routine Description:
@@ -43,7 +30,27 @@ Returns:
   EFI_INVALID_PARAMETER - The source data is corrupted
 
 **/
+EFI_STATUS
+EfiGetInfo (
+  IN      VOID    *Source,
+  IN      UINT32  SrcSize,
+  OUT     UINT32  *DstSize,
+  OUT     UINT32  *ScratchSize
+  );
 
+/**
+  The implementation of Efi Decompress().
+
+  @param Source      The source buffer containing the compressed data.
+  @param SrcSize     The size of source buffer
+  @param Destination The destination buffer to store the decompressed data
+  @param DstSize     The size of destination buffer.
+  @param Scratch     The buffer used internally by the decompress routine. This  buffer is needed to store intermediate data.
+  @param ScratchSize The size of scratch buffer.
+
+  @retval EFI_SUCCESS           Decompression is successful
+  @retval EFI_INVALID_PARAMETER The source data is corrupted
+**/
 EFI_STATUS
 EfiDecompress (
   IN      VOID    *Source,
@@ -53,28 +60,18 @@ EfiDecompress (
   IN OUT  VOID    *Scratch,
   IN      UINT32  ScratchSize
   );
+
 /**
+  The implementation Tiano Decompress GetInfo().
 
-Routine Description:
-
-  The implementation of Efi Decompress().
-
-Arguments:
-
-  Source      - The source buffer containing the compressed data.
-  SrcSize     - The size of source buffer
-  Destination - The destination buffer to store the decompressed data
-  DstSize     - The size of destination buffer.
-  Scratch     - The buffer used internally by the decompress routine. This  buffer is needed to store intermediate data.
-  ScratchSize - The size of scratch buffer.
-
-Returns:
-
-  EFI_SUCCESS           - Decompression is successfull
-  EFI_INVALID_PARAMETER - The source data is corrupted
+  @param Source      The source buffer containing the compressed data.
+  @param SrcSize     The size of source buffer
+  @param DstSize     The size of destination buffer.
+  @param ScratchSize The size of scratch buffer.
 
+  @retval EFI_SUCCESS           The size of destination buffer and the size of scratch buffer are successfully retrieved.
+  @retval EFI_INVALID_PARAMETER The source data is corrupted
 **/
-
 EFI_STATUS
 TianoGetInfo (
   IN      VOID    *Source,
@@ -82,26 +79,20 @@ TianoGetInfo (
   OUT     UINT32  *DstSize,
   OUT     UINT32  *ScratchSize
   );
+
 /**
+  The implementation of Tiano Decompress().
 
-Routine Description:
-
-  The implementation Tiano Decompress GetInfo().
-
-Arguments:
-
-  Source      - The source buffer containing the compressed data.
-  SrcSize     - The size of source buffer
-  DstSize     - The size of destination buffer.
-  ScratchSize - The size of scratch buffer.
-
-Returns:
-
-  EFI_SUCCESS           - The size of destination buffer and the size of scratch buffer are successfully retrieved.
-  EFI_INVALID_PARAMETER - The source data is corrupted
+  @param Source      The source buffer containing the compressed data.
+  @param SrcSize     The size of source buffer
+  @param Destination The destination buffer to store the decompressed data
+  @param DstSize     The size of destination buffer.
+  @param Scratch     The buffer used internally by the decompress routine. This  buffer is needed to store intermediate data.
+  @param ScratchSize The size of scratch buffer.
 
+  @retval EFI_SUCCESS           Decompression is successful
+  @retval EFI_INVALID_PARAMETER The source data is corrupted
 **/
-
 EFI_STATUS
 TianoDecompress (
   IN      VOID    *Source,
@@ -111,27 +102,6 @@ TianoDecompress (
   IN OUT  VOID    *Scratch,
   IN      UINT32  ScratchSize
   );
-/**
-
-Routine Description:
-
-  The implementation of Tiano Decompress().
-
-Arguments:
-
-  Source      - The source buffer containing the compressed data.
-  SrcSize     - The size of source buffer
-  Destination - The destination buffer to store the decompressed data
-  DstSize     - The size of destination buffer.
-  Scratch     - The buffer used internally by the decompress routine. This  buffer is needed to store intermediate data.
-  ScratchSize - The size of scratch buffer.
-
-Returns:
-
-  EFI_SUCCESS           - Decompression is successfull
-  EFI_INVALID_PARAMETER - The source data is corrupted
-
-**/
 
 typedef
 EFI_STATUS
