@@ -1,116 +1,135 @@
 /**@file
 
-Copyright (c) 2006 - 2018, Intel Corporation. All rights reserved.<BR>
+Copyright (c) 2006 - 2023, Intel Corporation. All rights reserved.<BR>
 SPDX-License-Identifier: BSD-2-Clause-Patent
 
 Module Name:
 
-  WinNtThunk.c
+  WinThunk.c
 
 Abstract:
 
   Since the SEC is the only windows program in our emulation we
-  must use a Tiano mechanism to export Win32 APIs to other modules.
-  This is the role of the EFI_WIN_NT_THUNK_PROTOCOL.
+  must use a Tiano mechanism to export operating system services
+  to other modules. This is the role of the EMU_THUNK_PROTOCOL.
 
-  The mWinNtThunkTable exists so that a change to EFI_WIN_NT_THUNK_PROTOCOL
+  The gEmuThunkProtocol exists so that a change to EMU_THUNK_PROTOCOL
   will cause an error in initializing the array if all the member functions
   are not added. It looks like adding a element to end and not initializing
-  it may cause the table to be initaliized with the members at the end being
-  set to zero. This is bad as jumping to zero will case the NT32 to crash.
-
-  All the member functions in mWinNtThunkTable are Win32
-  API calls, so please reference Microsoft documentation.
-
-
-  gWinNt is a a public exported global that contains the initialized
-  data.
+  it may cause the table to be initalized with the members at the end being
+  set to zero. This is bad as jumping to zero will case EmulatorPkg to crash.
 
 **/
 
 #include "WinHost.h"
 
+STATIC BOOLEAN  mEmulatorStdInConfigured = FALSE;
+STATIC DWORD    mOldStdInMode;
+#if defined (NTDDI_VERSION) && defined (NTDDI_WIN10_TH2) && (NTDDI_VERSION > NTDDI_WIN10_TH2)
+STATIC DWORD  mOldStdOutMode;
+#endif
+
+STATIC UINT64  mPerformanceFrequency = 0;
+
 UINTN
 SecWriteStdErr (
-  IN UINT8     *Buffer,
-  IN UINTN     NumberOfBytes
+  IN UINT8  *Buffer,
+  IN UINTN  NumberOfBytes
   )
 {
-  BOOL  Success;
-  DWORD CharCount;
+  BOOL   Success;
+  DWORD  CharCount;
 
   CharCount = (DWORD)NumberOfBytes;
-  Success = WriteFile (
-    GetStdHandle (STD_ERROR_HANDLE),
-    Buffer,
-    CharCount,
-    &CharCount,
-    NULL
-    );
+  Success   = WriteFile (
+                GetStdHandle (STD_ERROR_HANDLE),
+                Buffer,
+                CharCount,
+                &CharCount,
+                NULL
+                );
 
   return Success ? CharCount : 0;
 }
 
-
 EFI_STATUS
 SecConfigStdIn (
   VOID
   )
 {
-  BOOL     Success;
-  DWORD    Mode;
+  BOOL   Success;
+  DWORD  Mode;
 
   Success = GetConsoleMode (GetStdHandle (STD_INPUT_HANDLE), &Mode);
   if (Success) {
+    if (!mEmulatorStdInConfigured) {
+      //
+      // Save the original state of the console so it can be restored on exit
+      //
+      mOldStdInMode = Mode;
+    }
+
     //
     // Disable buffer (line input), echo, mouse, window
     //
     Mode &= ~(ENABLE_LINE_INPUT | ENABLE_ECHO_INPUT | ENABLE_MOUSE_INPUT | ENABLE_WINDOW_INPUT);
 
-#if defined(NTDDI_VERSION) && defined (NTDDI_WIN10_TH2) && (NTDDI_VERSION > NTDDI_WIN10_TH2)
+ #if defined (NTDDI_VERSION) && defined (NTDDI_WIN10_TH2) && (NTDDI_VERSION > NTDDI_WIN10_TH2)
     //
     // Enable virtual terminal input for Win10 above TH2
     //
     Mode |= ENABLE_VIRTUAL_TERMINAL_INPUT;
-#endif
+ #endif
 
     Success = SetConsoleMode (GetStdHandle (STD_INPUT_HANDLE), Mode);
   }
 
-#if defined(NTDDI_VERSION) && defined (NTDDI_WIN10_TH2) && (NTDDI_VERSION > NTDDI_WIN10_TH2)
+ #if defined (NTDDI_VERSION) && defined (NTDDI_WIN10_TH2) && (NTDDI_VERSION > NTDDI_WIN10_TH2)
   //
   // Enable terminal mode for Win10 above TH2
   //
   if (Success) {
     Success = GetConsoleMode (GetStdHandle (STD_OUTPUT_HANDLE), &Mode);
+    if (!mEmulatorStdInConfigured) {
+      //
+      // Save the original state of the console so it can be restored on exit
+      //
+      mOldStdOutMode = Mode;
+    }
+
     if (Success) {
       Success = SetConsoleMode (
-        GetStdHandle (STD_OUTPUT_HANDLE),
-        Mode | ENABLE_VIRTUAL_TERMINAL_PROCESSING | DISABLE_NEWLINE_AUTO_RETURN
-      );
+                  GetStdHandle (STD_OUTPUT_HANDLE),
+                  Mode | ENABLE_VIRTUAL_TERMINAL_PROCESSING | DISABLE_NEWLINE_AUTO_RETURN
+                  );
     }
   }
-#endif
+
+ #endif
+  if (Success) {
+    mEmulatorStdInConfigured = TRUE;
+  }
+
   return Success ? EFI_SUCCESS : EFI_DEVICE_ERROR;
 }
 
 UINTN
 SecWriteStdOut (
-  IN UINT8     *Buffer,
-  IN UINTN     NumberOfBytes
+  IN UINT8  *Buffer,
+  IN UINTN  NumberOfBytes
   )
 {
-  BOOL  Success;
-  DWORD CharCount;
+  BOOL   Success;
+  DWORD  CharCount;
 
   CharCount = (DWORD)NumberOfBytes;
-  Success = WriteFile (
-    GetStdHandle (STD_OUTPUT_HANDLE),
-    Buffer,
-    CharCount,
-    &CharCount,
-    NULL
-    );
+  Success   = WriteFile (
+                GetStdHandle (STD_OUTPUT_HANDLE),
+                Buffer,
+                CharCount,
+                &CharCount,
+                NULL
+                );
 
   return Success ? CharCount : 0;
 }
@@ -120,34 +139,35 @@ SecPollStdIn (
   VOID
   )
 {
-  BOOL           Success;
-  INPUT_RECORD   Record;
-  DWORD          RecordNum;
+  BOOL          Success;
+  INPUT_RECORD  Record;
+  DWORD         RecordNum;
 
   do {
     Success = GetNumberOfConsoleInputEvents (GetStdHandle (STD_INPUT_HANDLE), &RecordNum);
     if (!Success || (RecordNum == 0)) {
       break;
     }
+
     Success = PeekConsoleInput (
-      GetStdHandle (STD_INPUT_HANDLE),
-      &Record,
-      1,
-      &RecordNum
-    );
+                GetStdHandle (STD_INPUT_HANDLE),
+                &Record,
+                1,
+                &RecordNum
+                );
     if (Success && (RecordNum == 1)) {
-      if (Record.EventType == KEY_EVENT && Record.Event.KeyEvent.bKeyDown) {
+      if ((Record.EventType == KEY_EVENT) && Record.Event.KeyEvent.bKeyDown) {
         return TRUE;
       } else {
         //
         // Consume the non-key event.
         //
         Success = ReadConsoleInput (
-          GetStdHandle (STD_INPUT_HANDLE),
-          &Record,
-          1,
-          &RecordNum
-        );
+                    GetStdHandle (STD_INPUT_HANDLE),
+                    &Record,
+                    1,
+                    &RecordNum
+                    );
       }
     }
   } while (Success);
@@ -157,37 +177,38 @@ SecPollStdIn (
 
 UINTN
 SecReadStdIn (
-  IN UINT8     *Buffer,
-  IN UINTN     NumberOfBytes
+  IN UINT8  *Buffer,
+  IN UINTN  NumberOfBytes
   )
 {
-  BOOL           Success;
-  INPUT_RECORD   Record;
-  DWORD          RecordNum;
-  UINTN          BytesReturn;
+  BOOL          Success;
+  INPUT_RECORD  Record;
+  DWORD         RecordNum;
+  UINTN         BytesReturn;
 
   if (!SecPollStdIn ()) {
     return 0;
   }
+
   Success = ReadConsoleInput (
-    GetStdHandle (STD_INPUT_HANDLE),
-    &Record,
-    1,
-    &RecordNum
-  );
+              GetStdHandle (STD_INPUT_HANDLE),
+              &Record,
+              1,
+              &RecordNum
+              );
   ASSERT (Success && (RecordNum == 1) && (Record.EventType == KEY_EVENT) && (Record.Event.KeyEvent.bKeyDown));
   NumberOfBytes = MIN (Record.Event.KeyEvent.wRepeatCount, NumberOfBytes);
   BytesReturn   = NumberOfBytes;
   while (NumberOfBytes-- != 0) {
     Buffer[NumberOfBytes] = Record.Event.KeyEvent.uChar.AsciiChar;
   }
+
   return BytesReturn;
 }
 
-
 VOID *
 SecAlloc (
-  IN  UINTN Size
+  IN  UINTN  Size
   )
 {
   return malloc ((size_t)Size);
@@ -195,7 +216,7 @@ SecAlloc (
 
 BOOLEAN
 SecFree (
-  IN  VOID *Ptr
+  IN  VOID  *Ptr
   )
 {
   if (EfiSystemMemoryRange (Ptr)) {
@@ -208,13 +229,11 @@ SecFree (
   return TRUE;
 }
 
-
-
 //
 // Define a global that we can use to shut down the NT timer thread when
 // the timer is canceled.
 //
-BOOLEAN                 mCancelTimerThread = FALSE;
+BOOLEAN  mCancelTimerThread = FALSE;
 
 //
 // The notification function to call on every timer interrupt
@@ -224,41 +243,40 @@ EMU_SET_TIMER_CALLBACK  *mTimerNotifyFunction = NULL;
 //
 // The thread handle for this driver
 //
-HANDLE                  mNtMainThreadHandle;
+HANDLE  mNtMainThreadHandle;
 
 //
 // The timer value from the last timer interrupt
 //
-UINT32                  mNtLastTick;
+UINT32  mNtLastTick;
 
 //
 // Critical section used to update varibles shared between the main thread and
 // the timer interrupt thread.
 //
-CRITICAL_SECTION        mNtCriticalSection;
+CRITICAL_SECTION  mNtCriticalSection;
 
 //
 // Worker Functions
 //
-UINT                    mMMTimerThreadID = 0;
+UINT  mMMTimerThreadID = 0;
 
-volatile BOOLEAN        mInterruptEnabled = FALSE;
+volatile BOOLEAN  mInterruptEnabled = FALSE;
 
 VOID
 CALLBACK
 MMTimerThread (
-  UINT  wTimerID,
-  UINT  msg,
-  DWORD dwUser,
-  DWORD dw1,
-  DWORD dw2
-)
+  UINT       wTimerID,
+  UINT       msg,
+  DWORD_PTR  dwUser,
+  DWORD_PTR  dw1,
+  DWORD_PTR  dw2
+  )
 {
-  UINT32            CurrentTick;
-  UINT32            Delta;
+  UINT32  CurrentTick;
+  UINT32  Delta;
 
   if (!mCancelTimerThread) {
-
     //
     // Suspend the main thread until we are done.
     // Enter the critical section before suspending
@@ -306,14 +324,13 @@ MMTimerThread (
     //  Get the current system tick
     //
     CurrentTick = GetTickCount ();
-    Delta = CurrentTick - mNtLastTick;
+    Delta       = CurrentTick - mNtLastTick;
     mNtLastTick = CurrentTick;
 
     //
     //  If delay was more then 1 second, ignore it (probably debugging case)
     //
     if (Delta < 1000) {
-
       //
       // Only invoke the callback function if a Non-NULL handler has been
       // registered. Assume all other handlers are legal.
@@ -332,18 +349,17 @@ MMTimerThread (
     timeKillEvent (wTimerID);
     mMMTimerThreadID = 0;
   }
-
 }
 
 VOID
 SecSetTimer (
   IN  UINT64                  TimerPeriod,
   IN  EMU_SET_TIMER_CALLBACK  Callback
-)
+  )
 {
   //
-// If TimerPeriod is 0, then the timer thread should be canceled
-//
+  // If TimerPeriod is 0, then the timer thread should be canceled
+  //
   if (TimerPeriod == 0) {
     //
     // Cancel the timer thread
@@ -384,23 +400,24 @@ SecSetTimer (
     SetThreadPriority (
       GetCurrentThread (),
       THREAD_PRIORITY_HIGHEST
-    );
+      );
 
     mMMTimerThreadID = timeSetEvent (
-      (UINT)TimerPeriod,
-      0,
-      MMTimerThread,
-      (DWORD_PTR)NULL,
-      TIME_PERIODIC | TIME_KILL_SYNCHRONOUS | TIME_CALLBACK_FUNCTION
-    );
+                         (UINT)TimerPeriod,
+                         0,
+                         MMTimerThread,
+                         (DWORD_PTR)NULL,
+                         TIME_PERIODIC | TIME_KILL_SYNCHRONOUS | TIME_CALLBACK_FUNCTION
+                         );
   }
+
   mTimerNotifyFunction = Callback;
 }
 
 VOID
 SecInitializeThunk (
   VOID
-)
+  )
 {
   InitializeCriticalSection (&mNtCriticalSection);
 
@@ -412,7 +429,7 @@ SecInitializeThunk (
     0,
     FALSE,
     DUPLICATE_SAME_ACCESS
-  );
+    );
 }
 
 VOID
@@ -423,7 +440,6 @@ SecEnableInterrupt (
   mInterruptEnabled = TRUE;
 }
 
-
 VOID
 SecDisableInterrupt (
   VOID
@@ -432,14 +448,18 @@ SecDisableInterrupt (
   mInterruptEnabled = FALSE;
 }
 
-
 UINT64
 SecQueryPerformanceFrequency (
   VOID
   )
 {
-  // Hard code to nanoseconds
-  return 1000000000ULL;
+  if (mPerformanceFrequency) {
+    return mPerformanceFrequency;
+  }
+
+  QueryPerformanceFrequency ((LARGE_INTEGER *)&mPerformanceFrequency);
+
+  return mPerformanceFrequency;
 }
 
 UINT64
@@ -447,20 +467,21 @@ SecQueryPerformanceCounter (
   VOID
   )
 {
-  return 0;
+  UINT64  PerformanceCount;
+
+  QueryPerformanceCounter ((LARGE_INTEGER *)&PerformanceCount);
+
+  return PerformanceCount;
 }
 
-
-
 VOID
 SecSleep (
-  IN  UINT64 Nanoseconds
+  IN  UINT64  Nanoseconds
   )
 {
   Sleep ((DWORD)DivU64x32 (Nanoseconds, 1000000));
 }
 
-
 VOID
 SecCpuSleep (
   VOID
@@ -469,40 +490,53 @@ SecCpuSleep (
   Sleep (1);
 }
 
-
 VOID
 SecExit (
-  UINTN   Status
+  UINTN  Status
   )
 {
+  if (mEmulatorStdInConfigured) {
+    //
+    // Reset the console back to its original state
+    //
+ #if defined (NTDDI_VERSION) && defined (NTDDI_WIN10_TH2) && (NTDDI_VERSION > NTDDI_WIN10_TH2)
+    BOOL  Success = SetConsoleMode (GetStdHandle (STD_INPUT_HANDLE), mOldStdInMode);
+    if (Success) {
+      SetConsoleMode (GetStdHandle (STD_OUTPUT_HANDLE), mOldStdOutMode);
+    }
+
+ #else
+    SetConsoleMode (GetStdHandle (STD_INPUT_HANDLE), mOldStdInMode);
+ #endif
+  }
+
   exit ((int)Status);
 }
 
-
 VOID
 SecGetTime (
-  OUT  EFI_TIME               *Time,
-  OUT EFI_TIME_CAPABILITIES   *Capabilities OPTIONAL
+  OUT  EFI_TIME              *Time,
+  OUT EFI_TIME_CAPABILITIES  *Capabilities OPTIONAL
   )
 {
-  SYSTEMTIME            SystemTime;
-  TIME_ZONE_INFORMATION TimeZone;
+  SYSTEMTIME             SystemTime;
+  TIME_ZONE_INFORMATION  TimeZone;
 
   GetLocalTime (&SystemTime);
   GetTimeZoneInformation (&TimeZone);
 
-  Time->Year = (UINT16)SystemTime.wYear;
-  Time->Month = (UINT8)SystemTime.wMonth;
-  Time->Day = (UINT8)SystemTime.wDay;
-  Time->Hour = (UINT8)SystemTime.wHour;
-  Time->Minute = (UINT8)SystemTime.wMinute;
-  Time->Second = (UINT8)SystemTime.wSecond;
+  Time->Year       = (UINT16)SystemTime.wYear;
+  Time->Month      = (UINT8)SystemTime.wMonth;
+  Time->Day        = (UINT8)SystemTime.wDay;
+  Time->Hour       = (UINT8)SystemTime.wHour;
+  Time->Minute     = (UINT8)SystemTime.wMinute;
+  Time->Second     = (UINT8)SystemTime.wSecond;
   Time->Nanosecond = (UINT32)(SystemTime.wMilliseconds * 1000000);
-  Time->TimeZone = (INT16)TimeZone.Bias;
+  Time->TimeZone   = (INT16)TimeZone.Bias;
 
   if (Capabilities != NULL) {
     Capabilities->Resolution = 1;
-    Capabilities->Accuracy = 50000000;
+    Capabilities->Accuracy   = 50000000;
     Capabilities->SetsToZero = FALSE;
   }
 
@@ -514,30 +548,30 @@ SecGetTime (
 
 EFI_STATUS
 SecSetTime (
-  IN  EFI_TIME               *Time
+  IN  EFI_TIME  *Time
   )
 {
-  TIME_ZONE_INFORMATION TimeZone;
-  SYSTEMTIME            SystemTime;
-  BOOL                  Flag;
+  TIME_ZONE_INFORMATION  TimeZone;
+  SYSTEMTIME             SystemTime;
+  BOOL                   Flag;
 
   //
   // Set Daylight savings time information and Time Zone
   //
   GetTimeZoneInformation (&TimeZone);
   TimeZone.StandardDate.wMonth = Time->Daylight;
-  TimeZone.Bias = Time->TimeZone;
-  Flag = SetTimeZoneInformation (&TimeZone);
+  TimeZone.Bias                = Time->TimeZone;
+  Flag                         = SetTimeZoneInformation (&TimeZone);
   if (!Flag) {
     return EFI_DEVICE_ERROR;
   }
 
-  SystemTime.wYear = Time->Year;
-  SystemTime.wMonth = Time->Month;
-  SystemTime.wDay = Time->Day;
-  SystemTime.wHour = Time->Hour;
-  SystemTime.wMinute = Time->Minute;
-  SystemTime.wSecond = Time->Second;
+  SystemTime.wYear         = Time->Year;
+  SystemTime.wMonth        = Time->Month;
+  SystemTime.wDay          = Time->Day;
+  SystemTime.wHour         = Time->Hour;
+  SystemTime.wMinute       = Time->Minute;
+  SystemTime.wSecond       = Time->Second;
   SystemTime.wMilliseconds = (INT16)(Time->Nanosecond / 1000000);
 
   Flag = SetLocalTime (&SystemTime);
@@ -549,7 +583,7 @@ SecSetTime (
   }
 }
 
-EMU_THUNK_PROTOCOL gEmuThunkProtocol = {
+EMU_THUNK_PROTOCOL  gEmuThunkProtocol = {
   SecWriteStdErr,
   SecConfigStdIn,
   SecWriteStdOut,
@@ -574,7 +608,5 @@ EMU_THUNK_PROTOCOL gEmuThunkProtocol = {
   GetNextThunkProtocol
 };
 
-
 #pragma warning(default : 4996)
 #pragma warning(default : 4232)
-
