@@ -4,7 +4,7 @@
   information provided through PCDs and libraries.
 
   Copyright (c) Microsoft Corporation.<BR>
-  Copyright (c) 2018 - 2019, Intel Corporation. All rights reserved.<BR>
+  Copyright (c) 2018 - 2021, Intel Corporation. All rights reserved.<BR>
 
   SPDX-License-Identifier: BSD-2-Clause-Patent
 
@@ -33,11 +33,11 @@
 #include <Library/FmpDependencyDeviceLib.h>
 #include <Protocol/FirmwareManagement.h>
 #include <Protocol/FirmwareManagementProgress.h>
-#include <Protocol/VariableLock.h>
 #include <Guid/SystemResourceTable.h>
 #include <Guid/EventGroup.h>
 #include <LastAttemptStatus.h>
 #include <FmpLastAttemptStatus.h>
+#include <Library/VariablePolicyHelperLib.h>
 
 #define VERSION_STRING_NOT_SUPPORTED  L"VERSION STRING NOT SUPPORTED"
 #define VERSION_STRING_NOT_AVAILABLE  L"VERSION STRING NOT AVAILABLE"
@@ -45,18 +45,18 @@
 ///
 ///
 ///
-#define FIRMWARE_MANAGEMENT_PRIVATE_DATA_SIGNATURE SIGNATURE_32 ('f','m','p','p')
+#define FIRMWARE_MANAGEMENT_PRIVATE_DATA_SIGNATURE  SIGNATURE_32 ('f','m','p','p')
 
 typedef struct {
-  UINTN                                        Signature;
-  EFI_HANDLE                                   Handle;
-  EFI_FIRMWARE_MANAGEMENT_PROTOCOL             Fmp;
-  BOOLEAN                                      DescriptorPopulated;
-  EFI_FIRMWARE_IMAGE_DESCRIPTOR                Descriptor;
-  CHAR16                                       *ImageIdName;
-  CHAR16                                       *VersionName;
-  BOOLEAN                                      RuntimeVersionSupported;
-  EFI_EVENT                                    FmpDeviceLockEvent;
+  UINTN                               Signature;
+  EFI_HANDLE                          Handle;
+  EFI_FIRMWARE_MANAGEMENT_PROTOCOL    Fmp;
+  BOOLEAN                             DescriptorPopulated;
+  EFI_FIRMWARE_IMAGE_DESCRIPTOR       Descriptor;
+  CHAR16                              *ImageIdName;
+  CHAR16                              *VersionName;
+  BOOLEAN                             RuntimeVersionSupported;
+  EFI_EVENT                           FmpDeviceLockEvent;
   //
   // Indicates if an attempt has been made to lock a
   // FLASH storage device by calling FmpDeviceLock().
@@ -64,14 +64,14 @@ typedef struct {
   // so this variable is set to TRUE even if FmpDeviceLock()
   // returns an error.
   //
-  BOOLEAN                                      FmpDeviceLocked;
-  VOID                                         *FmpDeviceContext;
-  CHAR16                                       *VersionVariableName;
-  CHAR16                                       *LsvVariableName;
-  CHAR16                                       *LastAttemptStatusVariableName;
-  CHAR16                                       *LastAttemptVersionVariableName;
-  CHAR16                                       *FmpStateVariableName;
-  BOOLEAN                                      DependenciesSatisfied;
+  BOOLEAN                             FmpDeviceLocked;
+  VOID                                *FmpDeviceContext;
+  CHAR16                              *VersionVariableName;
+  CHAR16                              *LsvVariableName;
+  CHAR16                              *LastAttemptStatusVariableName;
+  CHAR16                              *LastAttemptVersionVariableName;
+  CHAR16                              *FmpStateVariableName;
+  BOOLEAN                             DependenciesSatisfied;
 } FIRMWARE_MANAGEMENT_PRIVATE_DATA;
 
 ///
@@ -183,7 +183,6 @@ GetTheImage (
   IN OUT UINTN                             *ImageSize
   );
 
-
 /**
   Checks if the firmware image is valid for the device.
 
