@@ -1,7 +1,7 @@
 /** @file
   Internal library implementation for PCI Bus module.
 
-Copyright (c) 2006 - 2021, Intel Corporation. All rights reserved.<BR>
+Copyright (c) 2006 - 2022, Intel Corporation. All rights reserved.<BR>
 (C) Copyright 2015 Hewlett Packard Enterprise Development LP<BR>
 SPDX-License-Identifier: BSD-2-Clause-Patent
 
@@ -1106,6 +1106,7 @@ PciScanBus (
   EFI_PCI_ROOT_BRIDGE_IO_PROTOCOL    *PciRootBridgeIo;
   BOOLEAN                            BusPadding;
   UINT32                             TempReservedBusNum;
+  BOOLEAN                            IsAriEnabled;
 
   PciRootBridgeIo = Bridge->PciRootBridgeIo;
   SecondBus       = 0;
@@ -1116,9 +1117,13 @@ PciScanBus (
   BusPadding      = FALSE;
   PciDevice       = NULL;
   PciAddress      = 0;
+  IsAriEnabled    = FALSE;
 
   for (Device = 0; Device <= PCI_MAX_DEVICE; Device++) {
-    TempReservedBusNum = 0;
+    if (!IsAriEnabled) {
+      TempReservedBusNum = 0;
+    }
+
     for (Func = 0; Func <= PCI_MAX_FUNC; Func++) {
       //
       // Check to see whether a pci device is present
@@ -1158,6 +1163,27 @@ PciScanBus (
         continue;
       }
 
+      //
+      // Per Pcie spec ARI Extended Capability
+      // This capability must be implemented by each function in an ARI device.
+      // It is not applicable to a Root Port, a Switch Downstream Port, an RCiEP, or a Root Complex Event Collector
+      //
+      if (((Device == 0) && (Func == 0)) && (PciDevice->IsAriEnabled)) {
+        IsAriEnabled = TRUE;
+      }
+
+      if (PciDevice->IsAriEnabled != IsAriEnabled) {
+        DEBUG ((
+          DEBUG_ERROR,
+          "ERROR: %02x:%02x:%02x device ARI Feature(%x) is not consistent with others Function\n",
+          StartBusNumber,
+          Device,
+          Func,
+          PciDevice->IsAriEnabled
+          ));
+        return EFI_DEVICE_ERROR;
+      }
+
       PciAddress = EFI_PCI_ADDRESS (StartBusNumber, Device, Func, 0);
 
       if (!IS_PCI_BRIDGE (&Pci)) {
@@ -1528,6 +1554,7 @@ PciHostBridgeEnumerator (
   UINT8                              StartBusNumber;
   LIST_ENTRY                         RootBridgeList;
   LIST_ENTRY                         *Link;
+  EFI_STATUS                         RootBridgeEnumerationStatus;
 
   if (FeaturePcdGet (PcdPciBusHotplugDeviceSupport)) {
     InitializeHotPlugSupport ();
@@ -1545,7 +1572,8 @@ PciHostBridgeEnumerator (
   }
 
   DEBUG ((DEBUG_INFO, "PCI Bus First Scanning\n"));
-  RootBridgeHandle = NULL;
+  RootBridgeHandle            = NULL;
+  RootBridgeEnumerationStatus = EFI_SUCCESS;
   while (PciResAlloc->GetNextRootBridge (PciResAlloc, &RootBridgeHandle) == EFI_SUCCESS) {
     //
     // if a root bridge instance is found, create root bridge device for it
@@ -1572,7 +1600,7 @@ PciHostBridgeEnumerator (
     }
 
     if (EFI_ERROR (Status)) {
-      return Status;
+      RootBridgeEnumerationStatus = Status;
     }
   }
 
@@ -1581,6 +1609,10 @@ PciHostBridgeEnumerator (
   //
   NotifyPhase (PciResAlloc, EfiPciHostBridgeEndBusAllocation);
 
+  if (EFI_ERROR (RootBridgeEnumerationStatus)) {
+    return RootBridgeEnumerationStatus;
+  }
+
   if ((gPciHotPlugInit != NULL) && FeaturePcdGet (PcdPciBusHotplugDeviceSupport)) {
     //
     // Reset all assigned PCI bus number in all PPB
@@ -1659,7 +1691,7 @@ PciHostBridgeEnumerator (
 
       DestroyRootBridge (RootBridgeDev);
       if (EFI_ERROR (Status)) {
-        return Status;
+        RootBridgeEnumerationStatus = Status;
       }
     }
 
@@ -1667,6 +1699,10 @@ PciHostBridgeEnumerator (
     // Notify the bus allocation phase is to end for the 2nd time
     //
     NotifyPhase (PciResAlloc, EfiPciHostBridgeEndBusAllocation);
+
+    if (EFI_ERROR (RootBridgeEnumerationStatus)) {
+      return RootBridgeEnumerationStatus;
+    }
   }
 
   //
