@@ -1,7 +1,7 @@
 /** @file
   DXE Core Main Entry Point
 
-Copyright (c) 2006 - 2018, Intel Corporation. All rights reserved.<BR>
+Copyright (c) 2006 - 2022, Intel Corporation. All rights reserved.<BR>
 SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
@@ -203,6 +203,8 @@ EFI_HANDLE            gDxeCoreImageHandle = NULL;
 
 BOOLEAN  gMemoryMapTerminated = FALSE;
 
+static BOOLEAN  mExitBootServicesCalled = FALSE;
+
 //
 // EFI Decompress Protocol
 //
@@ -253,9 +255,17 @@ DxeMain (
     VectorInfoList = (EFI_VECTOR_HANDOFF_INFO *)(GET_GUID_HOB_DATA (GuidHob));
   }
 
-  Status = InitializeCpuExceptionHandlersEx (VectorInfoList, NULL);
+  Status = InitializeCpuExceptionHandlers (VectorInfoList);
   ASSERT_EFI_ERROR (Status);
 
+  //
+  // Setup Stack Guard
+  //
+  if (PcdGetBool (PcdCpuStackGuard)) {
+    Status = InitializeSeparateExceptionStacks (NULL, NULL);
+    ASSERT_EFI_ERROR (Status);
+  }
+
   //
   // Initialize Debug Agent to support source level debug in DXE phase
   //
@@ -269,16 +279,10 @@ DxeMain (
   MemoryProfileInit (HobStart);
 
   //
-  // Allocate the EFI System Table and EFI Runtime Service Table from EfiRuntimeServicesData
-  // Use the templates to initialize the contents of the EFI System Table and EFI Runtime Services Table
+  // Start the Handle Services.
   //
-  gDxeCoreST = AllocateRuntimeCopyPool (sizeof (EFI_SYSTEM_TABLE), &mEfiSystemTableTemplate);
-  ASSERT (gDxeCoreST != NULL);
-
-  gDxeCoreRT = AllocateRuntimeCopyPool (sizeof (EFI_RUNTIME_SERVICES), &mEfiRuntimeServicesTableTemplate);
-  ASSERT (gDxeCoreRT != NULL);
-
-  gDxeCoreST->RuntimeServices = gDxeCoreRT;
+  Status = CoreInitializeHandleServices ();
+  ASSERT_EFI_ERROR (Status);
 
   //
   // Start the Image Services.
@@ -292,6 +296,23 @@ DxeMain (
   Status = CoreInitializeGcdServices (&HobStart, MemoryBaseAddress, MemoryLength);
   ASSERT_EFI_ERROR (Status);
 
+  //
+  // Allocate the EFI System Table and EFI Runtime Service Table from EfiRuntimeServicesData
+  // Use the templates to initialize the contents of the EFI System Table and EFI Runtime Services Table
+  //
+  gDxeCoreST = AllocateRuntimeCopyPool (sizeof (EFI_SYSTEM_TABLE), &mEfiSystemTableTemplate);
+  ASSERT (gDxeCoreST != NULL);
+
+  gDxeCoreRT = AllocateRuntimeCopyPool (sizeof (EFI_RUNTIME_SERVICES), &mEfiRuntimeServicesTableTemplate);
+  ASSERT (gDxeCoreRT != NULL);
+
+  gDxeCoreST->RuntimeServices = gDxeCoreRT;
+
+  //
+  // Update DXE Core Loaded Image Protocol with allocated UEFI System Table
+  //
+  gDxeCoreLoadedImage->SystemTable = gDxeCoreST;
+
   //
   // Call constructor for all libraries
   //
@@ -307,7 +328,7 @@ DxeMain (
   DEBUG ((
     DEBUG_INFO,
     "%a: MemoryBaseAddress=0x%Lx MemoryLength=0x%Lx\n",
-    __FUNCTION__,
+    __func__,
     MemoryBaseAddress,
     MemoryLength
     ));
@@ -442,6 +463,11 @@ DxeMain (
   Status = CoreInitializeEventServices ();
   ASSERT_EFI_ERROR (Status);
 
+  //
+  // Give the debug agent a chance to initialize with events.
+  //
+  InitializeDebugAgent (DEBUG_AGENT_INIT_DXE_CORE_LATE, HobStart, NULL);
+
   MemoryProfileInstallProtocol ();
 
   CoreInitializeMemoryAttributesTable ();
@@ -755,6 +781,15 @@ CoreExitBootServices (
 {
   EFI_STATUS  Status;
 
+  //
+  // Notify other drivers of their last chance to use boot services
+  // before the memory map is terminated.
+  //
+  if (!mExitBootServicesCalled) {
+    CoreNotifySignalList (&gEfiEventBeforeExitBootServicesGuid);
+    mExitBootServicesCalled = TRUE;
+  }
+
   //
   // Disable Timer
   //
