@@ -5,14 +5,8 @@
   for Firmware Basic Boot Performance Record and other boot performance records,
   and install FPDT to ACPI table.
 
-  Copyright (c) 2011 - 2018, Intel Corporation. All rights reserved.<BR>
-  This program and the accompanying materials
-  are licensed and made available under the terms and conditions of the BSD License
-  which accompanies this distribution.  The full text of the license may be found at
-  http://opensource.org/licenses/bsd-license.php
-
-  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-  WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+  Copyright (c) 2011 - 2021, Intel Corporation. All rights reserved.<BR>
+  SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
@@ -22,6 +16,7 @@
 #include <Protocol/AcpiTable.h>
 #include <Protocol/LockBox.h>
 #include <Protocol/Variable.h>
+#include <Protocol/VariablePolicy.h>
 
 #include <Guid/Acpi.h>
 #include <Guid/FirmwarePerformance.h>
@@ -38,32 +33,34 @@
 #include <Library/HobLib.h>
 #include <Library/LockBoxLib.h>
 #include <Library/UefiLib.h>
+#include <Library/VariablePolicyHelperLib.h>
+#include <Library/PerformanceLib.h>
 
-#define SMM_BOOT_RECORD_COMM_SIZE (OFFSET_OF (EFI_SMM_COMMUNICATE_HEADER, Data) + sizeof(SMM_BOOT_RECORD_COMMUNICATE))
+#define SMM_BOOT_RECORD_COMM_SIZE  (OFFSET_OF (EFI_SMM_COMMUNICATE_HEADER, Data) + sizeof(SMM_BOOT_RECORD_COMMUNICATE))
 
-EFI_RSC_HANDLER_PROTOCOL    *mRscHandlerProtocol = NULL;
+EFI_RSC_HANDLER_PROTOCOL  *mRscHandlerProtocol = NULL;
 
-BOOLEAN                     mLockBoxReady = FALSE;
-EFI_EVENT                   mReadyToBootEvent;
-EFI_EVENT                   mLegacyBootEvent;
-EFI_EVENT                   mExitBootServicesEvent;
-UINTN                       mFirmwarePerformanceTableTemplateKey  = 0;
-BOOLEAN                     mDxeCoreReportStatusCodeEnable = FALSE;
+BOOLEAN           mLockBoxReady = FALSE;
+EFI_EVENT         mReadyToBootEvent;
+EFI_EVENT         mLegacyBootEvent;
+static EFI_EVENT  mExitBootServicesEvent;
+UINTN             mFirmwarePerformanceTableTemplateKey = 0;
+BOOLEAN           mDxeCoreReportStatusCodeEnable       = FALSE;
 
-BOOT_PERFORMANCE_TABLE                      *mAcpiBootPerformanceTable = NULL;
-BOOT_PERFORMANCE_TABLE                      *mReceivedAcpiBootPerformanceTable = NULL;
-S3_PERFORMANCE_TABLE                        *mAcpiS3PerformanceTable   = NULL;
+BOOT_PERFORMANCE_TABLE  *mAcpiBootPerformanceTable         = NULL;
+BOOT_PERFORMANCE_TABLE  *mReceivedAcpiBootPerformanceTable = NULL;
+S3_PERFORMANCE_TABLE    *mAcpiS3PerformanceTable           = NULL;
 
 FIRMWARE_PERFORMANCE_TABLE  mFirmwarePerformanceTableTemplate = {
   {
     EFI_ACPI_5_0_FIRMWARE_PERFORMANCE_DATA_TABLE_SIGNATURE,
     sizeof (FIRMWARE_PERFORMANCE_TABLE),
-    EFI_ACPI_5_0_FIRMWARE_PERFORMANCE_DATA_TABLE_REVISION,    // Revision
-    0x00, // Checksum will be updated at runtime
+    EFI_ACPI_5_0_FIRMWARE_PERFORMANCE_DATA_TABLE_REVISION, // Revision
+    0x00,                                                  // Checksum will be updated at runtime
     //
     // It is expected that these values will be updated at EntryPoint.
     //
-    {0x00},     // OEM ID is a 6 bytes long field
+    { 0x00 },   // OEM ID is a 6 bytes long field
     0x00,       // OEM Table ID(8 bytes long)
     0x00,       // OEM Revision
     0x00,       // Creator ID
@@ -74,7 +71,7 @@ FIRMWARE_PERFORMANCE_TABLE  mFirmwarePerformanceTableTemplate = {
   //
   {
     {
-      EFI_ACPI_5_0_FPDT_RECORD_TYPE_FIRMWARE_BASIC_BOOT_POINTER ,       // Type
+      EFI_ACPI_5_0_FPDT_RECORD_TYPE_FIRMWARE_BASIC_BOOT_POINTER,        // Type
       sizeof (EFI_ACPI_5_0_FPDT_BOOT_PERFORMANCE_TABLE_POINTER_RECORD), // Length
       EFI_ACPI_5_0_FPDT_RECORD_REVISION_FIRMWARE_BASIC_BOOT_POINTER     // Revision
     },
@@ -95,7 +92,7 @@ FIRMWARE_PERFORMANCE_TABLE  mFirmwarePerformanceTableTemplate = {
   }
 };
 
-BOOT_PERFORMANCE_TABLE mBootPerformanceTableTemplate = {
+BOOT_PERFORMANCE_TABLE  mBootPerformanceTableTemplate = {
   {
     EFI_ACPI_5_0_FPDT_BOOT_PERFORMANCE_TABLE_SIGNATURE,
     sizeof (BOOT_PERFORMANCE_TABLE)
@@ -118,7 +115,7 @@ BOOT_PERFORMANCE_TABLE mBootPerformanceTableTemplate = {
   }
 };
 
-S3_PERFORMANCE_TABLE        mS3PerformanceTableTemplate = {
+S3_PERFORMANCE_TABLE  mS3PerformanceTableTemplate = {
   {
     EFI_ACPI_5_0_FPDT_S3_PERFORMANCE_TABLE_SIGNATURE,
     sizeof (S3_PERFORMANCE_TABLE)
@@ -159,11 +156,11 @@ S3_PERFORMANCE_TABLE        mS3PerformanceTableTemplate = {
 **/
 VOID
 FpdtAcpiTableChecksum (
-  IN UINT8      *Buffer,
-  IN UINTN      Size
+  IN UINT8  *Buffer,
+  IN UINTN  Size
   )
 {
-  UINTN ChecksumOffset;
+  UINTN  ChecksumOffset;
 
   ChecksumOffset = OFFSET_OF (EFI_ACPI_DESCRIPTION_HEADER, Checksum);
 
@@ -188,15 +185,15 @@ FpdtAcpiTableChecksum (
 VOID
 EFIAPI
 FpdtAllocateS3PerformanceTableMemory (
-  IN  EFI_EVENT                             Event,
-  IN  VOID                                  *Context
+  IN  EFI_EVENT  Event,
+  IN  VOID       *Context
   )
 {
-  EFI_STATUS                    Status;
-  VOID                          *Interface;
-  FIRMWARE_PERFORMANCE_VARIABLE PerformanceVariable;
-  UINTN                         Size;
-  EFI_PHYSICAL_ADDRESS          S3PerformanceTablePointer;
+  EFI_STATUS                     Status;
+  VOID                           *Interface;
+  FIRMWARE_PERFORMANCE_VARIABLE  PerformanceVariable;
+  UINTN                          Size;
+  EFI_PHYSICAL_ADDRESS           S3PerformanceTablePointer;
 
   if (mLockBoxReady && (mAcpiS3PerformanceTable != NULL)) {
     //
@@ -223,7 +220,7 @@ FpdtAllocateS3PerformanceTableMemory (
       // Try to allocate the same runtime buffer as last time boot.
       //
       ZeroMem (&PerformanceVariable, sizeof (PerformanceVariable));
-      Size = sizeof (PerformanceVariable);
+      Size   = sizeof (PerformanceVariable);
       Status = gRT->GetVariable (
                       EFI_FIRMWARE_PERFORMANCE_VARIABLE_NAME,
                       &gEfiFirmwarePerformanceGuid,
@@ -239,19 +236,21 @@ FpdtAllocateS3PerformanceTableMemory (
                         &PerformanceVariable.S3PerformanceTablePointer
                         );
         if (!EFI_ERROR (Status)) {
-          mAcpiS3PerformanceTable = (S3_PERFORMANCE_TABLE *) (UINTN) PerformanceVariable.S3PerformanceTablePointer;
+          mAcpiS3PerformanceTable = (S3_PERFORMANCE_TABLE *)(UINTN)PerformanceVariable.S3PerformanceTablePointer;
         }
       }
+
       if (mAcpiS3PerformanceTable == NULL) {
         //
         // Fail to allocate at specified address, continue to allocate at any address.
         //
-        mAcpiS3PerformanceTable = (S3_PERFORMANCE_TABLE *) AllocatePeiAccessiblePages (
-                                                             EfiReservedMemoryType,
-                                                             EFI_SIZE_TO_PAGES (sizeof (S3_PERFORMANCE_TABLE))
-                                                             );
+        mAcpiS3PerformanceTable = (S3_PERFORMANCE_TABLE *)AllocatePeiAccessiblePages (
+                                                            EfiReservedMemoryType,
+                                                            EFI_SIZE_TO_PAGES (sizeof (S3_PERFORMANCE_TABLE))
+                                                            );
       }
-      DEBUG ((EFI_D_INFO, "FPDT: ACPI S3 Performance Table address = 0x%x\n", mAcpiS3PerformanceTable));
+
+      DEBUG ((DEBUG_INFO, "FPDT: ACPI S3 Performance Table address = 0x%x\n", mAcpiS3PerformanceTable));
       if (mAcpiS3PerformanceTable != NULL) {
         CopyMem (mAcpiS3PerformanceTable, &mS3PerformanceTableTemplate, sizeof (mS3PerformanceTableTemplate));
       }
@@ -263,12 +262,12 @@ FpdtAllocateS3PerformanceTableMemory (
     // If LockBox services has been ready and memory for FPDT S3 performance table has been allocated,
     // save the pointer to LockBox for use in S3 resume.
     //
-    S3PerformanceTablePointer = (EFI_PHYSICAL_ADDRESS) (UINTN) mAcpiS3PerformanceTable;
-    Status = SaveLockBox (
-               &gFirmwarePerformanceS3PointerGuid,
-               &S3PerformanceTablePointer,
-               sizeof (EFI_PHYSICAL_ADDRESS)
-               );
+    S3PerformanceTablePointer = (EFI_PHYSICAL_ADDRESS)(UINTN)mAcpiS3PerformanceTable;
+    Status                    = SaveLockBox (
+                                  &gFirmwarePerformanceS3PointerGuid,
+                                  &S3PerformanceTablePointer,
+                                  sizeof (EFI_PHYSICAL_ADDRESS)
+                                  );
     ASSERT_EFI_ERROR (Status);
   }
 }
@@ -284,22 +283,31 @@ InstallFirmwarePerformanceDataTable (
   VOID
   )
 {
-  EFI_STATUS                    Status;
-  EFI_ACPI_TABLE_PROTOCOL       *AcpiTableProtocol;
-  UINTN                         BootPerformanceDataSize;
-  FIRMWARE_PERFORMANCE_VARIABLE PerformanceVariable;
-  UINTN                         Size;
+  EFI_STATUS                      Status;
+  EFI_ACPI_TABLE_PROTOCOL         *AcpiTableProtocol;
+  UINTN                           BootPerformanceDataSize;
+  FIRMWARE_PERFORMANCE_VARIABLE   PerformanceVariable;
+  UINTN                           Size;
+  EDKII_VARIABLE_POLICY_PROTOCOL  *VariablePolicyProtocol;
 
   //
   // Get AcpiTable Protocol.
   //
-  Status = gBS->LocateProtocol (&gEfiAcpiTableProtocolGuid, NULL, (VOID **) &AcpiTableProtocol);
+  Status = gBS->LocateProtocol (&gEfiAcpiTableProtocolGuid, NULL, (VOID **)&AcpiTableProtocol);
+  if (EFI_ERROR (Status)) {
+    return Status;
+  }
+
+  //
+  // Get VariablePolicy Protocol.
+  //
+  Status = gBS->LocateProtocol (&gEdkiiVariablePolicyProtocolGuid, NULL, (VOID **)&VariablePolicyProtocol);
   if (EFI_ERROR (Status)) {
     return Status;
   }
 
   if (mReceivedAcpiBootPerformanceTable != NULL) {
-    mAcpiBootPerformanceTable = mReceivedAcpiBootPerformanceTable;
+    mAcpiBootPerformanceTable                     = mReceivedAcpiBootPerformanceTable;
     mAcpiBootPerformanceTable->BasicBoot.ResetEnd = mBootPerformanceTableTemplate.BasicBoot.ResetEnd;
   } else {
     //
@@ -307,7 +315,7 @@ InstallFirmwarePerformanceDataTable (
     //
     BootPerformanceDataSize = sizeof (BOOT_PERFORMANCE_TABLE);
     ZeroMem (&PerformanceVariable, sizeof (PerformanceVariable));
-    Size = sizeof (PerformanceVariable);
+    Size   = sizeof (PerformanceVariable);
     Status = gRT->GetVariable (
                     EFI_FIRMWARE_PERFORMANCE_VARIABLE_NAME,
                     &gEfiFirmwarePerformanceGuid,
@@ -323,62 +331,84 @@ InstallFirmwarePerformanceDataTable (
                       &PerformanceVariable.BootPerformanceTablePointer
                       );
       if (!EFI_ERROR (Status)) {
-         mAcpiBootPerformanceTable = (BOOT_PERFORMANCE_TABLE *) (UINTN) PerformanceVariable.BootPerformanceTablePointer;
+        mAcpiBootPerformanceTable = (BOOT_PERFORMANCE_TABLE *)(UINTN)PerformanceVariable.BootPerformanceTablePointer;
       }
     }
+
     if (mAcpiBootPerformanceTable == NULL) {
       //
       // Fail to allocate at specified address, continue to allocate at any address.
       //
-      mAcpiBootPerformanceTable = (BOOT_PERFORMANCE_TABLE *) AllocatePeiAccessiblePages (
-                                                               EfiReservedMemoryType,
-                                                               EFI_SIZE_TO_PAGES (BootPerformanceDataSize)
-                                                               );
+      mAcpiBootPerformanceTable = (BOOT_PERFORMANCE_TABLE *)AllocatePeiAccessiblePages (
+                                                              EfiReservedMemoryType,
+                                                              EFI_SIZE_TO_PAGES (BootPerformanceDataSize)
+                                                              );
     }
+
     DEBUG ((DEBUG_INFO, "FPDT: ACPI Boot Performance Table address = 0x%x\n", mAcpiBootPerformanceTable));
     if (mAcpiBootPerformanceTable == NULL) {
       return EFI_OUT_OF_RESOURCES;
     }
+
     //
     // Fill Basic Boot record to Boot Performance Table.
     //
     CopyMem (mAcpiBootPerformanceTable, &mBootPerformanceTableTemplate, sizeof (mBootPerformanceTableTemplate));
   }
-  BootPerformanceDataSize   = mAcpiBootPerformanceTable->Header.Length;
+
+  BootPerformanceDataSize = mAcpiBootPerformanceTable->Header.Length;
 
   //
   // Save Boot Performance Table address to Variable for use in S4 resume.
   //
-  PerformanceVariable.BootPerformanceTablePointer = (EFI_PHYSICAL_ADDRESS) (UINTN) mAcpiBootPerformanceTable;
+  PerformanceVariable.BootPerformanceTablePointer = (EFI_PHYSICAL_ADDRESS)(UINTN)mAcpiBootPerformanceTable;
   //
   // Update Boot Performance Table Pointer in template.
   //
-  mFirmwarePerformanceTableTemplate.BootPointerRecord.BootPerformanceTablePointer = (UINT64) (UINTN) mAcpiBootPerformanceTable;
+  mFirmwarePerformanceTableTemplate.BootPointerRecord.BootPerformanceTablePointer = (UINT64)(UINTN)mAcpiBootPerformanceTable;
 
   //
   // Save S3 Performance Table address to Variable for use in S4 resume.
   //
-  PerformanceVariable.S3PerformanceTablePointer = (EFI_PHYSICAL_ADDRESS) (UINTN) mAcpiS3PerformanceTable;
+  PerformanceVariable.S3PerformanceTablePointer = (EFI_PHYSICAL_ADDRESS)(UINTN)mAcpiS3PerformanceTable;
   //
   // Update S3 Performance Table Pointer in template.
   //
-  mFirmwarePerformanceTableTemplate.S3PointerRecord.S3PerformanceTablePointer = (UINT64) (UINTN) mAcpiS3PerformanceTable;
+  mFirmwarePerformanceTableTemplate.S3PointerRecord.S3PerformanceTablePointer = (UINT64)(UINTN)mAcpiS3PerformanceTable;
   //
   // Save Runtime Performance Table pointers to Variable.
   // Don't check SetVariable return status. It doesn't impact FPDT table generation.
   //
   gRT->SetVariable (
-        EFI_FIRMWARE_PERFORMANCE_VARIABLE_NAME,
-        &gEfiFirmwarePerformanceGuid,
-        EFI_VARIABLE_NON_VOLATILE | EFI_VARIABLE_BOOTSERVICE_ACCESS,
-        sizeof (PerformanceVariable),
-        &PerformanceVariable
-        );
+         EFI_FIRMWARE_PERFORMANCE_VARIABLE_NAME,
+         &gEfiFirmwarePerformanceGuid,
+         EFI_VARIABLE_NON_VOLATILE | EFI_VARIABLE_BOOTSERVICE_ACCESS,
+         sizeof (PerformanceVariable),
+         &PerformanceVariable
+         );
+
+  //
+  // Lock the variable which stores the Performance Table pointers.
+  //
+  Status = RegisterBasicVariablePolicy (
+             VariablePolicyProtocol,
+             &gEfiFirmwarePerformanceGuid,
+             EFI_FIRMWARE_PERFORMANCE_VARIABLE_NAME,
+             VARIABLE_POLICY_NO_MIN_SIZE,
+             VARIABLE_POLICY_NO_MAX_SIZE,
+             VARIABLE_POLICY_NO_MUST_ATTR,
+             VARIABLE_POLICY_NO_CANT_ATTR,
+             VARIABLE_POLICY_TYPE_LOCK_NOW
+             );
+  if (EFI_ERROR (Status)) {
+    DEBUG ((DEBUG_ERROR, "[FirmwarePerformanceDxe] Error when lock variable %s, Status = %r\n", EFI_FIRMWARE_PERFORMANCE_VARIABLE_NAME, Status));
+    ASSERT_EFI_ERROR (Status);
+  }
 
   //
   // Publish Firmware Performance Data Table.
   //
-  FpdtAcpiTableChecksum ((UINT8 *) &mFirmwarePerformanceTableTemplate, mFirmwarePerformanceTableTemplate.Header.Length);
+  FpdtAcpiTableChecksum ((UINT8 *)&mFirmwarePerformanceTableTemplate, mFirmwarePerformanceTableTemplate.Header.Length);
   Status = AcpiTableProtocol->InstallAcpiTable (
                                 AcpiTableProtocol,
                                 &mFirmwarePerformanceTableTemplate,
@@ -389,13 +419,16 @@ InstallFirmwarePerformanceDataTable (
     if (mAcpiBootPerformanceTable != NULL) {
       FreePages (mAcpiBootPerformanceTable, EFI_SIZE_TO_PAGES (BootPerformanceDataSize));
     }
+
     if (mAcpiS3PerformanceTable != NULL) {
       FreePages (mAcpiS3PerformanceTable, EFI_SIZE_TO_PAGES (sizeof (S3_PERFORMANCE_TABLE)));
     }
+
     mAcpiBootPerformanceTable = NULL;
-    mAcpiS3PerformanceTable = NULL;
+    mAcpiS3PerformanceTable   = NULL;
     return Status;
   }
+
   return EFI_SUCCESS;
 }
 
@@ -421,11 +454,11 @@ InstallFirmwarePerformanceDataTable (
 EFI_STATUS
 EFIAPI
 FpdtStatusCodeListenerDxe (
-  IN EFI_STATUS_CODE_TYPE     CodeType,
-  IN EFI_STATUS_CODE_VALUE    Value,
-  IN UINT32                   Instance,
-  IN EFI_GUID                 *CallerId,
-  IN EFI_STATUS_CODE_DATA     *Data
+  IN EFI_STATUS_CODE_TYPE   CodeType,
+  IN EFI_STATUS_CODE_VALUE  Value,
+  IN UINT32                 Instance,
+  IN EFI_GUID               *CallerId,
+  IN EFI_STATUS_CODE_DATA   *Data
   )
 {
   EFI_STATUS  Status;
@@ -502,24 +535,18 @@ FpdtStatusCodeListenerDxe (
     //
     // Dump FPDT Boot Performance record.
     //
-    DEBUG ((EFI_D_INFO, "FPDT: Boot Performance - ResetEnd                = %ld\n", mAcpiBootPerformanceTable->BasicBoot.ResetEnd));
-    DEBUG ((EFI_D_INFO, "FPDT: Boot Performance - OsLoaderLoadImageStart  = 0\n"));
-    DEBUG ((EFI_D_INFO, "FPDT: Boot Performance - OsLoaderStartImageStart = %ld\n", mAcpiBootPerformanceTable->BasicBoot.OsLoaderStartImageStart));
-    DEBUG ((EFI_D_INFO, "FPDT: Boot Performance - ExitBootServicesEntry   = 0\n"));
-    DEBUG ((EFI_D_INFO, "FPDT: Boot Performance - ExitBootServicesExit    = 0\n"));
-  } else if (Value == (EFI_SOFTWARE_DXE_BS_DRIVER | EFI_SW_DXE_BS_PC_READY_TO_BOOT_EVENT)) {
-    if (mAcpiBootPerformanceTable == NULL) {
-      //
-      // ACPI Firmware Performance Data Table not installed yet, install it now.
-      //
-      InstallFirmwarePerformanceDataTable ();
-    }
-  } else if (Data != NULL && CompareGuid (&Data->Type, &gEdkiiFpdtExtendedFirmwarePerformanceGuid)) {
+    DEBUG ((DEBUG_INFO, "FPDT: Boot Performance - ResetEnd                = %ld\n", mAcpiBootPerformanceTable->BasicBoot.ResetEnd));
+    DEBUG ((DEBUG_INFO, "FPDT: Boot Performance - OsLoaderLoadImageStart  = 0\n"));
+    DEBUG ((DEBUG_INFO, "FPDT: Boot Performance - OsLoaderStartImageStart = %ld\n", mAcpiBootPerformanceTable->BasicBoot.OsLoaderStartImageStart));
+    DEBUG ((DEBUG_INFO, "FPDT: Boot Performance - ExitBootServicesEntry   = 0\n"));
+    DEBUG ((DEBUG_INFO, "FPDT: Boot Performance - ExitBootServicesExit    = 0\n"));
+  } else if ((Data != NULL) && CompareGuid (&Data->Type, &gEdkiiFpdtExtendedFirmwarePerformanceGuid)) {
     //
     // Get the Boot performance table and then install it to ACPI table.
     //
     CopyMem (&mReceivedAcpiBootPerformanceTable, Data + 1, Data->Size);
-  } else if (Data != NULL && CompareGuid (&Data->Type, &gEfiFirmwarePerformanceGuid)) {
+    InstallFirmwarePerformanceDataTable ();
+  } else if ((Data != NULL) && CompareGuid (&Data->Type, &gEfiFirmwarePerformanceGuid)) {
     DEBUG ((DEBUG_ERROR, "FpdtStatusCodeListenerDxe: Performance data reported through gEfiFirmwarePerformanceGuid will not be collected by FirmwarePerformanceDataTableDxe\n"));
     Status = EFI_UNSUPPORTED;
   } else {
@@ -532,6 +559,33 @@ FpdtStatusCodeListenerDxe (
   return Status;
 }
 
+/**
+  Notify function for event EndOfDxe.
+
+  This is used to install ACPI Firmware Performance Data Table for basic boot records.
+
+  @param[in]  Event   The Event that is being processed.
+  @param[in]  Context The Event Context.
+
+**/
+VOID
+EFIAPI
+FpdtEndOfDxeEventNotify (
+  IN EFI_EVENT  Event,
+  IN VOID       *Context
+  )
+{
+  //
+  // When performance is enabled, the FPDT will be installed when DxeCorePerformanceLib report the data to FimwarePerformanceDxe.
+  // This is used to install the FPDT for the basic boot recods when performance infrastructure is not enabled.
+  //
+  if ((PcdGet8 (PcdPerformanceLibraryPropertyMask) & PERFORMANCE_LIBRARY_PROPERTY_MEASUREMENT_ENABLED) != 0) {
+    return;
+  }
+
+  ASSERT (mReceivedAcpiBootPerformanceTable == NULL);
+  InstallFirmwarePerformanceDataTable ();
+}
 
 /**
   Notify function for event EVT_SIGNAL_EXIT_BOOT_SERVICES. This is used to record
@@ -544,8 +598,8 @@ FpdtStatusCodeListenerDxe (
 VOID
 EFIAPI
 FpdtExitBootServicesEventNotify (
-  IN EFI_EVENT        Event,
-  IN VOID             *Context
+  IN EFI_EVENT  Event,
+  IN VOID       *Context
   )
 {
   if (!mDxeCoreReportStatusCodeEnable) {
@@ -560,7 +614,7 @@ FpdtExitBootServicesEventNotify (
     //
     // Firmware Performance Data Table not installed, do nothing.
     //
-    return ;
+    return;
   }
 
   //
@@ -571,10 +625,10 @@ FpdtExitBootServicesEventNotify (
   //
   // Dump FPDT Boot Performance record.
   //
-  DEBUG ((EFI_D_INFO, "FPDT: Boot Performance - ResetEnd                = %ld\n", mAcpiBootPerformanceTable->BasicBoot.ResetEnd));
-  DEBUG ((EFI_D_INFO, "FPDT: Boot Performance - OsLoaderLoadImageStart  = %ld\n", mAcpiBootPerformanceTable->BasicBoot.OsLoaderLoadImageStart));
-  DEBUG ((EFI_D_INFO, "FPDT: Boot Performance - OsLoaderStartImageStart = %ld\n", mAcpiBootPerformanceTable->BasicBoot.OsLoaderStartImageStart));
-  DEBUG ((EFI_D_INFO, "FPDT: Boot Performance - ExitBootServicesEntry   = %ld\n", mAcpiBootPerformanceTable->BasicBoot.ExitBootServicesEntry));
+  DEBUG ((DEBUG_INFO, "FPDT: Boot Performance - ResetEnd                = %ld\n", mAcpiBootPerformanceTable->BasicBoot.ResetEnd));
+  DEBUG ((DEBUG_INFO, "FPDT: Boot Performance - OsLoaderLoadImageStart  = %ld\n", mAcpiBootPerformanceTable->BasicBoot.OsLoaderLoadImageStart));
+  DEBUG ((DEBUG_INFO, "FPDT: Boot Performance - OsLoaderStartImageStart = %ld\n", mAcpiBootPerformanceTable->BasicBoot.OsLoaderStartImageStart));
+  DEBUG ((DEBUG_INFO, "FPDT: Boot Performance - ExitBootServicesEntry   = %ld\n", mAcpiBootPerformanceTable->BasicBoot.ExitBootServicesEntry));
   //
   // ExitBootServicesExit will be updated later, so don't dump it here.
   //
@@ -593,15 +647,16 @@ FpdtExitBootServicesEventNotify (
 EFI_STATUS
 EFIAPI
 FirmwarePerformanceDxeEntryPoint (
-  IN EFI_HANDLE          ImageHandle,
-  IN EFI_SYSTEM_TABLE    *SystemTable
+  IN EFI_HANDLE        ImageHandle,
+  IN EFI_SYSTEM_TABLE  *SystemTable
   )
 {
-  EFI_STATUS               Status;
-  EFI_HOB_GUID_TYPE        *GuidHob;
-  FIRMWARE_SEC_PERFORMANCE *Performance;
-  VOID                     *Registration;
-  UINT64                   OemTableId;
+  EFI_STATUS                Status;
+  EFI_HOB_GUID_TYPE         *GuidHob;
+  FIRMWARE_SEC_PERFORMANCE  *Performance;
+  VOID                      *Registration;
+  UINT64                    OemTableId;
+  EFI_EVENT                 EndOfDxeEvent;
 
   CopyMem (
     mFirmwarePerformanceTableTemplate.Header.OemId,
@@ -610,14 +665,14 @@ FirmwarePerformanceDxeEntryPoint (
     );
   OemTableId = PcdGet64 (PcdAcpiDefaultOemTableId);
   CopyMem (&mFirmwarePerformanceTableTemplate.Header.OemTableId, &OemTableId, sizeof (UINT64));
-  mFirmwarePerformanceTableTemplate.Header.OemRevision      = PcdGet32 (PcdAcpiDefaultOemRevision);
-  mFirmwarePerformanceTableTemplate.Header.CreatorId        = PcdGet32 (PcdAcpiDefaultCreatorId);
-  mFirmwarePerformanceTableTemplate.Header.CreatorRevision  = PcdGet32 (PcdAcpiDefaultCreatorRevision);
+  mFirmwarePerformanceTableTemplate.Header.OemRevision     = PcdGet32 (PcdAcpiDefaultOemRevision);
+  mFirmwarePerformanceTableTemplate.Header.CreatorId       = PcdGet32 (PcdAcpiDefaultCreatorId);
+  mFirmwarePerformanceTableTemplate.Header.CreatorRevision = PcdGet32 (PcdAcpiDefaultCreatorRevision);
 
   //
   // Get Report Status Code Handler Protocol.
   //
-  Status = gBS->LocateProtocol (&gEfiRscHandlerProtocolGuid, NULL, (VOID **) &mRscHandlerProtocol);
+  Status = gBS->LocateProtocol (&gEfiRscHandlerProtocolGuid, NULL, (VOID **)&mRscHandlerProtocol);
   ASSERT_EFI_ERROR (Status);
 
   //
@@ -626,6 +681,19 @@ FirmwarePerformanceDxeEntryPoint (
   Status = mRscHandlerProtocol->Register (FpdtStatusCodeListenerDxe, TPL_HIGH_LEVEL);
   ASSERT_EFI_ERROR (Status);
 
+  //
+  // Register the notify function to install FPDT at EndOfDxe.
+  //
+  Status = gBS->CreateEventEx (
+                  EVT_NOTIFY_SIGNAL,
+                  TPL_NOTIFY,
+                  FpdtEndOfDxeEventNotify,
+                  NULL,
+                  &gEfiEndOfDxeEventGroupGuid,
+                  &EndOfDxeEvent
+                  );
+  ASSERT_EFI_ERROR (Status);
+
   //
   // Register the notify function to update FPDT on ExitBootServices Event.
   //
@@ -644,7 +712,7 @@ FirmwarePerformanceDxeEntryPoint (
   //
   GuidHob = GetFirstGuidHob (&gEfiFirmwarePerformanceGuid);
   if (GuidHob != NULL) {
-    Performance = (FIRMWARE_SEC_PERFORMANCE *) GET_GUID_HOB_DATA (GuidHob);
+    Performance                                      = (FIRMWARE_SEC_PERFORMANCE *)GET_GUID_HOB_DATA (GuidHob);
     mBootPerformanceTableTemplate.BasicBoot.ResetEnd = Performance->ResetEnd;
   } else {
     //
