@@ -43,33 +43,26 @@
   Caution: This module requires additional review when modified.
   This driver need to make sure the CommBuffer is not in the SMRAM range.
 
-Copyright (c) 2010 - 2018, Intel Corporation. All rights reserved.<BR>
-This program and the accompanying materials
-are licensed and made available under the terms and conditions of the BSD License
-which accompanies this distribution.  The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php
-
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+Copyright (c) 2010 - 2024, Intel Corporation. All rights reserved.<BR>
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
-#include <PiSmm.h>
-#include <Library/SmmServicesTableLib.h>
-#include <Library/SmmMemLib.h>
+#include <PiMm.h>
+#include <Library/MmServicesTableLib.h>
 #include <Library/BaseLib.h>
 #include <Protocol/SmmSwapAddressRange.h>
 #include "FaultTolerantWrite.h"
 #include "FaultTolerantWriteSmmCommon.h"
-#include <Protocol/SmmEndOfDxe.h>
+#include <Protocol/MmEndOfDxe.h>
 
-EFI_EVENT                                 mFvbRegistration = NULL;
-EFI_FTW_DEVICE                            *mFtwDevice      = NULL;
+VOID            *mFvbRegistration = NULL;
+EFI_FTW_DEVICE  *mFtwDevice       = NULL;
 
 ///
 /// The flag to indicate whether the platform has left the DXE phase of execution.
 ///
-BOOLEAN                                   mEndOfDxe = FALSE;
+BOOLEAN  mEndOfDxe = FALSE;
 
 /**
   Retrieve the SMM FVB protocol interface by HANDLE.
@@ -92,10 +85,10 @@ FtwGetFvbByHandle (
   //
   // To get the SMM FVB protocol interface on the handle
   //
-  return gSmst->SmmHandleProtocol (
+  return gMmst->MmHandleProtocol (
                   FvBlockHandle,
                   &gEfiSmmFirmwareVolumeBlockProtocolGuid,
-                  (VOID **) FvBlock
+                  (VOID **)FvBlock
                   );
 }
 
@@ -111,15 +104,15 @@ FtwGetFvbByHandle (
 **/
 EFI_STATUS
 FtwGetSarProtocol (
-  OUT VOID                                **SarProtocol
+  OUT VOID  **SarProtocol
   )
 {
-  EFI_STATUS                              Status;
+  EFI_STATUS  Status;
 
   //
   // Locate Smm Swap Address Range protocol
   //
-  Status = gSmst->SmmLocateProtocol (
+  Status = gMmst->MmLocateProtocol (
                     &gEfiSmmSwapAddressRangeProtocolGuid,
                     NULL,
                     SarProtocol
@@ -144,12 +137,12 @@ FtwGetSarProtocol (
 **/
 EFI_STATUS
 GetFvbCountAndBuffer (
-  OUT UINTN                               *NumberHandles,
-  OUT EFI_HANDLE                          **Buffer
+  OUT UINTN       *NumberHandles,
+  OUT EFI_HANDLE  **Buffer
   )
 {
-  EFI_STATUS                              Status;
-  UINTN                                   BufferSize;
+  EFI_STATUS  Status;
+  UINTN       BufferSize;
 
   if ((NumberHandles == NULL) || (Buffer == NULL)) {
     return EFI_INVALID_PARAMETER;
@@ -158,14 +151,14 @@ GetFvbCountAndBuffer (
   BufferSize     = 0;
   *NumberHandles = 0;
   *Buffer        = NULL;
-  Status = gSmst->SmmLocateHandle (
-                    ByProtocol,
-                    &gEfiSmmFirmwareVolumeBlockProtocolGuid,
-                    NULL,
-                    &BufferSize,
-                    *Buffer
-                    );
-  if (EFI_ERROR(Status) && Status != EFI_BUFFER_TOO_SMALL) {
+  Status         = gMmst->MmLocateHandle (
+                            ByProtocol,
+                            &gEfiSmmFirmwareVolumeBlockProtocolGuid,
+                            NULL,
+                            &BufferSize,
+                            *Buffer
+                            );
+  if (EFI_ERROR (Status) && (Status != EFI_BUFFER_TOO_SMALL)) {
     return EFI_NOT_FOUND;
   }
 
@@ -174,7 +167,7 @@ GetFvbCountAndBuffer (
     return EFI_OUT_OF_RESOURCES;
   }
 
-  Status = gSmst->SmmLocateHandle (
+  Status = gMmst->MmLocateHandle (
                     ByProtocol,
                     &gEfiSmmFirmwareVolumeBlockProtocolGuid,
                     NULL,
@@ -182,8 +175,8 @@ GetFvbCountAndBuffer (
                     *Buffer
                     );
 
-  *NumberHandles = BufferSize / sizeof(EFI_HANDLE);
-  if (EFI_ERROR(Status)) {
+  *NumberHandles = BufferSize / sizeof (EFI_HANDLE);
+  if (EFI_ERROR (Status)) {
     *NumberHandles = 0;
     FreePool (*Buffer);
     *Buffer = NULL;
@@ -192,7 +185,6 @@ GetFvbCountAndBuffer (
   return Status;
 }
 
-
 /**
   Get the handle of the SMM FVB protocol by the FVB base address and attributes.
 
@@ -206,9 +198,9 @@ GetFvbCountAndBuffer (
 **/
 EFI_STATUS
 GetFvbByAddressAndAttribute (
-  IN  EFI_PHYSICAL_ADDRESS            Address,
-  IN  EFI_FVB_ATTRIBUTES_2            Attributes,
-  OUT EFI_HANDLE                      *SmmFvbHandle
+  IN  EFI_PHYSICAL_ADDRESS  Address,
+  IN  EFI_FVB_ATTRIBUTES_2  Attributes,
+  OUT EFI_HANDLE            *SmmFvbHandle
   )
 {
   EFI_STATUS                          Status;
@@ -237,6 +229,7 @@ GetFvbByAddressAndAttribute (
     if (EFI_ERROR (Status)) {
       break;
     }
+
     //
     // Compare the address.
     //
@@ -244,8 +237,9 @@ GetFvbByAddressAndAttribute (
     if (EFI_ERROR (Status)) {
       continue;
     }
+
     if (Address != FvbBaseAddress) {
-     continue;
+      continue;
     }
 
     //
@@ -255,8 +249,9 @@ GetFvbByAddressAndAttribute (
     if (EFI_ERROR (Status)) {
       continue;
     }
+
     if (Attributes != FvbAttributes) {
-     continue;
+      continue;
     }
 
     //
@@ -300,44 +295,45 @@ GetFvbByAddressAndAttribute (
 EFI_STATUS
 EFIAPI
 SmmFaultTolerantWriteHandler (
-  IN     EFI_HANDLE                                DispatchHandle,
-  IN     CONST VOID                                *RegisterContext,
-  IN OUT VOID                                      *CommBuffer,
-  IN OUT UINTN                                     *CommBufferSize
+  IN     EFI_HANDLE  DispatchHandle,
+  IN     CONST VOID  *RegisterContext,
+  IN OUT VOID        *CommBuffer,
+  IN OUT UINTN       *CommBufferSize
   )
 {
-  EFI_STATUS                                       Status;
-  SMM_FTW_COMMUNICATE_FUNCTION_HEADER              *SmmFtwFunctionHeader;
-  SMM_FTW_GET_MAX_BLOCK_SIZE_HEADER                *SmmGetMaxBlockSizeHeader;
-  SMM_FTW_ALLOCATE_HEADER                          *SmmFtwAllocateHeader;
-  SMM_FTW_WRITE_HEADER                             *SmmFtwWriteHeader;
-  SMM_FTW_RESTART_HEADER                           *SmmFtwRestartHeader;
-  SMM_FTW_GET_LAST_WRITE_HEADER                    *SmmFtwGetLastWriteHeader;
-  VOID                                             *PrivateData;
-  EFI_HANDLE                                       SmmFvbHandle;
-  UINTN                                            InfoSize;
-  UINTN                                            CommBufferPayloadSize;
-  UINTN                                            PrivateDataSize;
-  UINTN                                            Length;
-  UINTN                                            TempCommBufferSize;
+  EFI_STATUS                           Status;
+  SMM_FTW_COMMUNICATE_FUNCTION_HEADER  *SmmFtwFunctionHeader;
+  SMM_FTW_GET_MAX_BLOCK_SIZE_HEADER    *SmmGetMaxBlockSizeHeader;
+  SMM_FTW_ALLOCATE_HEADER              *SmmFtwAllocateHeader;
+  SMM_FTW_WRITE_HEADER                 *SmmFtwWriteHeader;
+  SMM_FTW_RESTART_HEADER               *SmmFtwRestartHeader;
+  SMM_FTW_GET_LAST_WRITE_HEADER        *SmmFtwGetLastWriteHeader;
+  VOID                                 *PrivateData;
+  EFI_HANDLE                           SmmFvbHandle;
+  UINTN                                InfoSize;
+  UINTN                                CommBufferPayloadSize;
+  UINTN                                PrivateDataSize;
+  UINTN                                Length;
+  UINTN                                TempCommBufferSize;
 
   //
   // If input is invalid, stop processing this SMI
   //
-  if (CommBuffer == NULL || CommBufferSize == NULL) {
+  if ((CommBuffer == NULL) || (CommBufferSize == NULL)) {
     return EFI_SUCCESS;
   }
 
   TempCommBufferSize = *CommBufferSize;
 
   if (TempCommBufferSize < SMM_FTW_COMMUNICATE_HEADER_SIZE) {
-    DEBUG ((EFI_D_ERROR, "SmmFtwHandler: SMM communication buffer size invalid!\n"));
+    DEBUG ((DEBUG_ERROR, "SmmFtwHandler: SMM communication buffer size invalid!\n"));
     return EFI_SUCCESS;
   }
+
   CommBufferPayloadSize = TempCommBufferSize - SMM_FTW_COMMUNICATE_HEADER_SIZE;
 
-  if (!SmmIsBufferOutsideSmmValid ((UINTN)CommBuffer, TempCommBufferSize)) {
-    DEBUG ((EFI_D_ERROR, "SmmFtwHandler: SMM communication buffer in SMRAM or overflow!\n"));
+  if (!FtwSmmIsPrimaryBufferValid ((UINTN)CommBuffer, TempCommBufferSize)) {
+    DEBUG ((DEBUG_ERROR, "SmmFtwHandler: SMM Primary(communication buffer) is not valid!\n"));
     return EFI_SUCCESS;
   }
 
@@ -347,7 +343,7 @@ SmmFaultTolerantWriteHandler (
     //
     // It will be not safe to expose the operations after End Of Dxe.
     //
-    DEBUG ((EFI_D_ERROR, "SmmFtwHandler: Not safe to do the operation: %x after End Of Dxe, so access denied!\n", SmmFtwFunctionHeader->Function));
+    DEBUG ((DEBUG_ERROR, "SmmFtwHandler: Not safe to do the operation: %x after End Of Dxe, so access denied!\n", SmmFtwFunctionHeader->Function));
     SmmFtwFunctionHeader->ReturnStatus = EFI_ACCESS_DENIED;
     return EFI_SUCCESS;
   }
@@ -355,10 +351,11 @@ SmmFaultTolerantWriteHandler (
   switch (SmmFtwFunctionHeader->Function) {
     case FTW_FUNCTION_GET_MAX_BLOCK_SIZE:
       if (CommBufferPayloadSize < sizeof (SMM_FTW_GET_MAX_BLOCK_SIZE_HEADER)) {
-        DEBUG ((EFI_D_ERROR, "GetMaxBlockSize: SMM communication buffer size invalid!\n"));
+        DEBUG ((DEBUG_ERROR, "GetMaxBlockSize: SMM communication buffer size invalid!\n"));
         return EFI_SUCCESS;
       }
-      SmmGetMaxBlockSizeHeader = (SMM_FTW_GET_MAX_BLOCK_SIZE_HEADER *) SmmFtwFunctionHeader->Data;
+
+      SmmGetMaxBlockSizeHeader = (SMM_FTW_GET_MAX_BLOCK_SIZE_HEADER *)SmmFtwFunctionHeader->Data;
 
       Status = FtwGetMaxBlockSize (
                  &mFtwDevice->FtwInstance,
@@ -368,41 +365,45 @@ SmmFaultTolerantWriteHandler (
 
     case FTW_FUNCTION_ALLOCATE:
       if (CommBufferPayloadSize < sizeof (SMM_FTW_ALLOCATE_HEADER)) {
-        DEBUG ((EFI_D_ERROR, "Allocate: SMM communication buffer size invalid!\n"));
+        DEBUG ((DEBUG_ERROR, "Allocate: SMM communication buffer size invalid!\n"));
         return EFI_SUCCESS;
       }
-      SmmFtwAllocateHeader = (SMM_FTW_ALLOCATE_HEADER *) SmmFtwFunctionHeader->Data;
-      Status = FtwAllocate (
-                 &mFtwDevice->FtwInstance,
-                 &SmmFtwAllocateHeader->CallerId,
-                 SmmFtwAllocateHeader->PrivateDataSize,
-                 SmmFtwAllocateHeader->NumberOfWrites
-                 );
+
+      SmmFtwAllocateHeader = (SMM_FTW_ALLOCATE_HEADER *)SmmFtwFunctionHeader->Data;
+      Status               = FtwAllocate (
+                               &mFtwDevice->FtwInstance,
+                               &SmmFtwAllocateHeader->CallerId,
+                               SmmFtwAllocateHeader->PrivateDataSize,
+                               SmmFtwAllocateHeader->NumberOfWrites
+                               );
       break;
 
     case FTW_FUNCTION_WRITE:
       if (CommBufferPayloadSize < OFFSET_OF (SMM_FTW_WRITE_HEADER, Data)) {
-        DEBUG ((EFI_D_ERROR, "Write: SMM communication buffer size invalid!\n"));
+        DEBUG ((DEBUG_ERROR, "Write: SMM communication buffer size invalid!\n"));
         return EFI_SUCCESS;
       }
-      SmmFtwWriteHeader = (SMM_FTW_WRITE_HEADER *) SmmFtwFunctionHeader->Data;
-      Length = SmmFtwWriteHeader->Length;
-      PrivateDataSize = SmmFtwWriteHeader->PrivateDataSize;
+
+      SmmFtwWriteHeader = (SMM_FTW_WRITE_HEADER *)SmmFtwFunctionHeader->Data;
+      Length            = SmmFtwWriteHeader->Length;
+      PrivateDataSize   = SmmFtwWriteHeader->PrivateDataSize;
       if (((UINTN)(~0) - Length < OFFSET_OF (SMM_FTW_WRITE_HEADER, Data)) ||
-        ((UINTN)(~0) - PrivateDataSize < OFFSET_OF (SMM_FTW_WRITE_HEADER, Data) + Length)) {
+          ((UINTN)(~0) - PrivateDataSize < OFFSET_OF (SMM_FTW_WRITE_HEADER, Data) + Length))
+      {
         //
         // Prevent InfoSize overflow
         //
         Status = EFI_ACCESS_DENIED;
         break;
       }
+
       InfoSize = OFFSET_OF (SMM_FTW_WRITE_HEADER, Data) + Length + PrivateDataSize;
 
       //
       // SMRAM range check already covered before
       //
       if (InfoSize > CommBufferPayloadSize) {
-        DEBUG ((EFI_D_ERROR, "Write: Data size exceed communication buffer size limit!\n"));
+        DEBUG ((DEBUG_ERROR, "Write: Data size exceed communication buffer size limit!\n"));
         Status = EFI_ACCESS_DENIED;
         break;
       }
@@ -412,6 +413,7 @@ SmmFaultTolerantWriteHandler (
       } else {
         PrivateData = (VOID *)&SmmFtwWriteHeader->Data[Length];
       }
+
       Status = GetFvbByAddressAndAttribute (
                  SmmFtwWriteHeader->FvbBaseAddress,
                  SmmFtwWriteHeader->FvbAttributes,
@@ -419,12 +421,12 @@ SmmFaultTolerantWriteHandler (
                  );
       if (!EFI_ERROR (Status)) {
         //
-        // The AsmLfence() call here is to ensure the previous range/content
-        // checks for the CommBuffer have been completed before calling into
-        // FtwWrite().
+        // The SpeculationBarrier() call here is to ensure the previous
+        // range/content checks for the CommBuffer have been completed before
+        // calling into FtwWrite().
         //
-        AsmLfence ();
-        Status = FtwWrite(
+        SpeculationBarrier ();
+        Status = FtwWrite (
                    &mFtwDevice->FtwInstance,
                    SmmFtwWriteHeader->Lba,
                    SmmFtwWriteHeader->Offset,
@@ -434,22 +436,25 @@ SmmFaultTolerantWriteHandler (
                    SmmFtwWriteHeader->Data
                    );
       }
+
       break;
 
     case FTW_FUNCTION_RESTART:
       if (CommBufferPayloadSize < sizeof (SMM_FTW_RESTART_HEADER)) {
-        DEBUG ((EFI_D_ERROR, "Restart: SMM communication buffer size invalid!\n"));
+        DEBUG ((DEBUG_ERROR, "Restart: SMM communication buffer size invalid!\n"));
         return EFI_SUCCESS;
       }
-      SmmFtwRestartHeader = (SMM_FTW_RESTART_HEADER *) SmmFtwFunctionHeader->Data;
-      Status = GetFvbByAddressAndAttribute (
-                 SmmFtwRestartHeader->FvbBaseAddress,
-                 SmmFtwRestartHeader->FvbAttributes,
-                 &SmmFvbHandle
-                 );
+
+      SmmFtwRestartHeader = (SMM_FTW_RESTART_HEADER *)SmmFtwFunctionHeader->Data;
+      Status              = GetFvbByAddressAndAttribute (
+                              SmmFtwRestartHeader->FvbBaseAddress,
+                              SmmFtwRestartHeader->FvbAttributes,
+                              &SmmFvbHandle
+                              );
       if (!EFI_ERROR (Status)) {
         Status = FtwRestart (&mFtwDevice->FtwInstance, SmmFvbHandle);
       }
+
       break;
 
     case FTW_FUNCTION_ABORT:
@@ -458,25 +463,27 @@ SmmFaultTolerantWriteHandler (
 
     case FTW_FUNCTION_GET_LAST_WRITE:
       if (CommBufferPayloadSize < OFFSET_OF (SMM_FTW_GET_LAST_WRITE_HEADER, Data)) {
-        DEBUG ((EFI_D_ERROR, "GetLastWrite: SMM communication buffer size invalid!\n"));
+        DEBUG ((DEBUG_ERROR, "GetLastWrite: SMM communication buffer size invalid!\n"));
         return EFI_SUCCESS;
       }
-      SmmFtwGetLastWriteHeader = (SMM_FTW_GET_LAST_WRITE_HEADER *) SmmFtwFunctionHeader->Data;
-      PrivateDataSize = SmmFtwGetLastWriteHeader->PrivateDataSize;
-      if ((UINTN)(~0) - PrivateDataSize < OFFSET_OF (SMM_FTW_GET_LAST_WRITE_HEADER, Data)){
+
+      SmmFtwGetLastWriteHeader = (SMM_FTW_GET_LAST_WRITE_HEADER *)SmmFtwFunctionHeader->Data;
+      PrivateDataSize          = SmmFtwGetLastWriteHeader->PrivateDataSize;
+      if ((UINTN)(~0) - PrivateDataSize < OFFSET_OF (SMM_FTW_GET_LAST_WRITE_HEADER, Data)) {
         //
         // Prevent InfoSize overflow
         //
         Status = EFI_ACCESS_DENIED;
         break;
       }
+
       InfoSize = OFFSET_OF (SMM_FTW_GET_LAST_WRITE_HEADER, Data) + PrivateDataSize;
 
       //
       // SMRAM range check already covered before
       //
       if (InfoSize > CommBufferPayloadSize) {
-        DEBUG ((EFI_D_ERROR, "Data size exceed communication buffer size limit!\n"));
+        DEBUG ((DEBUG_ERROR, "Data size exceed communication buffer size limit!\n"));
         Status = EFI_ACCESS_DENIED;
         break;
       }
@@ -503,7 +510,6 @@ SmmFaultTolerantWriteHandler (
   return EFI_SUCCESS;
 }
 
-
 /**
   SMM Firmware Volume Block Protocol notification event handler.
 
@@ -517,24 +523,23 @@ SmmFaultTolerantWriteHandler (
 EFI_STATUS
 EFIAPI
 FvbNotificationEvent (
-  IN CONST EFI_GUID                       *Protocol,
-  IN VOID                                 *Interface,
-  IN EFI_HANDLE                           Handle
+  IN CONST EFI_GUID  *Protocol,
+  IN VOID            *Interface,
+  IN EFI_HANDLE      Handle
   )
 {
-  EFI_STATUS                              Status;
-  EFI_SMM_FAULT_TOLERANT_WRITE_PROTOCOL   *FtwProtocol;
-  EFI_HANDLE                              SmmFtwHandle;
-  EFI_HANDLE                              FtwHandle;
+  EFI_STATUS                             Status;
+  EFI_SMM_FAULT_TOLERANT_WRITE_PROTOCOL  *FtwProtocol;
+  EFI_HANDLE                             SmmFtwHandle;
 
   //
   // Just return to avoid install SMM FaultTolerantWriteProtocol again
   // if SMM Fault Tolerant Write protocol had been installed.
   //
-  Status = gSmst->SmmLocateProtocol (
+  Status = gMmst->MmLocateProtocol (
                     &gEfiSmmFaultTolerantWriteProtocolGuid,
                     NULL,
-                    (VOID **) &FtwProtocol
+                    (VOID **)&FtwProtocol
                     );
   if (!EFI_ERROR (Status)) {
     return EFI_SUCCESS;
@@ -544,14 +549,14 @@ FvbNotificationEvent (
   // Found proper FVB protocol and initialize FtwDevice for protocol installation
   //
   Status = InitFtwProtocol (mFtwDevice);
-  if (EFI_ERROR(Status)) {
+  if (EFI_ERROR (Status)) {
     return Status;
   }
 
   //
   // Install protocol interface
   //
-  Status = gSmst->SmmInstallProtocolInterface (
+  Status = gMmst->MmInstallProtocolInterface (
                     &mFtwDevice->Handle,
                     &gEfiSmmFaultTolerantWriteProtocolGuid,
                     EFI_NATIVE_INTERFACE,
@@ -562,20 +567,13 @@ FvbNotificationEvent (
   ///
   /// Register SMM FTW SMI handler
   ///
-  Status = gSmst->SmiHandlerRegister (SmmFaultTolerantWriteHandler, &gEfiSmmFaultTolerantWriteProtocolGuid, &SmmFtwHandle);
+  Status = gMmst->MmiHandlerRegister (SmmFaultTolerantWriteHandler, &gEfiSmmFaultTolerantWriteProtocolGuid, &SmmFtwHandle);
   ASSERT_EFI_ERROR (Status);
 
   //
   // Notify the Ftw wrapper driver SMM Ftw is ready
   //
-  FtwHandle = NULL;
-  Status = gBS->InstallProtocolInterface (
-                  &FtwHandle,
-                  &gEfiSmmFaultTolerantWriteProtocolGuid,
-                  EFI_NATIVE_INTERFACE,
-                  NULL
-                  );
-  ASSERT_EFI_ERROR (Status);
+  FtwNotifySmmReady ();
 
   return EFI_SUCCESS;
 }
@@ -592,10 +590,10 @@ FvbNotificationEvent (
 **/
 EFI_STATUS
 EFIAPI
-SmmEndOfDxeCallback (
-  IN CONST EFI_GUID                       *Protocol,
-  IN VOID                                 *Interface,
-  IN EFI_HANDLE                           Handle
+MmEndOfDxeCallback (
+  IN CONST EFI_GUID  *Protocol,
+  IN VOID            *Interface,
+  IN EFI_HANDLE      Handle
   )
 {
   mEndOfDxe = TRUE;
@@ -603,48 +601,42 @@ SmmEndOfDxeCallback (
 }
 
 /**
-  This function is the entry point of the Fault Tolerant Write driver.
-
-  @param[in] ImageHandle        A handle for the image that is initializing this driver
-  @param[in] SystemTable        A pointer to the EFI system table
+  Shared entry point of the module
 
   @retval EFI_SUCCESS           The initialization finished successfully.
   @retval EFI_OUT_OF_RESOURCES  Allocate memory error
   @retval EFI_INVALID_PARAMETER Workspace or Spare block does not exist
-
 **/
 EFI_STATUS
-EFIAPI
-SmmFaultTolerantWriteInitialize (
-  IN EFI_HANDLE                           ImageHandle,
-  IN EFI_SYSTEM_TABLE                     *SystemTable
+MmFaultTolerantWriteInitialize (
+  VOID
   )
 {
-  EFI_STATUS                              Status;
-  VOID                                    *SmmEndOfDxeRegistration;
+  EFI_STATUS  Status;
+  VOID        *MmEndOfDxeRegistration;
 
   //
   // Allocate private data structure for SMM FTW protocol and do some initialization
   //
   Status = InitFtwDevice (&mFtwDevice);
-  if (EFI_ERROR(Status)) {
+  if (EFI_ERROR (Status)) {
     return Status;
   }
 
   //
   // Register EFI_SMM_END_OF_DXE_PROTOCOL_GUID notify function.
   //
-  Status = gSmst->SmmRegisterProtocolNotify (
-                    &gEfiSmmEndOfDxeProtocolGuid,
-                    SmmEndOfDxeCallback,
-                    &SmmEndOfDxeRegistration
+  Status = gMmst->MmRegisterProtocolNotify (
+                    &gEfiMmEndOfDxeProtocolGuid,
+                    MmEndOfDxeCallback,
+                    &MmEndOfDxeRegistration
                     );
   ASSERT_EFI_ERROR (Status);
 
   //
   // Register FvbNotificationEvent () notify function.
   //
-  Status = gSmst->SmmRegisterProtocolNotify (
+  Status = gMmst->MmRegisterProtocolNotify (
                     &gEfiSmmFirmwareVolumeBlockProtocolGuid,
                     FvbNotificationEvent,
                     &mFvbRegistration
