@@ -7,7 +7,10 @@
 **/
 #include "BaseLibInternals.h"
 
-extern VOID RiscVDisableSupervisorModeInterrupts (VOID);
+extern VOID
+RiscVDisableSupervisorModeInterrupts (
+  VOID
+  );
 
 /**
   Disables CPU interrupts.
@@ -21,4 +24,3 @@ DisableInterrupts (
 {
   RiscVDisableSupervisorModeInterrupts ();
 }
-
