@@ -406,9 +406,14 @@ StrDecimalToUintn (
   IN      CONST CHAR16  *String
   )
 {
-  UINTN  Result;
+  UINTN          Result;
+  RETURN_STATUS  Status;
+
+  Status = StrDecimalToUintnS (String, (CHAR16 **)NULL, &Result);
+  if (Status == RETURN_INVALID_PARAMETER) {
+    Result = 0;
+  }
 
-  StrDecimalToUintnS (String, (CHAR16 **)NULL, &Result);
   return Result;
 }
 
@@ -452,9 +457,14 @@ StrDecimalToUint64 (
   IN      CONST CHAR16  *String
   )
 {
-  UINT64  Result;
+  UINT64         Result;
+  RETURN_STATUS  Status;
+
+  Status = StrDecimalToUint64S (String, (CHAR16 **)NULL, &Result);
+  if (Status == RETURN_INVALID_PARAMETER) {
+    Result = 0;
+  }
 
-  StrDecimalToUint64S (String, (CHAR16 **)NULL, &Result);
   return Result;
 }
 
@@ -499,9 +509,14 @@ StrHexToUintn (
   IN      CONST CHAR16  *String
   )
 {
-  UINTN  Result;
+  UINTN          Result;
+  RETURN_STATUS  Status;
+
+  Status = StrHexToUintnS (String, (CHAR16 **)NULL, &Result);
+  if (Status == RETURN_INVALID_PARAMETER) {
+    Result = 0;
+  }
 
-  StrHexToUintnS (String, (CHAR16 **)NULL, &Result);
   return Result;
 }
 
@@ -546,9 +561,14 @@ StrHexToUint64 (
   IN      CONST CHAR16  *String
   )
 {
-  UINT64  Result;
+  UINT64         Result;
+  RETURN_STATUS  Status;
+
+  Status = StrHexToUint64S (String, (CHAR16 **)NULL, &Result);
+  if (Status == RETURN_INVALID_PARAMETER) {
+    Result = 0;
+  }
 
-  StrHexToUint64S (String, (CHAR16 **)NULL, &Result);
   return Result;
 }
 
@@ -987,9 +1007,14 @@ AsciiStrDecimalToUintn (
   IN      CONST CHAR8  *String
   )
 {
-  UINTN  Result;
+  UINTN          Result;
+  RETURN_STATUS  Status;
+
+  Status = AsciiStrDecimalToUintnS (String, (CHAR8 **)NULL, &Result);
+  if (Status == RETURN_INVALID_PARAMETER) {
+    Result = 0;
+  }
 
-  AsciiStrDecimalToUintnS (String, (CHAR8 **)NULL, &Result);
   return Result;
 }
 
@@ -1029,9 +1054,14 @@ AsciiStrDecimalToUint64 (
   IN      CONST CHAR8  *String
   )
 {
-  UINT64  Result;
+  UINT64         Result;
+  RETURN_STATUS  Status;
+
+  Status = AsciiStrDecimalToUint64S (String, (CHAR8 **)NULL, &Result);
+  if (Status == RETURN_INVALID_PARAMETER) {
+    Result = 0;
+  }
 
-  AsciiStrDecimalToUint64S (String, (CHAR8 **)NULL, &Result);
   return Result;
 }
 
@@ -1075,9 +1105,14 @@ AsciiStrHexToUintn (
   IN      CONST CHAR8  *String
   )
 {
-  UINTN  Result;
+  UINTN          Result;
+  RETURN_STATUS  Status;
+
+  Status = AsciiStrHexToUintnS (String, (CHAR8 **)NULL, &Result);
+  if (Status == RETURN_INVALID_PARAMETER) {
+    Result = 0;
+  }
 
-  AsciiStrHexToUintnS (String, (CHAR8 **)NULL, &Result);
   return Result;
 }
 
@@ -1121,9 +1156,14 @@ AsciiStrHexToUint64 (
   IN      CONST CHAR8  *String
   )
 {
-  UINT64  Result;
+  UINT64         Result;
+  RETURN_STATUS  Status;
+
+  Status = AsciiStrHexToUint64S (String, (CHAR8 **)NULL, &Result);
+  if (Status == RETURN_INVALID_PARAMETER) {
+    Result = 0;
+  }
 
-  AsciiStrHexToUint64S (String, (CHAR8 **)NULL, &Result);
   return Result;
 }
 
