@@ -6,17 +6,10 @@
   Library.
 
   Copyright (c) 2006 - 2018, Intel Corporation. All rights reserved.<BR>
-  This program and the accompanying materials
-  are licensed and made available under the terms and conditions of the BSD License
-  which accompanies this distribution.  The full text of the license may be found at
-  http://opensource.org/licenses/bsd-license.php.
-
-  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-  WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+  SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
-
 #include <Base.h>
 
 #include <Library/BaseLib.h>
@@ -25,10 +18,10 @@
 #include <Library/DebugLib.h>
 #include <Library/PcdLib.h>
 
-
 /**
   Assert the validity of a PCI address. A valid PCI address should contain 1's
-  only in the low 28 bits.
+  only in the low 28 bits. PcdPciExpressBaseSize limits the size to the real
+  number of PCI busses in this segment.
 
   @param  A The address to validate.
 
@@ -77,12 +70,30 @@ PciExpressRegisterForRuntimeAccess (
   @return The base address of PCI Express.
 
 **/
-VOID*
+VOID *
 GetPciExpressBaseAddress (
   VOID
   )
 {
-  return (VOID*)(UINTN) PcdGet64 (PcdPciExpressBaseAddress);
+  return (VOID *)(UINTN)PcdGet64 (PcdPciExpressBaseAddress);
+}
+
+/**
+  Gets the size of PCI Express.
+
+  This internal functions retrieves PCI Express Base Size via a PCD entry
+  PcdPciExpressBaseSize.
+
+  @return The base size of PCI Express.
+
+**/
+STATIC
+UINTN
+PcdPciExpressBaseSize (
+  VOID
+  )
+{
+  return (UINTN)PcdGet64 (PcdPciExpressBaseSize);
 }
 
 /**
@@ -97,17 +108,22 @@ GetPciExpressBaseAddress (
   @param  Address The address that encodes the PCI Bus, Device, Function and
                   Register.
 
-  @return The read value from the PCI configuration register.
+  @retval 0xFF  Invalid PCI address.
+  @retval other The read value from the PCI configuration register.
 
 **/
 UINT8
 EFIAPI
 PciExpressRead8 (
-  IN      UINTN                     Address
+  IN      UINTN  Address
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
-  return MmioRead8 ((UINTN) GetPciExpressBaseAddress () + Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT8)-1;
+  }
+
+  return MmioRead8 ((UINTN)GetPciExpressBaseAddress () + Address);
 }
 
 /**
@@ -123,18 +139,23 @@ PciExpressRead8 (
                   Register.
   @param  Value   The value to write.
 
-  @return The value written to the PCI configuration register.
+  @retval 0xFF  Invalid PCI address.
+  @retval other The value written to the PCI configuration register.
 
 **/
 UINT8
 EFIAPI
 PciExpressWrite8 (
-  IN      UINTN                     Address,
-  IN      UINT8                     Value
+  IN      UINTN  Address,
+  IN      UINT8  Value
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
-  return MmioWrite8 ((UINTN) GetPciExpressBaseAddress () + Address, Value);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT8)-1;
+  }
+
+  return MmioWrite8 ((UINTN)GetPciExpressBaseAddress () + Address, Value);
 }
 
 /**
@@ -154,18 +175,23 @@ PciExpressWrite8 (
                   Register.
   @param  OrData  The value to OR with the PCI configuration register.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFF  Invalid PCI address.
+  @retval other The value written to the PCI configuration register.
 
 **/
 UINT8
 EFIAPI
 PciExpressOr8 (
-  IN      UINTN                     Address,
-  IN      UINT8                     OrData
+  IN      UINTN  Address,
+  IN      UINT8  OrData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
-  return MmioOr8 ((UINTN) GetPciExpressBaseAddress () + Address, OrData);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT8)-1;
+  }
+
+  return MmioOr8 ((UINTN)GetPciExpressBaseAddress () + Address, OrData);
 }
 
 /**
@@ -185,18 +211,23 @@ PciExpressOr8 (
                   Register.
   @param  AndData The value to AND with the PCI configuration register.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFF  Invalid PCI address.
+  @retval other The value written back to the PCI configuration register.
 
 **/
 UINT8
 EFIAPI
 PciExpressAnd8 (
-  IN      UINTN                     Address,
-  IN      UINT8                     AndData
+  IN      UINTN  Address,
+  IN      UINT8  AndData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
-  return MmioAnd8 ((UINTN) GetPciExpressBaseAddress () + Address, AndData);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT8)-1;
+  }
+
+  return MmioAnd8 ((UINTN)GetPciExpressBaseAddress () + Address, AndData);
 }
 
 /**
@@ -218,20 +249,25 @@ PciExpressAnd8 (
   @param  AndData The value to AND with the PCI configuration register.
   @param  OrData  The value to OR with the result of the AND operation.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFF  Invalid PCI address.
+  @retval other The value written back to the PCI configuration register.
 
 **/
 UINT8
 EFIAPI
 PciExpressAndThenOr8 (
-  IN      UINTN                     Address,
-  IN      UINT8                     AndData,
-  IN      UINT8                     OrData
+  IN      UINTN  Address,
+  IN      UINT8  AndData,
+  IN      UINT8  OrData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT8)-1;
+  }
+
   return MmioAndThenOr8 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            AndData,
            OrData
            );
@@ -255,20 +291,26 @@ PciExpressAndThenOr8 (
   @param  EndBit    The ordinal of the most significant bit in the bit field.
                     Range 0..7.
 
-  @return The value of the bit field read from the PCI configuration register.
+  @retval 0xFF  Invalid PCI address.
+  @retval other The value of the bit field read from the PCI configuration
+                register.
 
 **/
 UINT8
 EFIAPI
 PciExpressBitFieldRead8 (
-  IN      UINTN                     Address,
-  IN      UINTN                     StartBit,
-  IN      UINTN                     EndBit
+  IN      UINTN  Address,
+  IN      UINTN  StartBit,
+  IN      UINTN  EndBit
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT8)-1;
+  }
+
   return MmioBitFieldRead8 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            StartBit,
            EndBit
            );
@@ -295,21 +337,26 @@ PciExpressBitFieldRead8 (
                     Range 0..7.
   @param  Value     The new value of the bit field.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFF  Invalid PCI address.
+  @retval other The value written back to the PCI configuration register.
 
 **/
 UINT8
 EFIAPI
 PciExpressBitFieldWrite8 (
-  IN      UINTN                     Address,
-  IN      UINTN                     StartBit,
-  IN      UINTN                     EndBit,
-  IN      UINT8                     Value
+  IN      UINTN  Address,
+  IN      UINTN  StartBit,
+  IN      UINTN  EndBit,
+  IN      UINT8  Value
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT8)-1;
+  }
+
   return MmioBitFieldWrite8 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            StartBit,
            EndBit,
            Value
@@ -340,21 +387,26 @@ PciExpressBitFieldWrite8 (
                     Range 0..7.
   @param  OrData    The value to OR with the PCI configuration register.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFF  Invalid PCI address.
+  @retval other The value written back to the PCI configuration register.
 
 **/
 UINT8
 EFIAPI
 PciExpressBitFieldOr8 (
-  IN      UINTN                     Address,
-  IN      UINTN                     StartBit,
-  IN      UINTN                     EndBit,
-  IN      UINT8                     OrData
+  IN      UINTN  Address,
+  IN      UINTN  StartBit,
+  IN      UINTN  EndBit,
+  IN      UINT8  OrData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT8)-1;
+  }
+
   return MmioBitFieldOr8 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            StartBit,
            EndBit,
            OrData
@@ -385,21 +437,26 @@ PciExpressBitFieldOr8 (
                     Range 0..7.
   @param  AndData   The value to AND with the PCI configuration register.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFF  Invalid PCI address.
+  @retval other The value written back to the PCI configuration register.
 
 **/
 UINT8
 EFIAPI
 PciExpressBitFieldAnd8 (
-  IN      UINTN                     Address,
-  IN      UINTN                     StartBit,
-  IN      UINTN                     EndBit,
-  IN      UINT8                     AndData
+  IN      UINTN  Address,
+  IN      UINTN  StartBit,
+  IN      UINTN  EndBit,
+  IN      UINT8  AndData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT8)-1;
+  }
+
   return MmioBitFieldAnd8 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            StartBit,
            EndBit,
            AndData
@@ -434,22 +491,27 @@ PciExpressBitFieldAnd8 (
   @param  AndData   The value to AND with the PCI configuration register.
   @param  OrData    The value to OR with the result of the AND operation.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFF  Invalid PCI address.
+  @retval other The value written back to the PCI configuration register.
 
 **/
 UINT8
 EFIAPI
 PciExpressBitFieldAndThenOr8 (
-  IN      UINTN                     Address,
-  IN      UINTN                     StartBit,
-  IN      UINTN                     EndBit,
-  IN      UINT8                     AndData,
-  IN      UINT8                     OrData
+  IN      UINTN  Address,
+  IN      UINTN  StartBit,
+  IN      UINTN  EndBit,
+  IN      UINT8  AndData,
+  IN      UINT8  OrData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT8)-1;
+  }
+
   return MmioBitFieldAndThenOr8 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            StartBit,
            EndBit,
            AndData,
@@ -470,17 +532,22 @@ PciExpressBitFieldAndThenOr8 (
   @param  Address The address that encodes the PCI Bus, Device, Function and
                   Register.
 
-  @return The read value from the PCI configuration register.
+  @retval 0xFF  Invalid PCI address.
+  @retval other The read value from the PCI configuration register.
 
 **/
 UINT16
 EFIAPI
 PciExpressRead16 (
-  IN      UINTN                     Address
+  IN      UINTN  Address
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
-  return MmioRead16 ((UINTN) GetPciExpressBaseAddress () + Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT16)-1;
+  }
+
+  return MmioRead16 ((UINTN)GetPciExpressBaseAddress () + Address);
 }
 
 /**
@@ -497,18 +564,23 @@ PciExpressRead16 (
                   Register.
   @param  Value   The value to write.
 
-  @return The value written to the PCI configuration register.
+  @retval 0xFFFF  Invalid PCI address.
+  @retval other   The value written to the PCI configuration register.
 
 **/
 UINT16
 EFIAPI
 PciExpressWrite16 (
-  IN      UINTN                     Address,
-  IN      UINT16                    Value
+  IN      UINTN   Address,
+  IN      UINT16  Value
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
-  return MmioWrite16 ((UINTN) GetPciExpressBaseAddress () + Address, Value);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT16)-1;
+  }
+
+  return MmioWrite16 ((UINTN)GetPciExpressBaseAddress () + Address, Value);
 }
 
 /**
@@ -529,18 +601,23 @@ PciExpressWrite16 (
                   Register.
   @param  OrData  The value to OR with the PCI configuration register.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFFFF  Invalid PCI address.
+  @retval other   The value written back to the PCI configuration register.
 
 **/
 UINT16
 EFIAPI
 PciExpressOr16 (
-  IN      UINTN                     Address,
-  IN      UINT16                    OrData
+  IN      UINTN   Address,
+  IN      UINT16  OrData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
-  return MmioOr16 ((UINTN) GetPciExpressBaseAddress () + Address, OrData);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT16)-1;
+  }
+
+  return MmioOr16 ((UINTN)GetPciExpressBaseAddress () + Address, OrData);
 }
 
 /**
@@ -561,18 +638,23 @@ PciExpressOr16 (
                   Register.
   @param  AndData The value to AND with the PCI configuration register.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFFFF  Invalid PCI address.
+  @retval other   The value written back to the PCI configuration register.
 
 **/
 UINT16
 EFIAPI
 PciExpressAnd16 (
-  IN      UINTN                     Address,
-  IN      UINT16                    AndData
+  IN      UINTN   Address,
+  IN      UINT16  AndData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
-  return MmioAnd16 ((UINTN) GetPciExpressBaseAddress () + Address, AndData);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT16)-1;
+  }
+
+  return MmioAnd16 ((UINTN)GetPciExpressBaseAddress () + Address, AndData);
 }
 
 /**
@@ -595,20 +677,25 @@ PciExpressAnd16 (
   @param  AndData The value to AND with the PCI configuration register.
   @param  OrData  The value to OR with the result of the AND operation.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFFFF  Invalid PCI address.
+  @retval other   The value written back to the PCI configuration register.
 
 **/
 UINT16
 EFIAPI
 PciExpressAndThenOr16 (
-  IN      UINTN                     Address,
-  IN      UINT16                    AndData,
-  IN      UINT16                    OrData
+  IN      UINTN   Address,
+  IN      UINT16  AndData,
+  IN      UINT16  OrData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT16)-1;
+  }
+
   return MmioAndThenOr16 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            AndData,
            OrData
            );
@@ -633,20 +720,26 @@ PciExpressAndThenOr16 (
   @param  EndBit    The ordinal of the most significant bit in the bit field.
                     Range 0..15.
 
-  @return The value of the bit field read from the PCI configuration register.
+  @retval 0xFFFF  Invalid PCI address.
+  @retval other   The value of the bit field read from the PCI configuration
+                  register.
 
 **/
 UINT16
 EFIAPI
 PciExpressBitFieldRead16 (
-  IN      UINTN                     Address,
-  IN      UINTN                     StartBit,
-  IN      UINTN                     EndBit
+  IN      UINTN  Address,
+  IN      UINTN  StartBit,
+  IN      UINTN  EndBit
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT16)-1;
+  }
+
   return MmioBitFieldRead16 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            StartBit,
            EndBit
            );
@@ -674,21 +767,26 @@ PciExpressBitFieldRead16 (
                     Range 0..15.
   @param  Value     The new value of the bit field.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFFFF  Invalid PCI address.
+  @retval other   The value written back to the PCI configuration register.
 
 **/
 UINT16
 EFIAPI
 PciExpressBitFieldWrite16 (
-  IN      UINTN                     Address,
-  IN      UINTN                     StartBit,
-  IN      UINTN                     EndBit,
-  IN      UINT16                    Value
+  IN      UINTN   Address,
+  IN      UINTN   StartBit,
+  IN      UINTN   EndBit,
+  IN      UINT16  Value
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT16)-1;
+  }
+
   return MmioBitFieldWrite16 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            StartBit,
            EndBit,
            Value
@@ -720,21 +818,26 @@ PciExpressBitFieldWrite16 (
                     Range 0..15.
   @param  OrData    The value to OR with the PCI configuration register.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFFFF  Invalid PCI address.
+  @retval other   The value written back to the PCI configuration register.
 
 **/
 UINT16
 EFIAPI
 PciExpressBitFieldOr16 (
-  IN      UINTN                     Address,
-  IN      UINTN                     StartBit,
-  IN      UINTN                     EndBit,
-  IN      UINT16                    OrData
+  IN      UINTN   Address,
+  IN      UINTN   StartBit,
+  IN      UINTN   EndBit,
+  IN      UINT16  OrData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT16)-1;
+  }
+
   return MmioBitFieldOr16 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            StartBit,
            EndBit,
            OrData
@@ -766,21 +869,26 @@ PciExpressBitFieldOr16 (
                     Range 0..15.
   @param  AndData   The value to AND with the PCI configuration register.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFFFF  Invalid PCI address.
+  @retval other   The value written back to the PCI configuration register.
 
 **/
 UINT16
 EFIAPI
 PciExpressBitFieldAnd16 (
-  IN      UINTN                     Address,
-  IN      UINTN                     StartBit,
-  IN      UINTN                     EndBit,
-  IN      UINT16                    AndData
+  IN      UINTN   Address,
+  IN      UINTN   StartBit,
+  IN      UINTN   EndBit,
+  IN      UINT16  AndData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT16)-1;
+  }
+
   return MmioBitFieldAnd16 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            StartBit,
            EndBit,
            AndData
@@ -816,22 +924,27 @@ PciExpressBitFieldAnd16 (
   @param  AndData   The value to AND with the PCI configuration register.
   @param  OrData    The value to OR with the result of the AND operation.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFFFF  Invalid PCI address.
+  @retval other   The value written back to the PCI configuration register.
 
 **/
 UINT16
 EFIAPI
 PciExpressBitFieldAndThenOr16 (
-  IN      UINTN                     Address,
-  IN      UINTN                     StartBit,
-  IN      UINTN                     EndBit,
-  IN      UINT16                    AndData,
-  IN      UINT16                    OrData
+  IN      UINTN   Address,
+  IN      UINTN   StartBit,
+  IN      UINTN   EndBit,
+  IN      UINT16  AndData,
+  IN      UINT16  OrData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT16)-1;
+  }
+
   return MmioBitFieldAndThenOr16 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            StartBit,
            EndBit,
            AndData,
@@ -852,17 +965,22 @@ PciExpressBitFieldAndThenOr16 (
   @param  Address The address that encodes the PCI Bus, Device, Function and
                   Register.
 
-  @return The read value from the PCI configuration register.
+  @retval 0xFFFF  Invalid PCI address.
+  @retval other   The read value from the PCI configuration register.
 
 **/
 UINT32
 EFIAPI
 PciExpressRead32 (
-  IN      UINTN                     Address
+  IN      UINTN  Address
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
-  return MmioRead32 ((UINTN) GetPciExpressBaseAddress () + Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT32)-1;
+  }
+
+  return MmioRead32 ((UINTN)GetPciExpressBaseAddress () + Address);
 }
 
 /**
@@ -879,18 +997,23 @@ PciExpressRead32 (
                   Register.
   @param  Value   The value to write.
 
-  @return The value written to the PCI configuration register.
+  @retval 0xFFFFFFFF  Invalid PCI address.
+  @retval other       The value written to the PCI configuration register.
 
 **/
 UINT32
 EFIAPI
 PciExpressWrite32 (
-  IN      UINTN                     Address,
-  IN      UINT32                    Value
+  IN      UINTN   Address,
+  IN      UINT32  Value
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
-  return MmioWrite32 ((UINTN) GetPciExpressBaseAddress () + Address, Value);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT32)-1;
+  }
+
+  return MmioWrite32 ((UINTN)GetPciExpressBaseAddress () + Address, Value);
 }
 
 /**
@@ -911,18 +1034,23 @@ PciExpressWrite32 (
                   Register.
   @param  OrData  The value to OR with the PCI configuration register.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFFFFFFFF  Invalid PCI address.
+  @retval other       The value written back to the PCI configuration register.
 
 **/
 UINT32
 EFIAPI
 PciExpressOr32 (
-  IN      UINTN                     Address,
-  IN      UINT32                    OrData
+  IN      UINTN   Address,
+  IN      UINT32  OrData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
-  return MmioOr32 ((UINTN) GetPciExpressBaseAddress () + Address, OrData);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT32)-1;
+  }
+
+  return MmioOr32 ((UINTN)GetPciExpressBaseAddress () + Address, OrData);
 }
 
 /**
@@ -943,18 +1071,23 @@ PciExpressOr32 (
                   Register.
   @param  AndData The value to AND with the PCI configuration register.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFFFFFFFF  Invalid PCI address.
+  @retval other       The value written back to the PCI configuration register.
 
 **/
 UINT32
 EFIAPI
 PciExpressAnd32 (
-  IN      UINTN                     Address,
-  IN      UINT32                    AndData
+  IN      UINTN   Address,
+  IN      UINT32  AndData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
-  return MmioAnd32 ((UINTN) GetPciExpressBaseAddress () + Address, AndData);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT32)-1;
+  }
+
+  return MmioAnd32 ((UINTN)GetPciExpressBaseAddress () + Address, AndData);
 }
 
 /**
@@ -977,20 +1110,25 @@ PciExpressAnd32 (
   @param  AndData The value to AND with the PCI configuration register.
   @param  OrData  The value to OR with the result of the AND operation.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFFFFFFFF  Invalid PCI address.
+  @retval other       The value written back to the PCI configuration register.
 
 **/
 UINT32
 EFIAPI
 PciExpressAndThenOr32 (
-  IN      UINTN                     Address,
-  IN      UINT32                    AndData,
-  IN      UINT32                    OrData
+  IN      UINTN   Address,
+  IN      UINT32  AndData,
+  IN      UINT32  OrData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT32)-1;
+  }
+
   return MmioAndThenOr32 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            AndData,
            OrData
            );
@@ -1015,20 +1153,26 @@ PciExpressAndThenOr32 (
   @param  EndBit    The ordinal of the most significant bit in the bit field.
                     Range 0..31.
 
-  @return The value of the bit field read from the PCI configuration register.
+  @retval 0xFFFFFFFF  Invalid PCI address.
+  @retval other       The value of the bit field read from the PCI
+                      configuration register.
 
 **/
 UINT32
 EFIAPI
 PciExpressBitFieldRead32 (
-  IN      UINTN                     Address,
-  IN      UINTN                     StartBit,
-  IN      UINTN                     EndBit
+  IN      UINTN  Address,
+  IN      UINTN  StartBit,
+  IN      UINTN  EndBit
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT32)-1;
+  }
+
   return MmioBitFieldRead32 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            StartBit,
            EndBit
            );
@@ -1056,21 +1200,26 @@ PciExpressBitFieldRead32 (
                     Range 0..31.
   @param  Value     The new value of the bit field.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFFFFFFFF  Invalid PCI address.
+  @retval other       The value written back to the PCI configuration register.
 
 **/
 UINT32
 EFIAPI
 PciExpressBitFieldWrite32 (
-  IN      UINTN                     Address,
-  IN      UINTN                     StartBit,
-  IN      UINTN                     EndBit,
-  IN      UINT32                    Value
+  IN      UINTN   Address,
+  IN      UINTN   StartBit,
+  IN      UINTN   EndBit,
+  IN      UINT32  Value
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT32)-1;
+  }
+
   return MmioBitFieldWrite32 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            StartBit,
            EndBit,
            Value
@@ -1102,21 +1251,26 @@ PciExpressBitFieldWrite32 (
                     Range 0..31.
   @param  OrData    The value to OR with the PCI configuration register.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFFFFFFFF  Invalid PCI address.
+  @retval other       The value written back to the PCI configuration register.
 
 **/
 UINT32
 EFIAPI
 PciExpressBitFieldOr32 (
-  IN      UINTN                     Address,
-  IN      UINTN                     StartBit,
-  IN      UINTN                     EndBit,
-  IN      UINT32                    OrData
+  IN      UINTN   Address,
+  IN      UINTN   StartBit,
+  IN      UINTN   EndBit,
+  IN      UINT32  OrData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT32)-1;
+  }
+
   return MmioBitFieldOr32 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            StartBit,
            EndBit,
            OrData
@@ -1148,21 +1302,26 @@ PciExpressBitFieldOr32 (
                     Range 0..31.
   @param  AndData   The value to AND with the PCI configuration register.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFFFFFFFF  Invalid PCI address.
+  @retval other       The value written back to the PCI configuration register.
 
 **/
 UINT32
 EFIAPI
 PciExpressBitFieldAnd32 (
-  IN      UINTN                     Address,
-  IN      UINTN                     StartBit,
-  IN      UINTN                     EndBit,
-  IN      UINT32                    AndData
+  IN      UINTN   Address,
+  IN      UINTN   StartBit,
+  IN      UINTN   EndBit,
+  IN      UINT32  AndData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT32)-1;
+  }
+
   return MmioBitFieldAnd32 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            StartBit,
            EndBit,
            AndData
@@ -1198,22 +1357,27 @@ PciExpressBitFieldAnd32 (
   @param  AndData   The value to AND with the PCI configuration register.
   @param  OrData    The value to OR with the result of the AND operation.
 
-  @return The value written back to the PCI configuration register.
+  @retval 0xFFFFFFFF  Invalid PCI address.
+  @retval other       The value written back to the PCI configuration register.
 
 **/
 UINT32
 EFIAPI
 PciExpressBitFieldAndThenOr32 (
-  IN      UINTN                     Address,
-  IN      UINTN                     StartBit,
-  IN      UINTN                     EndBit,
-  IN      UINT32                    AndData,
-  IN      UINT32                    OrData
+  IN      UINTN   Address,
+  IN      UINTN   StartBit,
+  IN      UINTN   EndBit,
+  IN      UINT32  AndData,
+  IN      UINT32  OrData
   )
 {
   ASSERT_INVALID_PCI_ADDRESS (Address);
+  if (Address >= PcdPciExpressBaseSize ()) {
+    return (UINT32)-1;
+  }
+
   return MmioBitFieldAndThenOr32 (
-           (UINTN) GetPciExpressBaseAddress () + Address,
+           (UINTN)GetPciExpressBaseAddress () + Address,
            StartBit,
            EndBit,
            AndData,
@@ -1241,20 +1405,25 @@ PciExpressBitFieldAndThenOr32 (
   @param  Size          The size in bytes of the transfer.
   @param  Buffer        The pointer to a buffer receiving the data read.
 
-  @return Size read data from StartAddress.
+  @retval (UINTN)-1  Invalid PCI address.
+  @retval other      Size read data from StartAddress.
 
 **/
 UINTN
 EFIAPI
 PciExpressReadBuffer (
-  IN      UINTN                     StartAddress,
-  IN      UINTN                     Size,
-  OUT     VOID                      *Buffer
+  IN      UINTN  StartAddress,
+  IN      UINTN  Size,
+  OUT     VOID   *Buffer
   )
 {
-  UINTN   ReturnValue;
+  UINTN  ReturnValue;
 
   ASSERT_INVALID_PCI_ADDRESS (StartAddress);
+  if (StartAddress >= PcdPciExpressBaseSize ()) {
+    return (UINTN)-1;
+  }
+
   ASSERT (((StartAddress & 0xFFF) + Size) <= 0x1000);
 
   if (Size == 0) {
@@ -1273,41 +1442,41 @@ PciExpressReadBuffer (
     // Read a byte if StartAddress is byte aligned
     //
     *(volatile UINT8 *)Buffer = PciExpressRead8 (StartAddress);
-    StartAddress += sizeof (UINT8);
-    Size -= sizeof (UINT8);
-    Buffer = (UINT8*)Buffer + 1;
+    StartAddress             += sizeof (UINT8);
+    Size                     -= sizeof (UINT8);
+    Buffer                    = (UINT8 *)Buffer + 1;
   }
 
-  if (Size >= sizeof (UINT16) && (StartAddress & 2) != 0) {
+  if ((Size >= sizeof (UINT16)) && ((StartAddress & 2) != 0)) {
     //
     // Read a word if StartAddress is word aligned
     //
-    WriteUnaligned16 ((UINT16 *) Buffer, (UINT16) PciExpressRead16 (StartAddress));
+    WriteUnaligned16 ((UINT16 *)Buffer, (UINT16)PciExpressRead16 (StartAddress));
 
     StartAddress += sizeof (UINT16);
-    Size -= sizeof (UINT16);
-    Buffer = (UINT16*)Buffer + 1;
+    Size         -= sizeof (UINT16);
+    Buffer        = (UINT16 *)Buffer + 1;
   }
 
   while (Size >= sizeof (UINT32)) {
     //
     // Read as many double words as possible
     //
-    WriteUnaligned32 ((UINT32 *) Buffer, (UINT32) PciExpressRead32 (StartAddress));
+    WriteUnaligned32 ((UINT32 *)Buffer, (UINT32)PciExpressRead32 (StartAddress));
 
     StartAddress += sizeof (UINT32);
-    Size -= sizeof (UINT32);
-    Buffer = (UINT32*)Buffer + 1;
+    Size         -= sizeof (UINT32);
+    Buffer        = (UINT32 *)Buffer + 1;
   }
 
   if (Size >= sizeof (UINT16)) {
     //
     // Read the last remaining word if exist
     //
-    WriteUnaligned16 ((UINT16 *) Buffer, (UINT16) PciExpressRead16 (StartAddress));
+    WriteUnaligned16 ((UINT16 *)Buffer, (UINT16)PciExpressRead16 (StartAddress));
     StartAddress += sizeof (UINT16);
-    Size -= sizeof (UINT16);
-    Buffer = (UINT16*)Buffer + 1;
+    Size         -= sizeof (UINT16);
+    Buffer        = (UINT16 *)Buffer + 1;
   }
 
   if (Size >= sizeof (UINT8)) {
@@ -1341,20 +1510,25 @@ PciExpressReadBuffer (
   @param  Size          The size in bytes of the transfer.
   @param  Buffer        The pointer to a buffer containing the data to write.
 
-  @return Size written to StartAddress.
+  @retval (UINTN)-1  Invalid PCI address.
+  @retval other      Size written to StartAddress.
 
 **/
 UINTN
 EFIAPI
 PciExpressWriteBuffer (
-  IN      UINTN                     StartAddress,
-  IN      UINTN                     Size,
-  IN      VOID                      *Buffer
+  IN      UINTN  StartAddress,
+  IN      UINTN  Size,
+  IN      VOID   *Buffer
   )
 {
-  UINTN                             ReturnValue;
+  UINTN  ReturnValue;
 
   ASSERT_INVALID_PCI_ADDRESS (StartAddress);
+  if (StartAddress >= PcdPciExpressBaseSize ()) {
+    return (UINTN)-1;
+  }
+
   ASSERT (((StartAddress & 0xFFF) + Size) <= 0x1000);
 
   if (Size == 0) {
@@ -1372,47 +1546,47 @@ PciExpressWriteBuffer (
     //
     // Write a byte if StartAddress is byte aligned
     //
-    PciExpressWrite8 (StartAddress, *(UINT8*)Buffer);
+    PciExpressWrite8 (StartAddress, *(UINT8 *)Buffer);
     StartAddress += sizeof (UINT8);
-    Size -= sizeof (UINT8);
-    Buffer = (UINT8*)Buffer + 1;
+    Size         -= sizeof (UINT8);
+    Buffer        = (UINT8 *)Buffer + 1;
   }
 
-  if (Size >= sizeof (UINT16) && (StartAddress & 2) != 0) {
+  if ((Size >= sizeof (UINT16)) && ((StartAddress & 2) != 0)) {
     //
     // Write a word if StartAddress is word aligned
     //
-    PciExpressWrite16 (StartAddress, ReadUnaligned16 ((UINT16*)Buffer));
+    PciExpressWrite16 (StartAddress, ReadUnaligned16 ((UINT16 *)Buffer));
     StartAddress += sizeof (UINT16);
-    Size -= sizeof (UINT16);
-    Buffer = (UINT16*)Buffer + 1;
+    Size         -= sizeof (UINT16);
+    Buffer        = (UINT16 *)Buffer + 1;
   }
 
   while (Size >= sizeof (UINT32)) {
     //
     // Write as many double words as possible
     //
-    PciExpressWrite32 (StartAddress, ReadUnaligned32 ((UINT32*)Buffer));
+    PciExpressWrite32 (StartAddress, ReadUnaligned32 ((UINT32 *)Buffer));
     StartAddress += sizeof (UINT32);
-    Size -= sizeof (UINT32);
-    Buffer = (UINT32*)Buffer + 1;
+    Size         -= sizeof (UINT32);
+    Buffer        = (UINT32 *)Buffer + 1;
   }
 
   if (Size >= sizeof (UINT16)) {
     //
     // Write the last remaining word if exist
     //
-    PciExpressWrite16 (StartAddress, ReadUnaligned16 ((UINT16*)Buffer));
+    PciExpressWrite16 (StartAddress, ReadUnaligned16 ((UINT16 *)Buffer));
     StartAddress += sizeof (UINT16);
-    Size -= sizeof (UINT16);
-    Buffer = (UINT16*)Buffer + 1;
+    Size         -= sizeof (UINT16);
+    Buffer        = (UINT16 *)Buffer + 1;
   }
 
   if (Size >= sizeof (UINT8)) {
     //
     // Write the last remaining byte if exist
     //
-    PciExpressWrite8 (StartAddress, *(UINT8*)Buffer);
+    PciExpressWrite8 (StartAddress, *(UINT8 *)Buffer);
   }
 
   return ReturnValue;
