@@ -2,20 +2,11 @@
 Implementation of PcdLib class library for PEI phase.
 
 Copyright (c) 2006 - 2018, Intel Corporation. All rights reserved.<BR>
-This program and the accompanying materials
-are licensed and made available under the terms and conditions of the BSD License
-which accompanies this distribution.  The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php
-
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 
 **/
 
-
-
-
 #include <PiPei.h>
 
 #include <Ppi/Pcd.h>
@@ -42,8 +33,8 @@ GetPcdPpiPointer (
   VOID
   )
 {
-  EFI_STATUS        Status;
-  PCD_PPI           *PcdPpi;
+  EFI_STATUS  Status;
+  PCD_PPI     *PcdPpi;
 
   Status = PeiServicesLocatePpi (&gPcdPpiGuid, 0, NULL, (VOID **)&PcdPpi);
   ASSERT_EFI_ERROR (Status);
@@ -65,8 +56,8 @@ GetPiPcdPpiPointer (
   VOID
   )
 {
-  EFI_STATUS        Status;
-  EFI_PEI_PCD_PPI   *PiPcdPpi;
+  EFI_STATUS       Status;
+  EFI_PEI_PCD_PPI  *PiPcdPpi;
 
   Status = PeiServicesLocatePpi (&gEfiPeiPcdPpiGuid, 0, NULL, (VOID **)&PiPcdPpi);
   ASSERT_EFI_ERROR (Status);
@@ -88,8 +79,8 @@ GetPcdInfoPpiPointer (
   VOID
   )
 {
-  EFI_STATUS            Status;
-  GET_PCD_INFO_PPI      *PcdInfoPpi;
+  EFI_STATUS        Status;
+  GET_PCD_INFO_PPI  *PcdInfoPpi;
 
   Status = PeiServicesLocatePpi (&gGetPcdInfoPpiGuid, 0, NULL, (VOID **)&PcdInfoPpi);
   ASSERT_EFI_ERROR (Status);
@@ -134,16 +125,14 @@ GetPiPcdInfoPpiPointer (
 UINTN
 EFIAPI
 LibPcdSetSku (
-  IN UINTN   SkuId
+  IN UINTN  SkuId
   )
 {
-  GetPiPcdPpiPointer()->SetSku (SkuId);
+  GetPiPcdPpiPointer ()->SetSku (SkuId);
 
   return SkuId;
 }
 
-
-
 /**
   This function provides a means by which to retrieve a value for a given PCD token.
 
@@ -157,14 +146,12 @@ LibPcdSetSku (
 UINT8
 EFIAPI
 LibPcdGet8 (
-  IN UINTN             TokenNumber
+  IN UINTN  TokenNumber
   )
 {
   return (GetPcdPpiPointer ())->Get8 (TokenNumber);
 }
 
-
-
 /**
   This function provides a means by which to retrieve a value for a given PCD token.
 
@@ -178,14 +165,12 @@ LibPcdGet8 (
 UINT16
 EFIAPI
 LibPcdGet16 (
-  IN UINTN             TokenNumber
+  IN UINTN  TokenNumber
   )
 {
   return (GetPcdPpiPointer ())->Get16 (TokenNumber);
 }
 
-
-
 /**
   This function provides a means by which to retrieve a value for a given PCD token.
 
@@ -199,14 +184,12 @@ LibPcdGet16 (
 UINT32
 EFIAPI
 LibPcdGet32 (
-  IN UINTN             TokenNumber
+  IN UINTN  TokenNumber
   )
 {
   return (GetPcdPpiPointer ())->Get32 (TokenNumber);
 }
 
-
-
 /**
   This function provides a means by which to retrieve a value for a given PCD token.
 
@@ -220,14 +203,12 @@ LibPcdGet32 (
 UINT64
 EFIAPI
 LibPcdGet64 (
-  IN UINTN             TokenNumber
+  IN UINTN  TokenNumber
   )
 {
   return (GetPcdPpiPointer ())->Get64 (TokenNumber);
 }
 
-
-
 /**
   This function provides a means by which to retrieve a value for a given PCD token.
 
@@ -241,14 +222,12 @@ LibPcdGet64 (
 VOID *
 EFIAPI
 LibPcdGetPtr (
-  IN UINTN             TokenNumber
+  IN UINTN  TokenNumber
   )
 {
   return (GetPcdPpiPointer ())->GetPtr (TokenNumber);
 }
 
-
-
 /**
   This function provides a means by which to retrieve a value for a given PCD token.
 
@@ -262,14 +241,12 @@ LibPcdGetPtr (
 BOOLEAN
 EFIAPI
 LibPcdGetBool (
-  IN UINTN             TokenNumber
+  IN UINTN  TokenNumber
   )
 {
   return (GetPcdPpiPointer ())->GetBool (TokenNumber);
 }
 
-
-
 /**
   This function provides a means by which to retrieve the size of a given PCD token.
 
@@ -281,14 +258,12 @@ LibPcdGetBool (
 UINTN
 EFIAPI
 LibPcdGetSize (
-  IN UINTN             TokenNumber
+  IN UINTN  TokenNumber
   )
 {
   return (GetPcdPpiPointer ())->GetSize (TokenNumber);
 }
 
-
-
 /**
   This function provides a means by which to retrieve a value for a given PCD token.
 
@@ -306,8 +281,8 @@ LibPcdGetSize (
 UINT8
 EFIAPI
 LibPcdGetEx8 (
-  IN CONST GUID        *Guid,
-  IN UINTN             TokenNumber
+  IN CONST GUID  *Guid,
+  IN UINTN       TokenNumber
   )
 {
   ASSERT (Guid != NULL);
@@ -315,8 +290,6 @@ LibPcdGetEx8 (
   return (GetPiPcdPpiPointer ())->Get8 (Guid, TokenNumber);
 }
 
-
-
 /**
   This function provides a means by which to retrieve a value for a given PCD token.
 
@@ -334,18 +307,15 @@ LibPcdGetEx8 (
 UINT16
 EFIAPI
 LibPcdGetEx16 (
-  IN CONST GUID        *Guid,
-  IN UINTN             TokenNumber
+  IN CONST GUID  *Guid,
+  IN UINTN       TokenNumber
   )
 {
-
   ASSERT (Guid != NULL);
 
   return (GetPiPcdPpiPointer ())->Get16 (Guid, TokenNumber);
 }
 
-
-
 /**
   Returns the 32-bit value for the token specified by TokenNumber and Guid.
   If Guid is NULL, then ASSERT().
@@ -360,8 +330,8 @@ LibPcdGetEx16 (
 UINT32
 EFIAPI
 LibPcdGetEx32 (
-  IN CONST GUID        *Guid,
-  IN UINTN             TokenNumber
+  IN CONST GUID  *Guid,
+  IN UINTN       TokenNumber
   )
 {
   ASSERT (Guid != NULL);
@@ -369,9 +339,6 @@ LibPcdGetEx32 (
   return (GetPiPcdPpiPointer ())->Get32 (Guid, TokenNumber);
 }
 
-
-
-
 /**
   This function provides a means by which to retrieve a value for a given PCD token.
 
@@ -389,16 +356,14 @@ LibPcdGetEx32 (
 UINT64
 EFIAPI
 LibPcdGetEx64 (
-  IN CONST GUID        *Guid,
-  IN UINTN             TokenNumber
+  IN CONST GUID  *Guid,
+  IN UINTN       TokenNumber
   )
 {
   ASSERT (Guid != NULL);
   return (GetPiPcdPpiPointer ())->Get64 (Guid, TokenNumber);
 }
 
-
-
 /**
   This function provides a means by which to retrieve a value for a given PCD token.
 
@@ -416,8 +381,8 @@ LibPcdGetEx64 (
 VOID *
 EFIAPI
 LibPcdGetExPtr (
-  IN CONST GUID        *Guid,
-  IN UINTN             TokenNumber
+  IN CONST GUID  *Guid,
+  IN UINTN       TokenNumber
   )
 {
   ASSERT (Guid != NULL);
@@ -425,8 +390,6 @@ LibPcdGetExPtr (
   return (GetPiPcdPpiPointer ())->GetPtr (Guid, TokenNumber);
 }
 
-
-
 /**
   This function provides a means by which to retrieve a value for a given PCD token.
 
@@ -444,16 +407,14 @@ LibPcdGetExPtr (
 BOOLEAN
 EFIAPI
 LibPcdGetExBool (
-  IN CONST GUID        *Guid,
-  IN UINTN             TokenNumber
+  IN CONST GUID  *Guid,
+  IN UINTN       TokenNumber
   )
 {
   ASSERT (Guid != NULL);
   return (GetPiPcdPpiPointer ())->GetBool (Guid, TokenNumber);
 }
 
-
-
 /**
   This function provides a means by which to retrieve the size of a given PCD token.
 
@@ -471,412 +432,14 @@ LibPcdGetExBool (
 UINTN
 EFIAPI
 LibPcdGetExSize (
-  IN CONST GUID        *Guid,
-  IN UINTN             TokenNumber
+  IN CONST GUID  *Guid,
+  IN UINTN       TokenNumber
   )
 {
   ASSERT (Guid != NULL);
   return (GetPiPcdPpiPointer ())->GetSize (Guid, TokenNumber);
 }
 
-
-
-#ifndef DISABLE_NEW_DEPRECATED_INTERFACES
-/**
-  This function provides a means by which to set a value for a given PCD token.
-
-  Sets the 8-bit value for the token specified by TokenNumber
-  to the value specified by Value.  Value is returned.
-
-  @param[in]  TokenNumber   The PCD token number to set a current value for.
-  @param[in]  Value         The 8-bit value to set.
-
-  @return Return the value that was set.
-
-**/
-UINT8
-EFIAPI
-LibPcdSet8 (
-  IN UINTN             TokenNumber,
-  IN UINT8             Value
-  )
-{
-  (GetPcdPpiPointer ())->Set8 (TokenNumber, Value);
-
-  return Value;
-}
-
-
-
-/**
-  This function provides a means by which to set a value for a given PCD token.
-
-  Sets the 16-bit value for the token specified by TokenNumber
-  to the value specified by Value.  Value is returned.
-
-  @param[in]  TokenNumber   The PCD token number to set a current value for.
-  @param[in]  Value         The 16-bit value to set.
-
-  @return Return the value that was set.
-
-**/
-UINT16
-EFIAPI
-LibPcdSet16 (
-  IN UINTN             TokenNumber,
-  IN UINT16            Value
-  )
-{
-  (GetPcdPpiPointer ())->Set16 (TokenNumber, Value);
-
-  return Value;
-}
-
-
-
-/**
-  This function provides a means by which to set a value for a given PCD token.
-
-  Sets the 32-bit value for the token specified by TokenNumber
-  to the value specified by Value.  Value is returned.
-
-  @param[in]  TokenNumber   The PCD token number to set a current value for.
-  @param[in]  Value         The 32-bit value to set.
-
-  @return Return the value that was set.
-
-**/
-UINT32
-EFIAPI
-LibPcdSet32 (
-  IN UINTN             TokenNumber,
-  IN UINT32            Value
-  )
-{
-  (GetPcdPpiPointer ())->Set32 (TokenNumber, Value);
-
-  return Value;
-}
-
-
-
-/**
-  This function provides a means by which to set a value for a given PCD token.
-
-  Sets the 64-bit value for the token specified by TokenNumber
-  to the value specified by Value.  Value is returned.
-
-  @param[in]  TokenNumber   The PCD token number to set a current value for.
-  @param[in]  Value         The 64-bit value to set.
-
-  @return Return the value that was set.
-
-**/
-UINT64
-EFIAPI
-LibPcdSet64 (
-  IN UINTN             TokenNumber,
-  IN UINT64            Value
-  )
-{
-  (GetPcdPpiPointer ())->Set64 (TokenNumber, Value);
-
-  return Value;
-}
-
-
-
-/**
-  This function provides a means by which to set a value for a given PCD token.
-
-  Sets a buffer for the token specified by TokenNumber to the value
-  specified by Buffer and SizeOfBuffer.  Buffer is returned.
-  If SizeOfBuffer is greater than the maximum size support by TokenNumber,
-  then set SizeOfBuffer to the maximum size supported by TokenNumber and
-  return NULL to indicate that the set operation was not actually performed.
-
-  If SizeOfBuffer is set to MAX_ADDRESS, then SizeOfBuffer must be set to the
-  maximum size supported by TokenName and NULL must be returned.
-
-  If SizeOfBuffer is NULL, then ASSERT().
-  If SizeOfBuffer > 0 and Buffer is NULL, then ASSERT().
-
-  @param[in]      TokenNumber   The PCD token number to set a current value for.
-  @param[in, out] SizeOfBuffer  The size, in bytes, of Buffer.
-  @param[in]      Buffer        A pointer to the buffer to set.
-
-  @return Return the pointer for the buffer been set.
-
-**/
-VOID *
-EFIAPI
-LibPcdSetPtr (
-  IN        UINTN             TokenNumber,
-  IN OUT    UINTN             *SizeOfBuffer,
-  IN CONST  VOID              *Buffer
-  )
-{
-  EFI_STATUS Status;
-  UINTN      InputSizeOfBuffer;
-
-  ASSERT (SizeOfBuffer != NULL);
-
-  if (*SizeOfBuffer > 0) {
-    ASSERT (Buffer != NULL);
-  }
-
-  InputSizeOfBuffer = *SizeOfBuffer;
-  Status = (GetPcdPpiPointer ())->SetPtr (TokenNumber, SizeOfBuffer, (VOID *) Buffer);
-  if (EFI_ERROR (Status) && (*SizeOfBuffer < InputSizeOfBuffer)) {
-    return NULL;
-  }
-
-  return (VOID *) Buffer;
-}
-
-
-
-/**
-  This function provides a means by which to set a value for a given PCD token.
-
-  Sets the Boolean value for the token specified by TokenNumber
-  to the value specified by Value.  Value is returned.
-
-  @param[in]  TokenNumber   The PCD token number to set a current value for.
-  @param[in]  Value         The boolean value to set.
-
-  @return Return the value that was set.
-
-**/
-BOOLEAN
-EFIAPI
-LibPcdSetBool (
-  IN UINTN             TokenNumber,
-  IN BOOLEAN           Value
-  )
-{
-  (GetPcdPpiPointer ())->SetBool (TokenNumber, Value);
-
-  return Value;
-}
-
-
-
-/**
-  This function provides a means by which to set a value for a given PCD token.
-
-  Sets the 8-bit value for the token specified by TokenNumber and
-  Guid to the value specified by Value. Value is returned.
-
-  If Guid is NULL, then ASSERT().
-
-  @param[in]  Guid          The pointer to a 128-bit unique value that
-                            designates which namespace to set a value from.
-  @param[in]  TokenNumber   The PCD token number to set a current value for.
-  @param[in]  Value         The 8-bit value to set.
-
-  @return Return the value that was set.
-
-**/
-UINT8
-EFIAPI
-LibPcdSetEx8 (
-  IN CONST GUID        *Guid,
-  IN UINTN             TokenNumber,
-  IN UINT8             Value
-  )
-{
-  ASSERT (Guid != NULL);
-
-  (GetPiPcdPpiPointer ())->Set8 (Guid, TokenNumber, Value);
-
-  return Value;
-}
-
-
-
-/**
-  This function provides a means by which to set a value for a given PCD token.
-
-  Sets the 16-bit value for the token specified by TokenNumber and
-  Guid to the value specified by Value. Value is returned.
-
-  If Guid is NULL, then ASSERT().
-
-  @param[in]  Guid          The pointer to a 128-bit unique value that
-                            designates which namespace to set a value from.
-  @param[in]  TokenNumber   The PCD token number to set a current value for.
-  @param[in]  Value         The 16-bit value to set.
-
-  @return Return the value that was set.
-
-**/
-UINT16
-EFIAPI
-LibPcdSetEx16 (
-  IN CONST GUID        *Guid,
-  IN UINTN             TokenNumber,
-  IN UINT16            Value
-  )
-{
-  ASSERT (Guid != NULL);
-
-  (GetPiPcdPpiPointer ())->Set16 (Guid, TokenNumber, Value);
-
-  return Value;
-}
-
-
-
-/**
-  This function provides a means by which to set a value for a given PCD token.
-
-  Sets the 32-bit value for the token specified by TokenNumber and
-  Guid to the value specified by Value. Value is returned.
-
-  If Guid is NULL, then ASSERT().
-
-  @param[in]  Guid          The pointer to a 128-bit unique value that
-                            designates which namespace to set a value from.
-  @param[in]  TokenNumber   The PCD token number to set a current value for.
-  @param[in]  Value         The 32-bit value to set.
-
-  @return Return the value that was set.
-
-**/
-UINT32
-EFIAPI
-LibPcdSetEx32 (
-  IN CONST GUID        *Guid,
-  IN UINTN             TokenNumber,
-  IN UINT32            Value
-  )
-{
-  ASSERT (Guid != NULL);
-
-  (GetPiPcdPpiPointer ())->Set32 (Guid, TokenNumber, Value);
-
-  return Value;
-}
-
-
-
-/**
-  This function provides a means by which to set a value for a given PCD token.
-
-  Sets the 64-bit value for the token specified by TokenNumber and
-  Guid to the value specified by Value. Value is returned.
-
-  If Guid is NULL, then ASSERT().
-
-  @param[in]  Guid          The pointer to a 128-bit unique value that
-                            designates which namespace to set a value from.
-  @param[in]  TokenNumber   The PCD token number to set a current value for.
-  @param[in]  Value         The 64-bit value to set.
-
-  @return Return the value that was set.
-
-**/
-UINT64
-EFIAPI
-LibPcdSetEx64 (
-  IN CONST GUID        *Guid,
-  IN UINTN             TokenNumber,
-  IN UINT64            Value
-  )
-{
-  ASSERT (Guid != NULL);
-
-  (GetPiPcdPpiPointer ())->Set64 (Guid, TokenNumber, Value);
-
-  return Value;
-}
-
-
-
-/**
-  This function provides a means by which to set a value for a given PCD token.
-
-  Sets a buffer for the token specified by TokenNumber to the value specified by
-  Buffer and SizeOfBuffer.  Buffer is returned.  If SizeOfBuffer is greater than
-  the maximum size support by TokenNumber, then set SizeOfBuffer to the maximum size
-  supported by TokenNumber and return NULL to indicate that the set operation
-  was not actually performed.
-
-  If Guid is NULL, then ASSERT().
-  If SizeOfBuffer is NULL, then ASSERT().
-  If SizeOfBuffer > 0 and Buffer is NULL, then ASSERT().
-
-  @param[in]  Guid              The pointer to a 128-bit unique value that
-                                designates which namespace to set a value from.
-  @param[in]  TokenNumber       The PCD token number to set a current value for.
-  @param[in, out] SizeOfBuffer  The size, in bytes, of Buffer.
-  @param[in]  Buffer            A pointer to the buffer to set.
-
-  @return Return the pinter to the buffer been set.
-
-**/
-VOID *
-EFIAPI
-LibPcdSetExPtr (
-  IN      CONST GUID        *Guid,
-  IN      UINTN             TokenNumber,
-  IN OUT  UINTN             *SizeOfBuffer,
-  IN      VOID              *Buffer
-  )
-{
-  EFI_STATUS      Status;
-  UINTN           InputSizeOfBuffer;
-
-  ASSERT (SizeOfBuffer != NULL);
-  if (*SizeOfBuffer > 0) {
-    ASSERT (Buffer != NULL);
-  }
-  ASSERT (Guid != NULL);
-
-  InputSizeOfBuffer = *SizeOfBuffer;
-  Status = (GetPiPcdPpiPointer ())->SetPtr (Guid, TokenNumber, SizeOfBuffer, Buffer);
-  if (EFI_ERROR (Status) && (*SizeOfBuffer < InputSizeOfBuffer)) {
-    return NULL;
-  }
-
-  return Buffer;
-}
-
-
-
-/**
-  This function provides a means by which to set a value for a given PCD token.
-
-  Sets the Boolean value for the token specified by TokenNumber and
-  Guid to the value specified by Value. Value is returned.
-
-  If Guid is NULL, then ASSERT().
-
-  @param[in]  Guid          The pointer to a 128-bit unique value that
-                            designates which namespace to set a value from.
-  @param[in]  TokenNumber   The PCD token number to set a current value for.
-  @param[in]  Value         The Boolean value to set.
-
-  @return Return the value that was set.
-
-**/
-BOOLEAN
-EFIAPI
-LibPcdSetExBool (
-  IN CONST GUID        *Guid,
-  IN UINTN             TokenNumber,
-  IN BOOLEAN           Value
-  )
-{
-  ASSERT (Guid != NULL);
-
-  (GetPiPcdPpiPointer ())->SetBool (Guid, TokenNumber, Value);
-
-  return Value;
-}
-#endif
-
 /**
   This function provides a means by which to set a value for a given PCD token.
 
@@ -892,8 +455,8 @@ LibPcdSetExBool (
 RETURN_STATUS
 EFIAPI
 LibPcdSet8S (
-  IN UINTN          TokenNumber,
-  IN UINT8          Value
+  IN UINTN  TokenNumber,
+  IN UINT8  Value
   )
 {
   return (GetPcdPpiPointer ())->Set8 (TokenNumber, Value);
@@ -914,8 +477,8 @@ LibPcdSet8S (
 RETURN_STATUS
 EFIAPI
 LibPcdSet16S (
-  IN UINTN          TokenNumber,
-  IN UINT16         Value
+  IN UINTN   TokenNumber,
+  IN UINT16  Value
   )
 {
   return (GetPcdPpiPointer ())->Set16 (TokenNumber, Value);
@@ -936,8 +499,8 @@ LibPcdSet16S (
 RETURN_STATUS
 EFIAPI
 LibPcdSet32S (
-  IN UINTN          TokenNumber,
-  IN UINT32         Value
+  IN UINTN   TokenNumber,
+  IN UINT32  Value
   )
 {
   return (GetPcdPpiPointer ())->Set32 (TokenNumber, Value);
@@ -958,8 +521,8 @@ LibPcdSet32S (
 RETURN_STATUS
 EFIAPI
 LibPcdSet64S (
-  IN UINTN          TokenNumber,
-  IN UINT64         Value
+  IN UINTN   TokenNumber,
+  IN UINT64  Value
   )
 {
   return (GetPcdPpiPointer ())->Set64 (TokenNumber, Value);
@@ -990,9 +553,9 @@ LibPcdSet64S (
 RETURN_STATUS
 EFIAPI
 LibPcdSetPtrS (
-  IN       UINTN    TokenNumber,
-  IN OUT   UINTN    *SizeOfBuffer,
-  IN CONST VOID     *Buffer
+  IN       UINTN  TokenNumber,
+  IN OUT   UINTN  *SizeOfBuffer,
+  IN CONST VOID   *Buffer
   )
 {
   ASSERT (SizeOfBuffer != NULL);
@@ -1001,7 +564,7 @@ LibPcdSetPtrS (
     ASSERT (Buffer != NULL);
   }
 
-  return (GetPcdPpiPointer ())->SetPtr (TokenNumber, SizeOfBuffer, (VOID *) Buffer);
+  return (GetPcdPpiPointer ())->SetPtr (TokenNumber, SizeOfBuffer, (VOID *)Buffer);
 }
 
 /**
@@ -1019,8 +582,8 @@ LibPcdSetPtrS (
 RETURN_STATUS
 EFIAPI
 LibPcdSetBoolS (
-  IN UINTN          TokenNumber,
-  IN BOOLEAN        Value
+  IN UINTN    TokenNumber,
+  IN BOOLEAN  Value
   )
 {
   return (GetPcdPpiPointer ())->SetBool (TokenNumber, Value);
@@ -1045,9 +608,9 @@ LibPcdSetBoolS (
 RETURN_STATUS
 EFIAPI
 LibPcdSetEx8S (
-  IN CONST GUID     *Guid,
-  IN UINTN          TokenNumber,
-  IN UINT8          Value
+  IN CONST GUID  *Guid,
+  IN UINTN       TokenNumber,
+  IN UINT8       Value
   )
 {
   ASSERT (Guid != NULL);
@@ -1074,9 +637,9 @@ LibPcdSetEx8S (
 RETURN_STATUS
 EFIAPI
 LibPcdSetEx16S (
-  IN CONST GUID     *Guid,
-  IN UINTN          TokenNumber,
-  IN UINT16         Value
+  IN CONST GUID  *Guid,
+  IN UINTN       TokenNumber,
+  IN UINT16      Value
   )
 {
   ASSERT (Guid != NULL);
@@ -1103,9 +666,9 @@ LibPcdSetEx16S (
 RETURN_STATUS
 EFIAPI
 LibPcdSetEx32S (
-  IN CONST GUID     *Guid,
-  IN UINTN          TokenNumber,
-  IN UINT32         Value
+  IN CONST GUID  *Guid,
+  IN UINTN       TokenNumber,
+  IN UINT32      Value
   )
 {
   ASSERT (Guid != NULL);
@@ -1132,9 +695,9 @@ LibPcdSetEx32S (
 RETURN_STATUS
 EFIAPI
 LibPcdSetEx64S (
-  IN CONST GUID     *Guid,
-  IN UINTN          TokenNumber,
-  IN UINT64         Value
+  IN CONST GUID  *Guid,
+  IN UINTN       TokenNumber,
+  IN UINT64      Value
   )
 {
   ASSERT (Guid != NULL);
@@ -1167,10 +730,10 @@ LibPcdSetEx64S (
 RETURN_STATUS
 EFIAPI
 LibPcdSetExPtrS (
-  IN CONST GUID     *Guid,
-  IN       UINTN    TokenNumber,
-  IN OUT   UINTN    *SizeOfBuffer,
-  IN       VOID     *Buffer
+  IN CONST GUID   *Guid,
+  IN       UINTN  TokenNumber,
+  IN OUT   UINTN  *SizeOfBuffer,
+  IN       VOID   *Buffer
   )
 {
   ASSERT (Guid != NULL);
@@ -1203,9 +766,9 @@ LibPcdSetExPtrS (
 RETURN_STATUS
 EFIAPI
 LibPcdSetExBoolS (
-  IN CONST GUID     *Guid,
-  IN UINTN          TokenNumber,
-  IN BOOLEAN        Value
+  IN CONST GUID  *Guid,
+  IN UINTN       TokenNumber,
+  IN BOOLEAN     Value
   )
 {
   ASSERT (Guid != NULL);
@@ -1232,24 +795,22 @@ LibPcdSetExBoolS (
 VOID
 EFIAPI
 LibPcdCallbackOnSet (
-  IN CONST GUID               *Guid,       OPTIONAL
-  IN UINTN                    TokenNumber,
-  IN PCD_CALLBACK             NotificationFunction
+  IN CONST GUID    *Guid        OPTIONAL,
+  IN UINTN         TokenNumber,
+  IN PCD_CALLBACK  NotificationFunction
   )
 {
-  EFI_STATUS Status;
+  EFI_STATUS  Status;
 
   ASSERT (NotificationFunction != NULL);
 
-  Status = (GetPiPcdPpiPointer ())->CallbackOnSet (Guid, TokenNumber, (EFI_PEI_PCD_PPI_CALLBACK) NotificationFunction);
+  Status = (GetPiPcdPpiPointer ())->CallbackOnSet (Guid, TokenNumber, (EFI_PEI_PCD_PPI_CALLBACK)NotificationFunction);
 
   ASSERT_EFI_ERROR (Status);
 
   return;
 }
 
-
-
 /**
   Disable a notification function that was established with LibPcdCallbackonSet().
 
@@ -1266,24 +827,22 @@ LibPcdCallbackOnSet (
 VOID
 EFIAPI
 LibPcdCancelCallback (
-  IN CONST GUID               *Guid,       OPTIONAL
-  IN UINTN                    TokenNumber,
-  IN PCD_CALLBACK             NotificationFunction
+  IN CONST GUID    *Guid        OPTIONAL,
+  IN UINTN         TokenNumber,
+  IN PCD_CALLBACK  NotificationFunction
   )
 {
-  EFI_STATUS Status;
+  EFI_STATUS  Status;
 
   ASSERT (NotificationFunction != NULL);
 
-  Status = (GetPiPcdPpiPointer ())->CancelCallback (Guid, TokenNumber, (EFI_PEI_PCD_PPI_CALLBACK) NotificationFunction);
+  Status = (GetPiPcdPpiPointer ())->CancelCallback (Guid, TokenNumber, (EFI_PEI_PCD_PPI_CALLBACK)NotificationFunction);
 
   ASSERT_EFI_ERROR (Status);
 
   return;
 }
 
-
-
 /**
   Retrieves the next token in a token space.
 
@@ -1306,11 +865,11 @@ LibPcdCancelCallback (
 UINTN
 EFIAPI
 LibPcdGetNextToken (
-  IN CONST GUID               *Guid,       OPTIONAL
-  IN UINTN                    TokenNumber
+  IN CONST GUID  *Guid        OPTIONAL,
+  IN UINTN       TokenNumber
   )
 {
-  EFI_STATUS    Status;
+  EFI_STATUS  Status;
 
   Status = (GetPiPcdPpiPointer ())->GetNextToken (Guid, &TokenNumber);
   ASSERT (!EFI_ERROR (Status) || TokenNumber == 0);
@@ -1318,7 +877,6 @@ LibPcdGetNextToken (
   return TokenNumber;
 }
 
-
 /**
   Used to retrieve the list of available PCD token space GUIDs.
 
@@ -1340,11 +898,9 @@ LibPcdGetNextTokenSpace (
 {
   (GetPiPcdPpiPointer ())->GetNextTokenSpace (&TokenSpaceGuid);
 
-  return (GUID *) TokenSpaceGuid;
+  return (GUID *)TokenSpaceGuid;
 }
 
-
-
 /**
   Sets a value of a patchable PCD entry that is type pointer.
 
@@ -1371,10 +927,10 @@ LibPcdGetNextTokenSpace (
 VOID *
 EFIAPI
 LibPatchPcdSetPtr (
-  OUT       VOID        *PatchVariable,
-  IN        UINTN       MaximumDatumSize,
-  IN OUT    UINTN       *SizeOfBuffer,
-  IN CONST  VOID        *Buffer
+  OUT       VOID   *PatchVariable,
+  IN        UINTN  MaximumDatumSize,
+  IN OUT    UINTN  *SizeOfBuffer,
+  IN CONST  VOID   *Buffer
   )
 {
   ASSERT (PatchVariable != NULL);
@@ -1385,14 +941,15 @@ LibPatchPcdSetPtr (
   }
 
   if ((*SizeOfBuffer > MaximumDatumSize) ||
-      (*SizeOfBuffer == MAX_ADDRESS)) {
+      (*SizeOfBuffer == MAX_ADDRESS))
+  {
     *SizeOfBuffer = MaximumDatumSize;
     return NULL;
   }
 
   CopyMem (PatchVariable, Buffer, *SizeOfBuffer);
 
-  return (VOID *) Buffer;
+  return (VOID *)Buffer;
 }
 
 /**
@@ -1421,10 +978,10 @@ LibPatchPcdSetPtr (
 RETURN_STATUS
 EFIAPI
 LibPatchPcdSetPtrS (
-  OUT      VOID     *PatchVariable,
-  IN       UINTN    MaximumDatumSize,
-  IN OUT   UINTN    *SizeOfBuffer,
-  IN CONST VOID     *Buffer
+  OUT      VOID   *PatchVariable,
+  IN       UINTN  MaximumDatumSize,
+  IN OUT   UINTN  *SizeOfBuffer,
+  IN CONST VOID   *Buffer
   )
 {
   ASSERT (PatchVariable != NULL);
@@ -1435,7 +992,8 @@ LibPatchPcdSetPtrS (
   }
 
   if ((*SizeOfBuffer > MaximumDatumSize) ||
-      (*SizeOfBuffer == MAX_ADDRESS)) {
+      (*SizeOfBuffer == MAX_ADDRESS))
+  {
     *SizeOfBuffer = MaximumDatumSize;
     return RETURN_INVALID_PARAMETER;
   }
@@ -1445,7 +1003,6 @@ LibPatchPcdSetPtrS (
   return RETURN_SUCCESS;
 }
 
-
 /**
   Sets a value and size of a patchable PCD entry that is type pointer.
 
@@ -1474,11 +1031,11 @@ LibPatchPcdSetPtrS (
 VOID *
 EFIAPI
 LibPatchPcdSetPtrAndSize (
-  OUT       VOID        *PatchVariable,
-  OUT       UINTN       *SizeOfPatchVariable,
-  IN        UINTN       MaximumDatumSize,
-  IN OUT    UINTN       *SizeOfBuffer,
-  IN CONST  VOID        *Buffer
+  OUT       VOID   *PatchVariable,
+  OUT       UINTN  *SizeOfPatchVariable,
+  IN        UINTN  MaximumDatumSize,
+  IN OUT    UINTN  *SizeOfBuffer,
+  IN CONST  VOID   *Buffer
   )
 {
   ASSERT (PatchVariable != NULL);
@@ -1490,7 +1047,8 @@ LibPatchPcdSetPtrAndSize (
   }
 
   if ((*SizeOfBuffer > MaximumDatumSize) ||
-      (*SizeOfBuffer == MAX_ADDRESS)) {
+      (*SizeOfBuffer == MAX_ADDRESS))
+  {
     *SizeOfBuffer = MaximumDatumSize;
     return NULL;
   }
@@ -1498,7 +1056,7 @@ LibPatchPcdSetPtrAndSize (
   CopyMem (PatchVariable, Buffer, *SizeOfBuffer);
   *SizeOfPatchVariable = *SizeOfBuffer;
 
-  return (VOID *) Buffer;
+  return (VOID *)Buffer;
 }
 
 /**
@@ -1529,11 +1087,11 @@ LibPatchPcdSetPtrAndSize (
 RETURN_STATUS
 EFIAPI
 LibPatchPcdSetPtrAndSizeS (
-  OUT      VOID     *PatchVariable,
-  OUT      UINTN    *SizeOfPatchVariable,
-  IN       UINTN    MaximumDatumSize,
-  IN OUT   UINTN    *SizeOfBuffer,
-  IN CONST VOID     *Buffer
+  OUT      VOID   *PatchVariable,
+  OUT      UINTN  *SizeOfPatchVariable,
+  IN       UINTN  MaximumDatumSize,
+  IN OUT   UINTN  *SizeOfBuffer,
+  IN CONST VOID   *Buffer
   )
 {
   ASSERT (PatchVariable != NULL);
@@ -1545,7 +1103,8 @@ LibPatchPcdSetPtrAndSizeS (
   }
 
   if ((*SizeOfBuffer > MaximumDatumSize) ||
-      (*SizeOfBuffer == MAX_ADDRESS)) {
+      (*SizeOfBuffer == MAX_ADDRESS))
+  {
     *SizeOfBuffer = MaximumDatumSize;
     return RETURN_INVALID_PARAMETER;
   }
@@ -1571,13 +1130,13 @@ LibPatchPcdSetPtrAndSizeS (
 VOID
 EFIAPI
 LibPcdGetInfo (
-  IN        UINTN           TokenNumber,
-  OUT       PCD_INFO        *PcdInfo
+  IN        UINTN     TokenNumber,
+  OUT       PCD_INFO  *PcdInfo
   )
 {
-  EFI_STATUS Status;
+  EFI_STATUS  Status;
 
-  Status = GetPcdInfoPpiPointer()->GetInfo (TokenNumber, (EFI_PCD_INFO *) PcdInfo);
+  Status = GetPcdInfoPpiPointer ()->GetInfo (TokenNumber, (EFI_PCD_INFO *)PcdInfo);
   ASSERT_EFI_ERROR (Status);
 }
 
@@ -1597,14 +1156,14 @@ LibPcdGetInfo (
 VOID
 EFIAPI
 LibPcdGetInfoEx (
-  IN CONST  GUID            *Guid,
-  IN        UINTN           TokenNumber,
-  OUT       PCD_INFO        *PcdInfo
+  IN CONST  GUID      *Guid,
+  IN        UINTN     TokenNumber,
+  OUT       PCD_INFO  *PcdInfo
   )
 {
-  EFI_STATUS Status;
+  EFI_STATUS  Status;
 
-  Status = GetPiPcdInfoPpiPointer()->GetInfo (Guid, TokenNumber, (EFI_PCD_INFO *) PcdInfo);
+  Status = GetPiPcdInfoPpiPointer ()->GetInfo (Guid, TokenNumber, (EFI_PCD_INFO *)PcdInfo);
   ASSERT_EFI_ERROR (Status);
 }
 
@@ -1621,5 +1180,5 @@ LibPcdGetSku (
   VOID
   )
 {
-  return GetPiPcdInfoPpiPointer()->GetSku ();
+  return GetPiPcdInfoPpiPointer ()->GetSku ();
 }
