@@ -1,23 +1,17 @@
 /** @file
-  Driver Binding functions implementationfor for UefiPxeBc Driver.
+  Driver Binding functions implementation for UefiPxeBc Driver.
 
   (C) Copyright 2014 Hewlett-Packard Development Company, L.P.<BR>
-  Copyright (c) 2007 - 2018, Intel Corporation. All rights reserved.<BR>
+  Copyright (c) 2007 - 2019, Intel Corporation. All rights reserved.<BR>
+  Copyright (c) Microsoft Corporation
 
-  This program and the accompanying materials
-  are licensed and made available under the terms and conditions of the BSD License
-  which accompanies this distribution.  The full text of the license may be found at
-  http://opensource.org/licenses/bsd-license.php.
-
-  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-  WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+  SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
 #include "PxeBcImpl.h"
 
-
-EFI_DRIVER_BINDING_PROTOCOL gPxeBcIp4DriverBinding = {
+EFI_DRIVER_BINDING_PROTOCOL  gPxeBcIp4DriverBinding = {
   PxeBcIp4DriverBindingSupported,
   PxeBcIp4DriverBindingStart,
   PxeBcIp4DriverBindingStop,
@@ -26,7 +20,7 @@ EFI_DRIVER_BINDING_PROTOCOL gPxeBcIp4DriverBinding = {
   NULL
 };
 
-EFI_DRIVER_BINDING_PROTOCOL gPxeBcIp6DriverBinding = {
+EFI_DRIVER_BINDING_PROTOCOL  gPxeBcIp6DriverBinding = {
   PxeBcIp6DriverBindingSupported,
   PxeBcIp6DriverBindingStart,
   PxeBcIp6DriverBindingStop,
@@ -45,10 +39,10 @@ EFI_DRIVER_BINDING_PROTOCOL gPxeBcIp6DriverBinding = {
 **/
 EFI_HANDLE
 PxeBcGetNicByIp4Children (
-  IN EFI_HANDLE                 ControllerHandle
+  IN EFI_HANDLE  ControllerHandle
   )
 {
-  EFI_HANDLE                    NicHandle;
+  EFI_HANDLE  NicHandle;
 
   NicHandle = NetLibGetNicHandle (ControllerHandle, &gEfiArpProtocolGuid);
   if (NicHandle == NULL) {
@@ -70,7 +64,6 @@ PxeBcGetNicByIp4Children (
   return NicHandle;
 }
 
-
 /**
   Get the Nic handle using any child handle in the IPv6 stack.
 
@@ -81,10 +74,10 @@ PxeBcGetNicByIp4Children (
 **/
 EFI_HANDLE
 PxeBcGetNicByIp6Children (
-  IN EFI_HANDLE                  ControllerHandle
+  IN EFI_HANDLE  ControllerHandle
   )
 {
-  EFI_HANDLE                     NicHandle;
+  EFI_HANDLE  NicHandle;
 
   NicHandle = NetLibGetNicHandle (ControllerHandle, &gEfiIp6ProtocolGuid);
   if (NicHandle == NULL) {
@@ -103,7 +96,6 @@ PxeBcGetNicByIp6Children (
   return NicHandle;
 }
 
-
 /**
   Destroy the opened instances based on IPv4.
 
@@ -117,7 +109,7 @@ PxeBcDestroyIp4Children (
   IN PXEBC_PRIVATE_DATA           *Private
   )
 {
-  ASSERT(Private != NULL);
+  ASSERT (Private != NULL);
 
   if (Private->ArpChild != NULL) {
     //
@@ -181,11 +173,11 @@ PxeBcDestroyIp4Children (
     // Close Udp4 for PxeBc->UdpRead and destroy the instance.
     //
     gBS->CloseProtocol (
-          Private->Udp4ReadChild,
-          &gEfiUdp4ProtocolGuid,
-          This->DriverBindingHandle,
-          Private->Controller
-          );
+           Private->Udp4ReadChild,
+           &gEfiUdp4ProtocolGuid,
+           This->DriverBindingHandle,
+           Private->Controller
+           );
 
     NetLibDestroyServiceChild (
       Private->Controller,
@@ -200,11 +192,11 @@ PxeBcDestroyIp4Children (
     // Close Mtftp4 for PxeBc->Mtftp4 and destroy the instance.
     //
     gBS->CloseProtocol (
-          Private->Mtftp4Child,
-          &gEfiMtftp4ProtocolGuid,
-          This->DriverBindingHandle,
-          Private->Controller
-          );
+           Private->Mtftp4Child,
+           &gEfiMtftp4ProtocolGuid,
+           This->DriverBindingHandle,
+           Private->Controller
+           );
 
     NetLibDestroyServiceChild (
       Private->Controller,
@@ -219,11 +211,11 @@ PxeBcDestroyIp4Children (
     // Close Dhcp4 for PxeBc->Dhcp4 and destroy the instance.
     //
     gBS->CloseProtocol (
-          Private->Dhcp4Child,
-          &gEfiDhcp4ProtocolGuid,
-          This->DriverBindingHandle,
-          Private->Controller
-          );
+           Private->Dhcp4Child,
+           &gEfiDhcp4ProtocolGuid,
+           This->DriverBindingHandle,
+           Private->Controller
+           );
 
     NetLibDestroyServiceChild (
       Private->Controller,
@@ -254,7 +246,9 @@ PxeBcDestroyIp4Children (
            &Private->PxeBc,
            NULL
            );
-    FreePool (Private->Ip4Nic->DevicePath);
+    if (Private->Ip4Nic->DevicePath != NULL) {
+      FreePool (Private->Ip4Nic->DevicePath);
+    }
 
     if (Private->Snp != NULL) {
       //
@@ -273,19 +267,19 @@ PxeBcDestroyIp4Children (
              Private->Snp
              );
     }
+
     FreePool (Private->Ip4Nic);
   }
 
-  Private->ArpChild         = NULL;
-  Private->Ip4Child         = NULL;
-  Private->Udp4WriteChild   = NULL;
-  Private->Udp4ReadChild    = NULL;
-  Private->Mtftp4Child      = NULL;
-  Private->Dhcp4Child       = NULL;
-  Private->Ip4Nic           = NULL;
+  Private->ArpChild       = NULL;
+  Private->Ip4Child       = NULL;
+  Private->Udp4WriteChild = NULL;
+  Private->Udp4ReadChild  = NULL;
+  Private->Mtftp4Child    = NULL;
+  Private->Dhcp4Child     = NULL;
+  Private->Ip4Nic         = NULL;
 }
 
-
 /**
   Destroy the opened instances based on IPv6.
 
@@ -299,18 +293,18 @@ PxeBcDestroyIp6Children (
   IN PXEBC_PRIVATE_DATA           *Private
   )
 {
-  ASSERT(Private != NULL);
+  ASSERT (Private != NULL);
 
   if (Private->Ip6Child != NULL) {
     //
     // Close Ip6 for Ip6->Ip6Config and destroy the instance.
     //
     gBS->CloseProtocol (
-          Private->Ip6Child,
-          &gEfiIp6ProtocolGuid,
-          This->DriverBindingHandle,
-          Private->Controller
-          );
+           Private->Ip6Child,
+           &gEfiIp6ProtocolGuid,
+           This->DriverBindingHandle,
+           Private->Controller
+           );
 
     NetLibDestroyServiceChild (
       Private->Controller,
@@ -343,11 +337,11 @@ PxeBcDestroyIp6Children (
     // Close Udp6 for PxeBc->UdpRead and destroy the instance.
     //
     gBS->CloseProtocol (
-          Private->Udp6ReadChild,
-          &gEfiUdp6ProtocolGuid,
-          This->DriverBindingHandle,
-          Private->Controller
-          );
+           Private->Udp6ReadChild,
+           &gEfiUdp6ProtocolGuid,
+           This->DriverBindingHandle,
+           Private->Controller
+           );
     NetLibDestroyServiceChild (
       Private->Controller,
       This->DriverBindingHandle,
@@ -361,11 +355,11 @@ PxeBcDestroyIp6Children (
     // Close Mtftp6 for PxeBc->Mtftp and destroy the instance.
     //
     gBS->CloseProtocol (
-          Private->Mtftp6Child,
-          &gEfiMtftp6ProtocolGuid,
-          This->DriverBindingHandle,
-          Private->Controller
-          );
+           Private->Mtftp6Child,
+           &gEfiMtftp6ProtocolGuid,
+           This->DriverBindingHandle,
+           Private->Controller
+           );
 
     NetLibDestroyServiceChild (
       Private->Controller,
@@ -380,11 +374,11 @@ PxeBcDestroyIp6Children (
     // Close Dhcp6 for PxeBc->Dhcp and destroy the instance.
     //
     gBS->CloseProtocol (
-          Private->Dhcp6Child,
-          &gEfiDhcp6ProtocolGuid,
-          This->DriverBindingHandle,
-          Private->Controller
-          );
+           Private->Dhcp6Child,
+           &gEfiDhcp6ProtocolGuid,
+           This->DriverBindingHandle,
+           Private->Controller
+           );
 
     NetLibDestroyServiceChild (
       Private->Controller,
@@ -415,7 +409,9 @@ PxeBcDestroyIp6Children (
            &Private->PxeBc,
            NULL
            );
-    FreePool (Private->Ip6Nic->DevicePath);
+    if (Private->Ip6Nic->DevicePath != NULL) {
+      FreePool (Private->Ip6Nic->DevicePath);
+    }
 
     if (Private->Snp != NULL) {
       //
@@ -433,6 +429,7 @@ PxeBcDestroyIp6Children (
              Private->Snp
              );
     }
+
     FreePool (Private->Ip6Nic);
   }
 
@@ -458,20 +455,20 @@ PxeBcDestroyIp6Children (
 **/
 EFI_STATUS
 PxeBcCheckIpv6Support (
-  IN  EFI_HANDLE                   ControllerHandle,
-  IN  PXEBC_PRIVATE_DATA           *Private,
-  OUT BOOLEAN                      *Ipv6Support
+  IN  EFI_HANDLE          ControllerHandle,
+  IN  PXEBC_PRIVATE_DATA  *Private,
+  OUT BOOLEAN             *Ipv6Support
   )
 {
-  EFI_HANDLE                       Handle;
-  EFI_ADAPTER_INFORMATION_PROTOCOL *Aip;
-  EFI_STATUS                       Status;
-  EFI_GUID                         *InfoTypesBuffer;
-  UINTN                            InfoTypeBufferCount;
-  UINTN                            TypeIndex;
-  BOOLEAN                          Supported;
-  VOID                             *InfoBlock;
-  UINTN                            InfoBlockSize;
+  EFI_HANDLE                        Handle;
+  EFI_ADAPTER_INFORMATION_PROTOCOL  *Aip;
+  EFI_STATUS                        Status;
+  EFI_GUID                          *InfoTypesBuffer;
+  UINTN                             InfoTypeBufferCount;
+  UINTN                             TypeIndex;
+  BOOLEAN                           Supported;
+  VOID                              *InfoBlock;
+  UINTN                             InfoBlockSize;
 
   ASSERT (Private != NULL && Ipv6Support != NULL);
 
@@ -499,16 +496,16 @@ PxeBcCheckIpv6Support (
   Status = gBS->HandleProtocol (
                   Handle,
                   &gEfiAdapterInformationProtocolGuid,
-                  (VOID *) &Aip
+                  (VOID *)&Aip
                   );
-  if (EFI_ERROR (Status) || Aip == NULL) {
+  if (EFI_ERROR (Status) || (Aip == NULL)) {
     return EFI_NOT_FOUND;
   }
 
   InfoTypesBuffer     = NULL;
   InfoTypeBufferCount = 0;
-  Status = Aip->GetSupportedTypes (Aip, &InfoTypesBuffer, &InfoTypeBufferCount);
-  if (EFI_ERROR (Status) || InfoTypesBuffer == NULL) {
+  Status              = Aip->GetSupportedTypes (Aip, &InfoTypesBuffer, &InfoTypeBufferCount);
+  if (EFI_ERROR (Status) || (InfoTypesBuffer == NULL)) {
     FreePool (InfoTypesBuffer);
     return EFI_NOT_FOUND;
   }
@@ -531,16 +528,15 @@ PxeBcCheckIpv6Support (
   //
   InfoBlock     = NULL;
   InfoBlockSize = 0;
-  Status = Aip->GetInformation (Aip, &gEfiAdapterInfoUndiIpv6SupportGuid, &InfoBlock, &InfoBlockSize);
-  if (EFI_ERROR (Status) || InfoBlock == NULL) {
+  Status        = Aip->GetInformation (Aip, &gEfiAdapterInfoUndiIpv6SupportGuid, &InfoBlock, &InfoBlockSize);
+  if (EFI_ERROR (Status) || (InfoBlock == NULL)) {
     FreePool (InfoBlock);
     return EFI_NOT_FOUND;
   }
 
-  *Ipv6Support = ((EFI_ADAPTER_INFO_UNDI_IPV6_SUPPORT *) InfoBlock)->Ipv6Support;
+  *Ipv6Support = ((EFI_ADAPTER_INFO_UNDI_IPV6_SUPPORT *)InfoBlock)->Ipv6Support;
   FreePool (InfoBlock);
   return EFI_SUCCESS;
-
 }
 
 /**
@@ -561,14 +557,14 @@ PxeBcCreateIp4Children (
   IN PXEBC_PRIVATE_DATA           *Private
   )
 {
-  EFI_STATUS                      Status;
-  IPv4_DEVICE_PATH                Ip4Node;
-  EFI_PXE_BASE_CODE_MODE          *Mode;
-  EFI_UDP4_CONFIG_DATA            *Udp4CfgData;
-  EFI_IP4_CONFIG_DATA             *Ip4CfgData;
-  EFI_IP4_MODE_DATA               Ip4ModeData;
-  PXEBC_PRIVATE_PROTOCOL          *Id;
-  EFI_SIMPLE_NETWORK_PROTOCOL     *Snp;
+  EFI_STATUS                   Status;
+  IPv4_DEVICE_PATH             Ip4Node;
+  EFI_PXE_BASE_CODE_MODE       *Mode;
+  EFI_UDP4_CONFIG_DATA         *Udp4CfgData;
+  EFI_IP4_CONFIG_DATA          *Ip4CfgData;
+  EFI_IP4_MODE_DATA            Ip4ModeData;
+  PXEBC_PRIVATE_PROTOCOL       *Id;
+  EFI_SIMPLE_NETWORK_PROTOCOL  *Snp;
 
   if (Private->Ip4Nic != NULL) {
     //
@@ -593,7 +589,7 @@ PxeBcCreateIp4Children (
   Status = gBS->OpenProtocol (
                   Private->Dhcp4Child,
                   &gEfiDhcp4ProtocolGuid,
-                  (VOID **) &Private->Dhcp4,
+                  (VOID **)&Private->Dhcp4,
                   This->DriverBindingHandle,
                   ControllerHandle,
                   EFI_OPEN_PROTOCOL_BY_DRIVER
@@ -618,7 +614,7 @@ PxeBcCreateIp4Children (
   Status = gBS->OpenProtocol (
                   Private->Mtftp4Child,
                   &gEfiMtftp4ProtocolGuid,
-                  (VOID **) &Private->Mtftp4,
+                  (VOID **)&Private->Mtftp4,
                   This->DriverBindingHandle,
                   ControllerHandle,
                   EFI_OPEN_PROTOCOL_BY_DRIVER
@@ -643,7 +639,7 @@ PxeBcCreateIp4Children (
   Status = gBS->OpenProtocol (
                   Private->Udp4ReadChild,
                   &gEfiUdp4ProtocolGuid,
-                  (VOID **) &Private->Udp4Read,
+                  (VOID **)&Private->Udp4Read,
                   This->DriverBindingHandle,
                   ControllerHandle,
                   EFI_OPEN_PROTOCOL_BY_DRIVER
@@ -668,7 +664,7 @@ PxeBcCreateIp4Children (
   Status = gBS->OpenProtocol (
                   Private->Udp4WriteChild,
                   &gEfiUdp4ProtocolGuid,
-                  (VOID **) &Private->Udp4Write,
+                  (VOID **)&Private->Udp4Write,
                   This->DriverBindingHandle,
                   ControllerHandle,
                   EFI_OPEN_PROTOCOL_BY_DRIVER
@@ -693,7 +689,7 @@ PxeBcCreateIp4Children (
   Status = gBS->OpenProtocol (
                   Private->ArpChild,
                   &gEfiArpProtocolGuid,
-                  (VOID **) &Private->Arp,
+                  (VOID **)&Private->Arp,
                   This->DriverBindingHandle,
                   ControllerHandle,
                   EFI_OPEN_PROTOCOL_BY_DRIVER
@@ -718,7 +714,7 @@ PxeBcCreateIp4Children (
   Status = gBS->OpenProtocol (
                   Private->Ip4Child,
                   &gEfiIp4ProtocolGuid,
-                  (VOID **) &Private->Ip4,
+                  (VOID **)&Private->Ip4,
                   This->DriverBindingHandle,
                   ControllerHandle,
                   EFI_OPEN_PROTOCOL_BY_DRIVER
@@ -745,13 +741,13 @@ PxeBcCreateIp4Children (
   Private->Ip4Nic->Private   = Private;
   Private->Ip4Nic->Signature = PXEBC_VIRTUAL_NIC_SIGNATURE;
 
-   //
+  //
   // Locate Ip4->Ip4Config2 and store it for set IPv4 Policy.
   //
   Status = gBS->HandleProtocol (
                   ControllerHandle,
                   &gEfiIp4Config2ProtocolGuid,
-                  (VOID **) &Private->Ip4Config2
+                  (VOID **)&Private->Ip4Config2
                   );
   if (EFI_ERROR (Status)) {
     goto ON_ERROR;
@@ -820,7 +816,7 @@ PxeBcCreateIp4Children (
     Status = gBS->OpenProtocol (
                     Private->Ip4Nic->Controller,
                     &gEfiSimpleNetworkProtocolGuid,
-                    (VOID **) &Snp,
+                    (VOID **)&Snp,
                     This->DriverBindingHandle,
                     Private->Ip4Nic->Controller,
                     EFI_OPEN_PROTOCOL_BY_DRIVER|EFI_OPEN_PROTOCOL_EXCLUSIVE
@@ -837,7 +833,7 @@ PxeBcCreateIp4Children (
   Status = gBS->OpenProtocol (
                   ControllerHandle,
                   &gEfiCallerIdGuid,
-                  (VOID **) &Id,
+                  (VOID **)&Id,
                   This->DriverBindingHandle,
                   Private->Ip4Nic->Controller,
                   EFI_OPEN_PROTOCOL_BY_CHILD_CONTROLLER
@@ -849,9 +845,9 @@ PxeBcCreateIp4Children (
   //
   // Set default configure data for Udp4Read and Ip4 instance.
   //
-  Mode                            = Private->PxeBc.Mode;
-  Udp4CfgData                     = &Private->Udp4CfgData;
-  Ip4CfgData                      = &Private->Ip4CfgData;
+  Mode        = Private->PxeBc.Mode;
+  Udp4CfgData = &Private->Udp4CfgData;
+  Ip4CfgData  = &Private->Ip4CfgData;
 
   Udp4CfgData->AcceptBroadcast    = FALSE;
   Udp4CfgData->AcceptAnyPort      = TRUE;
@@ -861,12 +857,12 @@ PxeBcCreateIp4Children (
   Udp4CfgData->ReceiveTimeout     = PXEBC_DEFAULT_LIFETIME;
   Udp4CfgData->TransmitTimeout    = PXEBC_DEFAULT_LIFETIME;
 
-  Ip4CfgData->AcceptIcmpErrors    = TRUE;
-  Ip4CfgData->DefaultProtocol     = EFI_IP_PROTO_ICMP;
-  Ip4CfgData->TypeOfService       = Mode->ToS;
-  Ip4CfgData->TimeToLive          = Mode->TTL;
-  Ip4CfgData->ReceiveTimeout      = PXEBC_DEFAULT_LIFETIME;
-  Ip4CfgData->TransmitTimeout     = PXEBC_DEFAULT_LIFETIME;
+  Ip4CfgData->AcceptIcmpErrors = TRUE;
+  Ip4CfgData->DefaultProtocol  = EFI_IP_PROTO_ICMP;
+  Ip4CfgData->TypeOfService    = Mode->ToS;
+  Ip4CfgData->TimeToLive       = Mode->TTL;
+  Ip4CfgData->ReceiveTimeout   = PXEBC_DEFAULT_LIFETIME;
+  Ip4CfgData->TransmitTimeout  = PXEBC_DEFAULT_LIFETIME;
 
   return EFI_SUCCESS;
 
@@ -875,7 +871,6 @@ ON_ERROR:
   return Status;
 }
 
-
 /**
   Create the opened instances based on IPv6.
 
@@ -894,14 +889,21 @@ PxeBcCreateIp6Children (
   IN PXEBC_PRIVATE_DATA           *Private
   )
 {
-  EFI_STATUS                      Status;
-  IPv6_DEVICE_PATH                Ip6Node;
-  EFI_UDP6_CONFIG_DATA            *Udp6CfgData;
-  EFI_IP6_CONFIG_DATA             *Ip6CfgData;
-  EFI_IP6_MODE_DATA               Ip6ModeData;
-  PXEBC_PRIVATE_PROTOCOL          *Id;
-  EFI_SIMPLE_NETWORK_PROTOCOL     *Snp;
-  UINTN                           Index;
+  EFI_STATUS                   Status;
+  IPv6_DEVICE_PATH             Ip6Node;
+  EFI_UDP6_CONFIG_DATA         *Udp6CfgData;
+  EFI_IP6_CONFIG_DATA          *Ip6CfgData;
+  EFI_IP6_MODE_DATA            Ip6ModeData;
+  PXEBC_PRIVATE_PROTOCOL       *Id;
+  EFI_SIMPLE_NETWORK_PROTOCOL  *Snp;
+  UINTN                        Index;
+  UINT32                       Random;
+
+  Status = PseudoRandomU32 (&Random);
+  if (EFI_ERROR (Status)) {
+    DEBUG ((DEBUG_ERROR, "Failed to generate random number using EFI_RNG_PROTOCOL: %r\n", Status));
+    return Status;
+  }
 
   if (Private->Ip6Nic != NULL) {
     //
@@ -935,7 +937,7 @@ PxeBcCreateIp6Children (
   Status = gBS->OpenProtocol (
                   Private->Dhcp6Child,
                   &gEfiDhcp6ProtocolGuid,
-                  (VOID **) &Private->Dhcp6,
+                  (VOID **)&Private->Dhcp6,
                   This->DriverBindingHandle,
                   ControllerHandle,
                   EFI_OPEN_PROTOCOL_BY_DRIVER
@@ -945,9 +947,9 @@ PxeBcCreateIp6Children (
   }
 
   //
-  // Generate a random IAID for the Dhcp6 assigned address.
+  // Set a random IAID for the Dhcp6 assigned address.
   //
-  Private->IaId = NET_RANDOM (NetRandomInitSeed ());
+  Private->IaId = Random;
   if (Private->Snp != NULL) {
     for (Index = 0; Index < Private->Snp->Mode->HwAddressSize; Index++) {
       Private->IaId |= (Private->Snp->Mode->CurrentAddress.Addr[Index] << ((Index << 3) & 31));
@@ -970,7 +972,7 @@ PxeBcCreateIp6Children (
   Status = gBS->OpenProtocol (
                   Private->Mtftp6Child,
                   &gEfiMtftp6ProtocolGuid,
-                  (VOID **) &Private->Mtftp6,
+                  (VOID **)&Private->Mtftp6,
                   This->DriverBindingHandle,
                   ControllerHandle,
                   EFI_OPEN_PROTOCOL_BY_DRIVER
@@ -995,7 +997,7 @@ PxeBcCreateIp6Children (
   Status = gBS->OpenProtocol (
                   Private->Udp6ReadChild,
                   &gEfiUdp6ProtocolGuid,
-                  (VOID **) &Private->Udp6Read,
+                  (VOID **)&Private->Udp6Read,
                   This->DriverBindingHandle,
                   ControllerHandle,
                   EFI_OPEN_PROTOCOL_BY_DRIVER
@@ -1020,7 +1022,7 @@ PxeBcCreateIp6Children (
   Status = gBS->OpenProtocol (
                   Private->Udp6WriteChild,
                   &gEfiUdp6ProtocolGuid,
-                  (VOID **) &Private->Udp6Write,
+                  (VOID **)&Private->Udp6Write,
                   This->DriverBindingHandle,
                   ControllerHandle,
                   EFI_OPEN_PROTOCOL_BY_DRIVER
@@ -1045,7 +1047,7 @@ PxeBcCreateIp6Children (
   Status = gBS->OpenProtocol (
                   Private->Ip6Child,
                   &gEfiIp6ProtocolGuid,
-                  (VOID **) &Private->Ip6,
+                  (VOID **)&Private->Ip6,
                   This->DriverBindingHandle,
                   ControllerHandle,
                   EFI_OPEN_PROTOCOL_BY_DRIVER
@@ -1094,7 +1096,7 @@ PxeBcCreateIp6Children (
   Status = gBS->HandleProtocol (
                   ControllerHandle,
                   &gEfiIp6ConfigProtocolGuid,
-                  (VOID **) &Private->Ip6Cfg
+                  (VOID **)&Private->Ip6Cfg
                   );
   if (EFI_ERROR (Status)) {
     goto ON_ERROR;
@@ -1104,9 +1106,9 @@ PxeBcCreateIp6Children (
   // Create a device path node for Ipv6 virtual nic, and append it.
   //
   ZeroMem (&Ip6Node, sizeof (IPv6_DEVICE_PATH));
-  Ip6Node.Header.Type     = MESSAGING_DEVICE_PATH;
-  Ip6Node.Header.SubType  = MSG_IPv6_DP;
-  Ip6Node.PrefixLength    = IP6_PREFIX_LENGTH;
+  Ip6Node.Header.Type    = MESSAGING_DEVICE_PATH;
+  Ip6Node.Header.SubType = MSG_IPv6_DP;
+  Ip6Node.PrefixLength   = IP6_PREFIX_LENGTH;
 
   SetDevicePathNodeLength (&Ip6Node.Header, sizeof (Ip6Node));
 
@@ -1163,7 +1165,7 @@ PxeBcCreateIp6Children (
     Status = gBS->OpenProtocol (
                     Private->Ip6Nic->Controller,
                     &gEfiSimpleNetworkProtocolGuid,
-                    (VOID **) &Snp,
+                    (VOID **)&Snp,
                     This->DriverBindingHandle,
                     Private->Ip6Nic->Controller,
                     EFI_OPEN_PROTOCOL_BY_DRIVER|EFI_OPEN_PROTOCOL_EXCLUSIVE
@@ -1180,7 +1182,7 @@ PxeBcCreateIp6Children (
   Status = gBS->OpenProtocol (
                   ControllerHandle,
                   &gEfiCallerIdGuid,
-                  (VOID **) &Id,
+                  (VOID **)&Id,
                   This->DriverBindingHandle,
                   Private->Ip6Nic->Controller,
                   EFI_OPEN_PROTOCOL_BY_CHILD_CONTROLLER
@@ -1190,7 +1192,7 @@ PxeBcCreateIp6Children (
   }
 
   //
-  // Set IPv6 avaiable flag and set default configure data for
+  // Set IPv6 available flag and set default configure data for
   // Udp6Read and Ip6 instance.
   //
   Status = PxeBcCheckIpv6Support (ControllerHandle, Private, &Private->Mode.Ipv6Available);
@@ -1198,15 +1200,15 @@ PxeBcCreateIp6Children (
     //
     // Fail to get the data whether UNDI supports IPv6. Set default value.
     //
-    Private->Mode.Ipv6Available   = TRUE;
+    Private->Mode.Ipv6Available = TRUE;
   }
 
   if (!Private->Mode.Ipv6Available) {
     goto ON_ERROR;
   }
 
-  Udp6CfgData                     = &Private->Udp6CfgData;
-  Ip6CfgData                      = &Private->Ip6CfgData;
+  Udp6CfgData = &Private->Udp6CfgData;
+  Ip6CfgData  = &Private->Ip6CfgData;
 
   Udp6CfgData->AcceptAnyPort      = TRUE;
   Udp6CfgData->AllowDuplicatePort = TRUE;
@@ -1214,11 +1216,11 @@ PxeBcCreateIp6Children (
   Udp6CfgData->ReceiveTimeout     = PXEBC_DEFAULT_LIFETIME;
   Udp6CfgData->TransmitTimeout    = PXEBC_DEFAULT_LIFETIME;
 
-  Ip6CfgData->AcceptIcmpErrors    = TRUE;
-  Ip6CfgData->DefaultProtocol     = IP6_ICMP;
-  Ip6CfgData->HopLimit            = PXEBC_DEFAULT_HOPLIMIT;
-  Ip6CfgData->ReceiveTimeout      = PXEBC_DEFAULT_LIFETIME;
-  Ip6CfgData->TransmitTimeout     = PXEBC_DEFAULT_LIFETIME;
+  Ip6CfgData->AcceptIcmpErrors = TRUE;
+  Ip6CfgData->DefaultProtocol  = IP6_ICMP;
+  Ip6CfgData->HopLimit         = PXEBC_DEFAULT_HOPLIMIT;
+  Ip6CfgData->ReceiveTimeout   = PXEBC_DEFAULT_LIFETIME;
+  Ip6CfgData->TransmitTimeout  = PXEBC_DEFAULT_LIFETIME;
 
   return EFI_SUCCESS;
 
@@ -1227,7 +1229,6 @@ ON_ERROR:
   return Status;
 }
 
-
 /**
   The entry point for UefiPxeBc driver that installs the driver
   binding and component name protocol on its image.
@@ -1242,12 +1243,16 @@ ON_ERROR:
 EFI_STATUS
 EFIAPI
 PxeBcDriverEntryPoint (
-  IN EFI_HANDLE             ImageHandle,
-  IN EFI_SYSTEM_TABLE       *SystemTable
+  IN EFI_HANDLE        ImageHandle,
+  IN EFI_SYSTEM_TABLE  *SystemTable
   )
 {
   EFI_STATUS  Status;
 
+  if ((PcdGet8 (PcdIPv4PXESupport) == PXE_DISABLED) && (PcdGet8 (PcdIPv6PXESupport) == PXE_DISABLED)) {
+    return EFI_UNSUPPORTED;
+  }
+
   Status = EfiLibInstallDriverBindingComponentName2 (
              ImageHandle,
              SystemTable,
@@ -1269,16 +1274,11 @@ PxeBcDriverEntryPoint (
              &gPxeBcComponentName2
              );
   if (EFI_ERROR (Status)) {
-    gBS->UninstallMultipleProtocolInterfaces (
-           ImageHandle,
-           &gEfiDriverBindingProtocolGuid,
-           &gPxeBcIp4DriverBinding,
-           &gEfiComponentName2ProtocolGuid,
-           &gPxeBcComponentName2,
-           &gEfiComponentNameProtocolGuid,
-           &gPxeBcComponentName,
-           NULL
-           );
+    EfiLibUninstallDriverBindingComponentName2 (
+      &gPxeBcIp4DriverBinding,
+      &gPxeBcComponentName,
+      &gPxeBcComponentName2
+      );
   }
 
   return Status;
@@ -1307,14 +1307,22 @@ PxeBcSupported (
   IN UINT8                        IpVersion
   )
 {
-  EFI_STATUS                      Status;
-  EFI_GUID                        *DhcpServiceBindingGuid;
-  EFI_GUID                        *MtftpServiceBindingGuid;
+  EFI_STATUS  Status;
+  EFI_GUID    *DhcpServiceBindingGuid;
+  EFI_GUID    *MtftpServiceBindingGuid;
 
   if (IpVersion == IP_VERSION_4) {
+    if (PcdGet8 (PcdIPv4PXESupport) == PXE_DISABLED) {
+      return EFI_UNSUPPORTED;
+    }
+
     DhcpServiceBindingGuid  = &gEfiDhcp4ServiceBindingProtocolGuid;
     MtftpServiceBindingGuid = &gEfiMtftp4ServiceBindingProtocolGuid;
   } else {
+    if (PcdGet8 (PcdIPv6PXESupport) == PXE_DISABLED) {
+      return EFI_UNSUPPORTED;
+    }
+
     DhcpServiceBindingGuid  = &gEfiDhcp6ServiceBindingProtocolGuid;
     MtftpServiceBindingGuid = &gEfiMtftp6ServiceBindingProtocolGuid;
   }
@@ -1323,22 +1331,22 @@ PxeBcSupported (
   // Try to open the Mtftp and Dhcp protocol to test whether IP stack is ready.
   //
   Status = gBS->OpenProtocol (
-                     ControllerHandle,
-                     DhcpServiceBindingGuid,
-                     NULL,
-                     This->DriverBindingHandle,
-                     ControllerHandle,
-                     EFI_OPEN_PROTOCOL_TEST_PROTOCOL
-                     );
+                  ControllerHandle,
+                  DhcpServiceBindingGuid,
+                  NULL,
+                  This->DriverBindingHandle,
+                  ControllerHandle,
+                  EFI_OPEN_PROTOCOL_TEST_PROTOCOL
+                  );
   if (!EFI_ERROR (Status)) {
     Status = gBS->OpenProtocol (
-                       ControllerHandle,
-                       MtftpServiceBindingGuid,
-                       NULL,
-                       This->DriverBindingHandle,
-                       ControllerHandle,
-                       EFI_OPEN_PROTOCOL_TEST_PROTOCOL
-                       );
+                    ControllerHandle,
+                    MtftpServiceBindingGuid,
+                    NULL,
+                    This->DriverBindingHandle,
+                    ControllerHandle,
+                    EFI_OPEN_PROTOCOL_TEST_PROTOCOL
+                    );
   }
 
   //
@@ -1376,20 +1384,20 @@ PxeBcStart (
   IN UINT8                        IpVersion
   )
 {
-  PXEBC_PRIVATE_DATA              *Private;
-  EFI_STATUS                      Status;
-  PXEBC_PRIVATE_PROTOCOL          *Id;
-  BOOLEAN                         FirstStart;
+  PXEBC_PRIVATE_DATA      *Private;
+  EFI_STATUS              Status;
+  PXEBC_PRIVATE_PROTOCOL  *Id;
+  BOOLEAN                 FirstStart;
 
   FirstStart = FALSE;
-  Status = gBS->OpenProtocol (
-                  ControllerHandle,
-                  &gEfiCallerIdGuid,
-                  (VOID **) &Id,
-                  This->DriverBindingHandle,
-                  ControllerHandle,
-                  EFI_OPEN_PROTOCOL_GET_PROTOCOL
-                  );
+  Status     = gBS->OpenProtocol (
+                      ControllerHandle,
+                      &gEfiCallerIdGuid,
+                      (VOID **)&Id,
+                      This->DriverBindingHandle,
+                      ControllerHandle,
+                      EFI_OPEN_PROTOCOL_GET_PROTOCOL
+                      );
   if (!EFI_ERROR (Status)) {
     //
     // Skip the initialization if the driver has been started already.
@@ -1426,7 +1434,7 @@ PxeBcStart (
     Status = gBS->OpenProtocol (
                     ControllerHandle,
                     &gEfiDevicePathProtocolGuid,
-                    (VOID **) &Private->DevicePath,
+                    (VOID **)&Private->DevicePath,
                     This->DriverBindingHandle,
                     ControllerHandle,
                     EFI_OPEN_PROTOCOL_GET_PROTOCOL
@@ -1442,7 +1450,7 @@ PxeBcStart (
     Status = gBS->OpenProtocol (
                     ControllerHandle,
                     &gEfiNetworkInterfaceIdentifierProtocolGuid_31,
-                    (VOID **) &Private->Nii,
+                    (VOID **)&Private->Nii,
                     This->DriverBindingHandle,
                     ControllerHandle,
                     EFI_OPEN_PROTOCOL_GET_PROTOCOL
@@ -1470,7 +1478,7 @@ PxeBcStart (
     //
     // Try to locate SNP protocol.
     //
-    NetLibGetSnpHandle(ControllerHandle, &Private->Snp);
+    NetLibGetSnpHandle (ControllerHandle, &Private->Snp);
   }
 
   if (IpVersion == IP_VERSION_4) {
@@ -1484,6 +1492,7 @@ PxeBcStart (
     //
     Status = PxeBcCreateIp6Children (This, ControllerHandle, Private);
   }
+
   if (EFI_ERROR (Status)) {
     //
     // Failed to start PXE driver if IPv4 and IPv6 stack are both not available.
@@ -1509,14 +1518,13 @@ ON_ERROR:
     PxeBcDestroyIp6Children (This, Private);
   }
 
-  if (FirstStart && Private != NULL) {
+  if (FirstStart && (Private != NULL)) {
     FreePool (Private);
   }
 
   return Status;
 }
 
-
 /**
   Stop this driver on ControllerHandle. This is the worker function for
   PxeBcIp4(6)DriverBindingStop.
@@ -1543,12 +1551,12 @@ PxeBcStop (
   IN UINT8                        IpVersion
   )
 {
-  PXEBC_PRIVATE_DATA              *Private;
-  PXEBC_VIRTUAL_NIC               *VirtualNic;
-  EFI_LOAD_FILE_PROTOCOL          *LoadFile;
-  EFI_STATUS                      Status;
-  EFI_HANDLE                      NicHandle;
-  PXEBC_PRIVATE_PROTOCOL          *Id;
+  PXEBC_PRIVATE_DATA      *Private;
+  PXEBC_VIRTUAL_NIC       *VirtualNic;
+  EFI_LOAD_FILE_PROTOCOL  *LoadFile;
+  EFI_STATUS              Status;
+  EFI_HANDLE              NicHandle;
+  PXEBC_PRIVATE_PROTOCOL  *Id;
 
   Private    = NULL;
   NicHandle  = NULL;
@@ -1559,7 +1567,7 @@ PxeBcStop (
   Status = gBS->OpenProtocol (
                   ControllerHandle,
                   &gEfiLoadFileProtocolGuid,
-                  (VOID **) &LoadFile,
+                  (VOID **)&LoadFile,
                   This->DriverBindingHandle,
                   ControllerHandle,
                   EFI_OPEN_PROTOCOL_GET_PROTOCOL
@@ -1573,6 +1581,7 @@ PxeBcStop (
     } else {
       NicHandle = PxeBcGetNicByIp6Children (ControllerHandle);
     }
+
     if (NicHandle == NULL) {
       return EFI_SUCCESS;
     }
@@ -1583,7 +1592,7 @@ PxeBcStop (
     Status = gBS->OpenProtocol (
                     NicHandle,
                     &gEfiCallerIdGuid,
-                    (VOID **) &Id,
+                    (VOID **)&Id,
                     This->DriverBindingHandle,
                     ControllerHandle,
                     EFI_OPEN_PROTOCOL_GET_PROTOCOL
@@ -1591,8 +1600,8 @@ PxeBcStop (
     if (EFI_ERROR (Status)) {
       return Status;
     }
-    Private = PXEBC_PRIVATE_DATA_FROM_ID (Id);
 
+    Private = PXEBC_PRIVATE_DATA_FROM_ID (Id);
   } else {
     //
     // It's a virtual handle with LoadFileProtocol.
@@ -1600,7 +1609,7 @@ PxeBcStop (
     Status = gBS->OpenProtocol (
                     ControllerHandle,
                     &gEfiLoadFileProtocolGuid,
-                    (VOID **) &LoadFile,
+                    (VOID **)&LoadFile,
                     This->DriverBindingHandle,
                     ControllerHandle,
                     EFI_OPEN_PROTOCOL_GET_PROTOCOL
@@ -1618,20 +1627,19 @@ PxeBcStop (
   // Stop functionality of PXE Base Code protocol
   //
   Status = Private->PxeBc.Stop (&Private->PxeBc);
-  if (Status != EFI_SUCCESS && Status != EFI_NOT_STARTED) {
+  if ((Status != EFI_SUCCESS) && (Status != EFI_NOT_STARTED)) {
     return Status;
   }
 
-
-  if (Private->Ip4Nic != NULL && IpVersion == IP_VERSION_4) {
+  if ((Private->Ip4Nic != NULL) && (IpVersion == IP_VERSION_4)) {
     PxeBcDestroyIp4Children (This, Private);
   }
 
-  if (Private->Ip6Nic != NULL && IpVersion == IP_VERSION_6) {
+  if ((Private->Ip6Nic != NULL) && (IpVersion == IP_VERSION_6)) {
     PxeBcDestroyIp6Children (This, Private);
   }
 
-  if (Private->Ip4Nic == NULL && Private->Ip6Nic == NULL) {
+  if ((Private->Ip4Nic == NULL) && (Private->Ip6Nic == NULL)) {
     gBS->UninstallProtocolInterface (
            NicHandle,
            &gEfiCallerIdGuid,
