@@ -76,7 +76,7 @@ AcpiTimerLibConstructor (
       DEBUG ((
         DEBUG_ERROR,
         "%a: Unknown Host Bridge Device ID: 0x%04x\n",
-        __FUNCTION__,
+        __func__,
         HostBridgeDevId
         ));
       ASSERT (FALSE);
