@@ -87,7 +87,11 @@ OpalSupportGetAvailableActions (
     // Secure erase is performed by generating a new encryption key
     // this is only available if encryption is supported
     //
-    AvalDiskActions->SecureErase = 1;
+    if (SupportedAttributes->MediaEncryption) {
+      AvalDiskActions->SecureErase = 1;
+    } else {
+      AvalDiskActions->SecureErase = 0;
+    }
   } else {
     AvalDiskActions->PsidRevert  = 0;
     AvalDiskActions->SecureErase = 0;
@@ -491,7 +495,7 @@ OpalEndOfDxeEventNotify (
 {
   OPAL_DRIVER_DEVICE  *TmpDev;
 
-  DEBUG ((DEBUG_INFO, "%a() - enter\n", __FUNCTION__));
+  DEBUG ((DEBUG_INFO, "%a() - enter\n", __func__));
 
   mOpalEndOfDxe = TRUE;
 
@@ -529,7 +533,7 @@ OpalEndOfDxeEventNotify (
   //
   SendBlockSidCommand ();
 
-  DEBUG ((DEBUG_INFO, "%a() - exit\n", __FUNCTION__));
+  DEBUG ((DEBUG_INFO, "%a() - exit\n", __func__));
 
   gBS->CloseEvent (Event);
 }
@@ -888,7 +892,7 @@ OpalDriverRequestPassword (
     return;
   }
 
-  DEBUG ((DEBUG_INFO, "%a()\n", __FUNCTION__));
+  DEBUG ((DEBUG_INFO, "%a()\n", __func__));
 
   PopUpString = OpalGetPopUpString (Dev, RequestString);
 
@@ -1074,7 +1078,7 @@ ProcessOpalRequestEnableFeature (
     return;
   }
 
-  DEBUG ((DEBUG_INFO, "%a()\n", __FUNCTION__));
+  DEBUG ((DEBUG_INFO, "%a()\n", __func__));
 
   PopUpString = OpalGetPopUpString (Dev, RequestString);
 
@@ -1223,7 +1227,7 @@ ProcessOpalRequestDisableUser (
     return;
   }
 
-  DEBUG ((DEBUG_INFO, "%a()\n", __FUNCTION__));
+  DEBUG ((DEBUG_INFO, "%a()\n", __func__));
 
   PopUpString = OpalGetPopUpString (Dev, RequestString);
 
@@ -1339,7 +1343,7 @@ ProcessOpalRequestPsidRevert (
     return;
   }
 
-  DEBUG ((DEBUG_INFO, "%a()\n", __FUNCTION__));
+  DEBUG ((DEBUG_INFO, "%a()\n", __func__));
 
   PopUpString = OpalGetPopUpString (Dev, RequestString);
 
@@ -1478,7 +1482,7 @@ ProcessOpalRequestRevert (
     return;
   }
 
-  DEBUG ((DEBUG_INFO, "%a()\n", __FUNCTION__));
+  DEBUG ((DEBUG_INFO, "%a()\n", __func__));
 
   PopUpString = OpalGetPopUpString (Dev, RequestString);
 
@@ -1646,7 +1650,7 @@ ProcessOpalRequestSecureErase (
     return;
   }
 
-  DEBUG ((DEBUG_INFO, "%a()\n", __FUNCTION__));
+  DEBUG ((DEBUG_INFO, "%a()\n", __func__));
 
   PopUpString = OpalGetPopUpString (Dev, RequestString);
 
@@ -1784,7 +1788,7 @@ ProcessOpalRequestSetUserPwd (
     return;
   }
 
-  DEBUG ((DEBUG_INFO, "%a()\n", __FUNCTION__));
+  DEBUG ((DEBUG_INFO, "%a()\n", __func__));
 
   PopUpString = OpalGetPopUpString (Dev, RequestString);
 
@@ -1990,7 +1994,7 @@ ProcessOpalRequestSetAdminPwd (
     return;
   }
 
-  DEBUG ((DEBUG_INFO, "%a()\n", __FUNCTION__));
+  DEBUG ((DEBUG_INFO, "%a()\n", __func__));
 
   PopUpString = OpalGetPopUpString (Dev, RequestString);
 
@@ -2182,7 +2186,7 @@ ProcessOpalRequest (
   UINTN                     DevicePathSize;
   BOOLEAN                   KeepUserData;
 
-  DEBUG ((DEBUG_INFO, "%a() - enter\n", __FUNCTION__));
+  DEBUG ((DEBUG_INFO, "%a() - enter\n", __func__));
 
   if (mOpalRequestVariable == NULL) {
     Status = GetVariable2 (
@@ -2278,7 +2282,7 @@ ProcessOpalRequest (
     TempVariable  = (OPAL_REQUEST_VARIABLE *)((UINTN)TempVariable + TempVariable->Length);
   }
 
-  DEBUG ((DEBUG_INFO, "%a() - exit\n", __FUNCTION__));
+  DEBUG ((DEBUG_INFO, "%a() - exit\n", __func__));
 }
 
 /**
