@@ -214,7 +214,13 @@ ShellCommandRunMemMap (
       if (Status == EFI_BUFFER_TOO_SMALL) {
         Size       += SIZE_1KB;
         Descriptors = AllocateZeroPool (Size);
-        Status      = gBS->GetMemoryMap (&Size, Descriptors, &MapKey, &ItemSize, &Version);
+        if (Descriptors == NULL) {
+          ShellPrintHiiEx (-1, -1, NULL, STRING_TOKEN (STR_GEN_OUT_MEM), gShellDebug1HiiHandle, L"memmap");
+          ShellCommandLineFreeVarList (Package);
+          return SHELL_OUT_OF_RESOURCES;
+        }
+
+        Status = gBS->GetMemoryMap (&Size, Descriptors, &MapKey, &ItemSize, &Version);
       }
 
       if (EFI_ERROR (Status)) {
