@@ -1,7 +1,7 @@
 /** @file
 SMM profile internal header file.
 
-Copyright (c) 2012 - 2018, Intel Corporation. All rights reserved.<BR>
+Copyright (c) 2012 - 2024, Intel Corporation. All rights reserved.<BR>
 Copyright (c) 2020, AMD Incorporated. All rights reserved.<BR>
 SPDX-License-Identifier: BSD-2-Clause-Patent
 
@@ -11,10 +11,7 @@ SPDX-License-Identifier: BSD-2-Clause-Patent
 #define _SMM_PROFILE_INTERNAL_H_
 
 #include <Protocol/SmmReadyToLock.h>
-#include <Library/UefiRuntimeServicesTableLib.h>
-#include <Library/DxeServicesTableLib.h>
 #include <Library/CpuLib.h>
-#include <Library/UefiCpuLib.h>
 #include <IndustryStandard/Acpi.h>
 
 #include "SmmProfileArch.h"
@@ -42,21 +39,13 @@ SPDX-License-Identifier: BSD-2-Clause-Patent
 //
 // CPU generic definition
 //
-#define   CPUID1_EDX_XD_SUPPORT  0x100000
-#define   MSR_EFER               0xc0000080
-#define   MSR_EFER_XD            0x800
+#define MSR_EFER_XD  0x800
 
-#define   CPUID1_EDX_BTS_AVAILABLE  0x200000
+#define CPUID1_EDX_BTS_AVAILABLE  0x200000
 
-#define   DR6_SINGLE_STEP  0x4000
-#define   RFLAG_TF         0x100
+#define DR6_SINGLE_STEP  0x4000
 
-#define MSR_DEBUG_CTL          0x1D9
-#define   MSR_DEBUG_CTL_LBR    0x1
-#define   MSR_DEBUG_CTL_TR     0x40
-#define   MSR_DEBUG_CTL_BTS    0x80
-#define   MSR_DEBUG_CTL_BTINT  0x100
-#define MSR_DS_AREA            0x600
+#define MSR_DS_AREA  0x600
 
 #define HEAP_GUARD_NONSTOP_MODE      \
         ((PcdGet8 (PcdHeapGuardPropertyMask) & (BIT6|BIT3|BIT2)) > BIT6)
@@ -97,12 +86,11 @@ typedef struct {
   UINT64    SmiCmd;
 } SMM_PROFILE_ENTRY;
 
-extern SMM_S3_RESUME_STATE  *mSmmS3ResumeState;
-extern UINTN                gSmiExceptionHandlers[];
-extern BOOLEAN              mXdSupported;
-X86_ASSEMBLY_PATCH_LABEL    gPatchXdSupported;
-X86_ASSEMBLY_PATCH_LABEL    gPatchMsrIa32MiscEnableSupported;
-extern UINTN                *mPFEntryCount;
+extern UINTN              gSmiExceptionHandlers[];
+extern BOOLEAN            mXdSupported;
+X86_ASSEMBLY_PATCH_LABEL  gPatchXdSupported;
+X86_ASSEMBLY_PATCH_LABEL  gPatchMsrIa32MiscEnableSupported;
+extern UINTN              *mPFEntryCount;
 extern UINT64 (*mLastPFEntryValue)[MAX_PF_ENTRY_COUNT];
 extern UINT64                    *(*mLastPFEntryPointer)[MAX_PF_ENTRY_COUNT];
 
@@ -131,24 +119,38 @@ IsAddressSplit (
   );
 
 /**
-  Check if the memory address will be mapped by 4KB-page.
+  Check if the SMM profile page fault address above 4GB is in protected range or not.
 
-  @param  Address  The address of Memory.
-  @param  Nx       The flag indicates if the memory is execute-disable.
+  @param[in]   Address  The address of Memory.
+  @param[out]  Nx       The flag indicates if the memory is execute-disable.
+
+  @retval TRUE     The input address is in protected range.
+  @retval FALSE    The input address is not in protected range.
 
 **/
 BOOLEAN
-IsAddressValid (
-  IN EFI_PHYSICAL_ADDRESS  Address,
-  IN BOOLEAN               *Nx
+IsSmmProfilePFAddressAbove4GValid (
+  IN  EFI_PHYSICAL_ADDRESS  Address,
+  OUT BOOLEAN               *Nx
   );
 
 /**
-  Page Fault handler for SMM use.
+  Allocate free Page for PageFault handler use.
+
+  @return Page address.
+
+**/
+UINT64
+AllocPage (
+  VOID
+  );
+
+/**
+  Create new entry in page table for page fault address in SmmProfilePFHandler.
 
 **/
 VOID
-SmiDefaultPFHandler (
+SmmProfileMapPFAddress (
   VOID
   );
 
